//----------------------------------------------------------------------------
//  Project weinadmin
//
//  Copyright  1996. Alle Rechte vorbehalten.
//
//  SUBSYSTEM:    weinadmin.apx Application
//  FILE:         dialogcustomizer.cpp
//  AUTHOR:       Thomas Haenselmann
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of DialogCustomizer (TDialog).
//
//----------------------------------------------------------------------------
#include <owl/pch.h>
#include <stdio.h>
#include "dialogcustomizer.h"
#include <list/list.h>
#include <pbk/geogruppe/geogruppe.h>
#include <pbk/customizer/customizer.h>
#include <pbk/preis/preisverwaltung.h>

//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE1(DialogCustomizer, TDialog)
//{{DialogCustomizerRSP_TBL_BEGIN}}
  EV_BN_CLICKED(IDC_CU_ADDGEOGRUPPE, CmAddGeogruppe),
  EV_BN_CLICKED(IDC_CU_ADDPREISLISTE, CmAddPreisliste),
  EV_BN_CLICKED(IDOK, CmOk),
//{{DialogCustomizerRSP_TBL_END}}
END_RESPONSE_TABLE;


//{{DialogCustomizer Implementation}}


static DialogCustomizerXfer DialogCustomizerData;

DialogCustomizer::DialogCustomizer(TWindow* parent, ModuleBroker* pModuleBroker, TResId resId, TModule* module)
:
    _p_module_broker(pModuleBroker), TDialog(parent, resId, module)
{
//{{DialogCustomizerXFER_USE}}
  pEuroPreise = new TCheckBox(this, IDC_CU_EUROPREISE, 0);
  pGeogruppe = new TComboBox(this, IDC_CU_GEOGRUPPE, 9);
  pPreisliste = new TComboBox(this, IDC_CU_PREISLISTE, 9);
  pWaehrung = new TEdit(this, IDC_CU_WAEHRUNG, 255);
  pMWSEnthalten = new TCheckBox(this, IDC_CU_MWS_ENTHALTEN, 0);
  pMWS = new TEdit(this, IDC_CU_MWS, 255);
  pVordatierung = new TEdit(this, IDC_CU_VORDATIERUNG, 255);
  pBemerkung = new TEdit(this, IDC_CU_BEMERKUNG, 300);

  SetTransferBuffer(&DialogCustomizerData);
//{{DialogCustomizerXFER_USE_END}}

  // INSERT>> Your constructor code here.

}


DialogCustomizer::~DialogCustomizer()
{
  Destroy(IDCANCEL);

  // INSERT>> Your destructor code here.

}


void DialogCustomizer::SetupWindow()
{
  TDialog::SetupWindow();

  // INSERT>> Your code here.

  GenericList<GeoGruppe::GeoString>& geo_liste = _p_module_broker->GetGeoGruppePtr()->GetList();
  char  					     text[20];

  if(_p_module_broker->GetCustomizerPtr()->GetEuroAktiv()) pEuroPreise->SetCheck(BF_CHECKED);
  else pEuroPreise->SetCheck(BF_UNCHECKED);

  if(_p_module_broker->GetCustomizerPtr()->GetSteuerEnthalten()) pMWSEnthalten->SetCheck(BF_CHECKED);
  else pMWSEnthalten->SetCheck(BF_UNCHECKED);

  pWaehrung->SetText(_p_module_broker->GetCustomizerPtr()->GetWaehrungString());
  sprintf(text, "%0.2f", _p_module_broker->GetCustomizerPtr()->GetMehrwertsteuer());
  pMWS->SetText(text);

  pBemerkung->SetText(_p_module_broker->GetCustomizerPtr()->GetRechnungBemerkung());

  sprintf(text, "%d", _p_module_broker->GetCustomizerPtr()->GetVordatierung());
  pVordatierung->SetText(text);

  pGeogruppe->ClearList();
  pPreisliste->ClearList();

  for(int i = 0; i < geo_liste.GetNoElements(); i++)
    pGeogruppe->AddString(geo_liste[i]._text);

  for(int i = 0; i < _p_module_broker->GetPreisVerwaltungPtr()->GetNoPreisliste(); i++)
    pPreisliste->AddString(_p_module_broker->GetPreisVerwaltungPtr()->GetPreisListeName(i));
}


void DialogCustomizer::CmAddGeogruppe()
{
  // INSERT>> Your code here.

  GenericList<GeoGruppe::GeoString>& geo_liste = _p_module_broker->GetGeoGruppePtr()->GetList();
  char 										 text[256];

  pGeogruppe->GetText(text, 255);

  if((strlen(text) < 2) || (strlen(text) > 8))
  {
    MessageBox("Das Krzel fr die Geogruppe sollte aus 2-8 Buchstaben bestehen.", "Geogruppe hinzufgen", MB_OK);
    return;
  } // if

  for(int i = 0; i < geo_liste.GetNoElements(); i++)
  {
    if(strcmp(geo_liste[i]._text, text) == 0)
    {
      MessageBox("Diese Geogruppe existiert bereits in der Liste. Vergeben Sie bitte nur Namen, die noch nicht vorkommen.", "Geogruppe hinzufgen", MB_OK);
      return;
    } // if
  } // for

	_p_module_broker->GetGeoGruppePtr()->Add(text);
   pGeogruppe->AddString(text);
}


void DialogCustomizer::CmAddPreisliste()
{
  // INSERT>> Your code here.

  char text[256];

  pPreisliste->GetText(text, 255);

  if((strlen(text) < 2) || (strlen(text) > 8))
  {
    MessageBox("Das Krzel fr die Preisliste sollte aus 2-8 Buchstaben bestehen.", "Preisliste hinzufgen", MB_OK);
    return;
  } // if

  for(int i = 0; i < _p_module_broker->GetPreisVerwaltungPtr()->GetNoPreisliste(); i++)
  {
    if(strcmp(_p_module_broker->GetPreisVerwaltungPtr()->GetPreisListeName(i), text) == 0)
    {
      MessageBox("Diese Bezeichnung fr eine Preisliste existiert bereits. Vergeben Sie bitte nur Namen, die noch nicht vorkommen.", "Preisliste hinzufgen", MB_OK);
      return;
    } // if
  } // for

	_p_module_broker->GetPreisVerwaltungPtr()->AddPreisliste(text);
   pPreisliste->AddString(text);
}


void DialogCustomizer::CmOk()
{
	// INSERT>> Your code here.

   char text[301];
   int  tage;

	if(pEuroPreise->GetCheck() == BF_CHECKED) _p_module_broker->GetCustomizerPtr()->SetEuroAktiv(true);
   else _p_module_broker->GetCustomizerPtr()->SetEuroAktiv(false);

	if(pMWSEnthalten->GetCheck() == BF_CHECKED) _p_module_broker->GetCustomizerPtr()->SetSteuerEnthalten(true);
   else _p_module_broker->GetCustomizerPtr()->SetSteuerEnthalten(false);

   pMWS->GetText(text, 31); _p_module_broker->GetCustomizerPtr()->SetMehrwertsteuer(atof(text));
   pWaehrung->GetText(text, 31); _p_module_broker->GetCustomizerPtr()->SetWaehrungString(text);

	pBemerkung->GetText(text, 300); _p_module_broker->GetCustomizerPtr()->SetRechnungBemerkung(text);

   pVordatierung->GetText(text, 31); tage = atoi(text);
	if(tage < 0) tage = 0;
   if(tage > 1000) tage = 1000;
   _p_module_broker->GetCustomizerPtr()->SetVordatierung(tage);

  sprintf(text, "%d", _p_module_broker->GetCustomizerPtr()->GetVordatierung());
  pVordatierung->SetText(text);


   TDialog::CmOk();
}

