//----------------------------------------------------------------------------
//  Project weinadmin
//
//  Copyright  1996. Alle Rechte vorbehalten.
//
//  SUBSYSTEM:    weinadmin.apx Application
//  FILE:         geofilterdlg.cpp
//  AUTHOR:       Thomas Haenselmann
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of GeoFilterDlg (TDialog).
//
//----------------------------------------------------------------------------
#include <owl/pch.h>

#include "geofilterdlg.h"
#include <owl/opensave.h>

#include <list/list.h>
#include <pbk/geogruppe/geogruppe.h>
#include <pbk/kunde/kundenverwaltung.h>


//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE1(GeoFilterDlg, TDialog)
//{{GeoFilterDlgRSP_TBL_BEGIN}}
  EV_BN_CLICKED(IDC_GEOFIL_DATEI, CmAdressDatei),
  EV_BN_CLICKED(IDC_GEOFIL_USEGEO, CmUseGeo),
  EV_BN_CLICKED(IDC_GEOFIL_USEPLZ, CmUsePLZ),
//{{GeoFilterDlgRSP_TBL_END}}
END_RESPONSE_TABLE;


//{{GeoFilterDlg Implementation}}


static GeoFilterDlgXfer GeoFilterDlgData;

GeoFilterDlg::GeoFilterDlg(TWindow* parent, ModuleBroker* pModuleBroker, TResId resId, TModule* module)
:
    TDialog(parent, resId, module), _p_module_broker(pModuleBroker)
{
//{{GeoFilterDlgXFER_USE}}
  pGeoGruppe = new TListBox(this, IDC_GEOFIL_GRUPPE);
  pPLZBis = new TEdit(this, IDC_GEOFIL_PLZ_BIS, 255);
  pPLZVon = new TEdit(this, IDC_GEOFIL_PLZ_VON, 255);
  pUsePLZ = new TCheckBox(this, IDC_GEOFIL_USEPLZ, 0);
  pUseGeo = new TCheckBox(this, IDC_GEOFIL_USEGEO, 0);
  pGeoDatei = new TButton(this, IDC_GEOFIL_DATEI);

  SetTransferBuffer(&GeoFilterDlgData);
//{{GeoFilterDlgXFER_USE_END}}

  // INSERT>> Your constructor code here.

}


GeoFilterDlg::~GeoFilterDlg()
{
  Destroy(IDC_GEOFIL_DATEI);

  // INSERT>> Your destructor code here.

}


void GeoFilterDlg::SetupWindow()
{
  TDialog::SetupWindow();

  // INSERT>> Your code here.

  GenericList<GeoGruppe::GeoString>& geo_liste = _p_module_broker->GetGeoGruppePtr()->GetList();

  for(int i = 0; i < geo_liste.GetNoElements(); i++)
    pGeoGruppe->AddString(geo_liste[i]._text);

  pUseGeo->SetCheck(BF_CHECKED);
  pUsePLZ->SetCheck(BF_UNCHECKED);

  pPLZVon->EnableWindow(false);
  pPLZBis->EnableWindow(false);
}


void GeoFilterDlg::CmAdressDatei()
{
  // INSERT>> Your code here.

  TOpenSaveDialog::TData file_data;
  bool 					    use_geo, use_plz;
  char 						 text[30];
  long 						 plz_min, plz_max, geo_gruppe;

  // Daten aus Feldern holen

  use_geo = use_plz = false;

  if(pUseGeo->GetCheck() == BF_CHECKED) use_geo = true;
  if(pUsePLZ->GetCheck() == BF_CHECKED) use_plz = true;

  if(use_geo)
  {
    geo_gruppe = pGeoGruppe->GetSelIndex();
  } // if

  if(use_plz)
  {
    pPLZVon->GetText(text, 20);
    plz_min = atoi(text);

    pPLZBis->GetText(text, 20);
    plz_max = atoi(text);
  } // if

  TOpenSaveDialog::TData FileData((uint32)OFN_EXPLORER, (char*)"Adressdatei (*.CSV)|*.csv| Alle Dateien (*.*)|*.*|",
  										    (char*)"*.csv", (char*)"", (char*)"csv");

  FileData.Flags = FileData.Flags & (~OFN_EXPLORER);

  if(TFileSaveDialog(NULL, FileData, 0, "Adressdatei erzeugen").DoExecute() == IDOK)
  {
     char fileTitle[MAXPATH];
     TOpenSaveDialog::GetFileTitle(FileData.FileName, fileTitle, MAXPATH);
	  fstream stream(fileTitle, ios::out);

     SetCursor(0, IDC_WAIT);
	  _p_module_broker->GetKundenVerwaltungPtr()->ExportAsCSV(stream, use_geo, use_plz, geo_gruppe, plz_min, plz_max);
     SetCursor(0, IDC_ARROW);
     Destroy();
  } // if
}


void GeoFilterDlg::CmUseGeo()
{
  // INSERT>> Your code here.

  if(pUseGeo->GetCheck() == BF_CHECKED) pGeoGruppe->EnableWindow(true);
  else pGeoGruppe->EnableWindow(false);
}


void GeoFilterDlg::CmUsePLZ()
{
  // INSERT>> Your code here.

  if(pUsePLZ->GetCheck() == BF_CHECKED)
  {
    pPLZVon->EnableWindow(true);
    pPLZBis->EnableWindow(true);
  } // if
  else
  {
    pPLZVon->EnableWindow(false);
    pPLZBis->EnableWindow(false);
  } // else
}

