//----------------------------------------------------------------------------
//  Project weinadmin
//
//  Copyright  1996. Alle Rechte vorbehalten.
//
//  SUBSYSTEM:    weinadmin.apx Application
//  FILE:         kundendialog.cpp
//  AUTHOR:       Thomas Haenselmann
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of KundenDialog (TDialog).
//
//----------------------------------------------------------------------------
#include <owl/pch.h>
#include <stdio.h>

#include "kundendialog.h"
#include "kundenliste.h"
#include "weinliste.h"
#include "rechnungsliste.h"

#include "pbk/kunde/kundenverwaltung.h"
#include "pbk/wein/weinlistenverwaltung.h"
#include "pbk/preis/preisverwaltung.h"
#include "pbk/rechnung/rechnungsverwaltung.h"

//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE1(KundenDialog, TDialog)
//{{KundenDialogRSP_TBL_BEGIN}}
  EV_COMMAND(IDC_STORE, CmStore),
  EV_COMMAND(IDC_CLEAR, CmClear),
  EV_EN_CHANGE(IDC_BEMERKUNG, ENChange),
  EV_EN_CHANGE(IDC_EMAIL, ENChange),
  EV_EN_CHANGE(IDC_FAX, ENChange),
  EV_EN_CHANGE(IDC_NACHNAME, ENChange),
  EV_EN_CHANGE(IDC_ORT, ENChange),
  EV_EN_CHANGE(IDC_PLZ, ENChange),
  EV_EN_CHANGE(IDC_STRASSE, ENChange),
  EV_EN_CHANGE(IDC_TELEFON, ENChange),
  EV_EN_CHANGE(IDC_VORNAME, ENChange),
  EV_CBN_SELCHANGE(IDC_ANREDE, ENChange),
  EV_CBN_SELCHANGE(IDC_PREISLISTE, ENChange),
  EV_CBN_SELCHANGE(IDC_GEOGRUPPE, ENChange),
  EV_WM_TIMER,
  EV_CBN_EDITCHANGE(IDC_ANREDE, ENChange),
  EV_BN_CLICKED(IDC_RETURN, CmPressReturn),
//{{KundenDialogRSP_TBL_END}}
END_RESPONSE_TABLE;


//{{KundenDialog Implementation}}


static KundenDialogXfer KundenDialogData;

KundenDialog::KundenDialog(TWindow* parent, Mode mode, TSDIDecFrame* pParentWindow, ModuleBroker* pModuleBroker, Kunde* pKunde, TResId resId,
                           TModule* module) : TDialog(parent, resId, module), _p_module_broker(pModuleBroker), _mode(mode), _p_kunde(pKunde)
{
//{{KundenDialogXFER_USE}}

  pAnrede = new TComboBox(this, IDC_ANREDE, KS_SIZE_ANREDE);
  pVorname = new TEdit(this, IDC_VORNAME, KS_SIZE_VORNAME);
  pNachname = new TEdit(this, IDC_NACHNAME, KS_SIZE_NACHNAME);
  pStrasse = new TEdit(this, IDC_STRASSE, KS_SIZE_STRASSE);
  pPLZ = new TEdit(this, IDC_PLZ, 6);
  pOrt = new TEdit(this, IDC_ORT, KS_SIZE_ORT);
  pTelefon = new TEdit(this, IDC_TELEFON, KS_SIZE_TELEFON);
  pFax = new TEdit(this, IDC_FAX, KS_SIZE_FAX);
  pEMail = new TEdit(this, IDC_EMAIL, KS_SIZE_EMAIL);
  pBemerkung = new TEdit(this, IDC_BEMERKUNG, KS_SIZE_BEMERKUNG);
  pClear = new TButton(this, IDC_CLEAR);
  pStore = new TButton(this, IDC_STORE);
  pTreffer = new TEdit(this, IDC_TREFFER, 255);
//  pTrefferListe = new TListBox(this, IDC_TREFFERLISTE);
  pPreisListe = new TComboBox(this, IDC_PREISLISTE, 90, 298, 102, 150, WS_CHILD|WS_VISIBLE|WS_GROUP|WS_TABSTOP|WS_VSCROLL|CBS_DROPDOWNLIST, 28);
  pGeoGruppe = new TComboBox(this, IDC_GEOGRUPPE, 90, 325, 102, 150, WS_CHILD|WS_VISIBLE|WS_GROUP|WS_TABSTOP|WS_VSCROLL|CBS_DROPDOWNLIST, 28);

  pBmpEditieren = new TStatic(this, IDC_BMP_EDITIEREN, 255);
  pBmpErfassen = new TStatic(this, IDC_BMP_ERFASSEN, 255);
  pBmpRechnung = new TStatic(this, IDC_BMP_RECHNUNG, 255);
  pBmpSuchen = new TStatic(this, IDC_BMP_SUCHEN, 255);

  SetTransferBuffer(&KundenDialogData);
//{{KundenDialogXFER_USE_END}}

  // INSERT>> Your constructor code here.

  _set_data_mode = true; // Beim programmgesteuerten Setzen von Feldern soll kein Daten-Change Alarm ausgelst werden
  _dirty = false;

  _p_parent_window = pParentWindow;

  switch(_mode)
  {
    case ADD:
		pAnrede->ClearList();
		pAnrede->AddString("Herrn");
		pAnrede->AddString("Frau");
		pAnrede->AddString("Firma");
		pAnrede->AddString("Eheleute");
      break;

    case RECHNUNG:
    case SEARCH:
	   _set_data_mode = true;
		pAnrede->ClearList();
		pAnrede->AddString("Herrn");
		pAnrede->AddString("Frau");
		pAnrede->AddString("Firma");
		pAnrede->AddString("Eheleute");
      break;

    case EDIT:
		pAnrede->ClearList();
		pAnrede->AddString("Herrn");
		pAnrede->AddString("Frau");
		pAnrede->AddString("Firma");
		pAnrede->AddString("Eheleute");
      _current_kunde = *_p_kunde;
      break;
  } // switch
}


KundenDialog::~KundenDialog()
{
  Destroy(IDCANCEL);

  // INSERT>> Your destructor code here.

}


void KundenDialog::SetupWindow()
{
  TDialog::SetupWindow();

  // INSERT>> Your code here.

  Kunde LeerKunde;

  pAnrede->ClearList();
  pAnrede->AddString("Herrn");
  pAnrede->AddString("Frau");
  pAnrede->AddString("Firma");
  pAnrede->AddString("Eheleute");

  pBmpEditieren->Show(SW_HIDE);
  pBmpErfassen->Show(SW_HIDE);
  pBmpRechnung->Show(SW_HIDE);
  pBmpSuchen->Show(SW_HIDE);

  for(int i = 0; i < _p_module_broker->GetPreisVerwaltungPtr()->GetNoPreisliste(); i++)
    pPreisListe->InsertString(_p_module_broker->GetPreisVerwaltungPtr()->GetPreisListeName(i), i);

  pPreisListe->SetSelIndex(0);

	for(int i = 0;i < _p_module_broker->GetGeoGruppePtr()->GetList().GetNoElements(); i++)
	   pGeoGruppe->InsertString((_p_module_broker->GetGeoGruppePtr()->GetList())[i]._text, i);

  pGeoGruppe->SetSelIndex(0);

  pPreisListe->Create();
  pClear->SetCaption("Formular lschen");

  switch(_mode)
  {
    case ADD:
		pStore->SetCaption("Speichern");
	   pBmpErfassen->Show(SW_SHOW);
      SetKunde(LeerKunde);
      break;

    case RECHNUNG:
		pStore->SetCaption("Trefferliste");
	   pBemerkung->EnableWindow(false);
      pStore->EnableWindow(false);
	   _set_data_mode = false;
      pBmpRechnung->Show(SW_SHOW);
		SetTimer(1, 200);
	   _internal_search_delay = -1;
      SetKunde(LeerKunde);
	   pNachname->SetFocus();
      break;

    case SEARCH:
		pStore->SetCaption("Trefferliste");
	   pBemerkung->EnableWindow(false);
      pStore->EnableWindow(false);
	   _set_data_mode = false;
      pBmpSuchen->Show(SW_SHOW);
//      pTrefferListe->Show(SW_SHOW);
		SetTimer(1, 200);
	   _internal_search_delay = -1;
      SetKunde(LeerKunde);
	   pNachname->SetFocus();
      break;

    case EDIT:
		pStore->SetCaption("Speichern");
      pBmpEditieren->Show(SW_SHOW);
      SetKunde(*_p_kunde);
		pClear->EnableWindow(false);
      break;
  } // switch

  _set_data_mode = false; // ab jetzt gibt der Benutzer wieder die Daten ein
}


void KundenDialog::FillKundenListe(GenericList<Kunde>& KundenListe)
{
	Kunde	tmpKunde;

	_p_module_broker->GetKundenVerwaltungPtr()->Search(_current_kunde);

   while(_p_module_broker->GetKundenVerwaltungPtr()->GetNextMatch(tmpKunde))
   	KundenListe.Add(tmpKunde);
}


void KundenDialog::ENChange()
{
  // INSERT>> Your code here.

  char  text[260];
  Kunde found_kunde;
  long  PLZ;

  if(!_set_data_mode)
  {
     pAnrede->GetText(text, 258); _current_kunde.SetAnrede(text);
     pVorname->GetText(text, 258); _current_kunde.SetVorname(text);
     pNachname->GetText(text, 258); _current_kunde.SetNachname(text);
     pStrasse->GetText(text, 258); _current_kunde.SetStrasse(text);
     pPLZ->GetText(text, 258); _current_kunde.SetPLZ((PLZ = atoi(text)));
     pOrt->GetText(text, 258);

     // Evtl. kann der Ort anhand der PLZ ermittelt werden

     if((_mode == ADD) && (PLZ > 9999) && (strcmp(text, "") == 0))
     {
		 _set_data_mode = true;
       _p_module_broker->GetKundenVerwaltungPtr()->FindOrt(PLZ, text);
       pOrt->SetText(text);
       _set_data_mode = false;
     } // if

     _current_kunde.SetOrt(text);
     pTelefon->GetText(text, 258); _current_kunde.SetTelefon(text);
     pFax->GetText(text, 258); _current_kunde.SetFax(text);
     pEMail->GetText(text, 258); _current_kunde.SetEMail(text);
     pBemerkung->GetText(text, 258); _current_kunde.SetBemerkung(text);
     _current_kunde.SetPreisliste(pPreisListe->GetSelIndex());
     _current_kunde.SetGeoGruppe(pGeoGruppe->GetSelIndex());

	  _internal_search_delay = 0;
	  _dirty = true; // die Eingabe wurde gendert
  } // if
}


void KundenDialog::SetKunde(const Kunde& kunde)
{
	char text[100];

  _set_data_mode = true;
  _current_kunde = kunde;

  pAnrede->SetText(_current_kunde.GetAnrede());
  pVorname->SetText(_current_kunde.GetVorname());
  pNachname->SetText(_current_kunde.GetNachname());
  pStrasse->SetText(_current_kunde.GetStrasse());

  sprintf(text, "%d", _current_kunde.GetPLZ());
  pPLZ->SetText(text);

  pOrt->SetText(_current_kunde.GetOrt());
  pTelefon->SetText(_current_kunde.GetTelefon());
  pFax->SetText(_current_kunde.GetFax());
  pEMail->SetText(_current_kunde.GetEMail());
  pBemerkung->SetText(_current_kunde.GetBemerkung());

  pPreisListe->SetSelIndex(_current_kunde.GetPreisliste());
  pGeoGruppe->SetSelIndex(_current_kunde.GetGeoGruppe());

  _set_data_mode = false;
}

void KundenDialog::FillRechnungsListe(GenericList<Rechnung>& RechnungsListe, bool _nur_offene)
{
   _p_module_broker->GetRechnungsVerwaltungPtr()->GetListeByKunde(_current_kunde.GetNr(), RechnungsListe, _nur_offene);
} // KundenDialog::FillRechnungsListe

void KundenDialog::CmStore()
{
  unsigned long      anzahl_kunden;
  GenericList<Kunde> match_kunden_liste;

  if(strcmp(_current_kunde.GetBemerkung(), "import pascal kunden") == 0)
  {
    if(MessageBox("Soll der alte Kundenstamm aus der Pascal Version importiert werden? VORSICHT: Dabei gehen alle bisher eingegebenen Kunden verloren!", "KundenDialog::CmStore()", MB_YESNO) == IDYES)
    {
		SetCursor(GetModule(), IDC_WAIT);
      _p_module_broker->GetKundenVerwaltungPtr()->ReadPascalData();
		SetCursor(GetModule(), IDC_ARROW);
      MessageBox("Die alte Kundendatei wurde importiert", "KundenDialog::CmStore()", MB_OK);
    } // if
    return;
  } // if

  if(strcmp(_current_kunde.GetBemerkung(), "import csv") == 0)
  {
    if(MessageBox("Soll die Kundendatei kunden.csv importiert werden? VORSICHT: Dabei gehen alle bisher eingegebenen Kunden verloren!", "KundenDialog::CmStore()", MB_YESNO) == IDYES)
    {
		SetCursor(GetModule(), IDC_WAIT);
      _p_module_broker->GetKundenVerwaltungPtr()->ReadCSVData();
		SetCursor(GetModule(), IDC_ARROW);
      MessageBox("Die CSV Kundendatei wurde importiert", "KundenDialog::CmStore()", MB_OK);
    } // if
    return;
  } // if

  switch(_mode)
  {
    case ADD:
      ENChange();
      if(_current_kunde.IsValid())
      {
		  _dirty = false; // es gibt noch keine Eingabe zu speichern
        _p_module_broker->GetKundenVerwaltungPtr()->Add(_current_kunde);
        CmClear();
      } // if
      else
        MessageBox("Es mu mindestens der Nachname angegeben werden", "Kunden speichern", MB_OK);
      break;

    case SEARCH:
      if((anzahl_kunden = _p_module_broker->GetKundenVerwaltungPtr()->Search(_current_kunde)) > 1)
      {
        KundenListe MeineKundenListe(this, _p_parent_window, this, _p_module_broker, false);
        MeineKundenListe.Execute();
      } // if
      else
        if(anzahl_kunden == 1)
        {
          RechnungsListe MyRechnungsListe(this, this, _p_module_broker, &_current_kunde);
          MyRechnungsListe.Execute();
          Destroy();
        } // if
      break;

    case RECHNUNG:
      if(_p_module_broker->GetKundenVerwaltungPtr()->Search(_current_kunde) == 1)
      {
        WeinListe MeineBestellliste(this, _p_module_broker, WeinListe::BESTELLUNG, &_current_kunde);
        MeineBestellliste.Execute();
        Destroy();
      } // if
      else
      {
        KundenListe MeineKundenListe(this, _p_parent_window, this, _p_module_broker, true);
        MeineKundenListe.Execute();
      } // else
      break;

    case EDIT:
		if(MessageBox("Vorsicht: Sie versuchen die Daten eines Kunden durch nderungen zu berschreiben. Mchten Sie den Vorgang fortsetzen?", "Vorgang Kundendaten verndern", MB_YESNO) == IDYES)
      {
		   _dirty = false; // es gibt noch keine Eingabe zu speichern
			_p_module_broker->GetKundenVerwaltungPtr()->Update(_current_kunde);
      } // if
      else
			MessageBox("Die Kundenkartei bleibt unverndert", "Vorgang Kundendaten verndern", MB_OK);
      break;
  } // switch
}


void KundenDialog::CmClear()
{
  _set_data_mode = true;

  pAnrede->EnableWindow(true);
  pVorname->EnableWindow(true);
  pNachname->EnableWindow(true);
  pStrasse->EnableWindow(true);
  pPLZ->EnableWindow(true);
  pOrt->EnableWindow(true);
  pTelefon->EnableWindow(true);
  pFax->EnableWindow(true);
  pEMail->EnableWindow(true);
  pPreisListe->EnableWindow(true);
  pGeoGruppe->EnableWindow(true);

  pAnrede->Clear();
  pVorname->Clear();
  pNachname->Clear();
  pStrasse->Clear();
  pPLZ->Clear();
  pOrt->Clear();
  pTelefon->Clear();
  pFax->Clear();
  pEMail->Clear();
  pBemerkung->Clear();
  pNachname->SetFocus();

  _set_data_mode = false;

  ENChange();
  _dirty = false; // es gibt noch keine Eingabe zu speichern
}


void KundenDialog::CBNEditchange()
{
  // INSERT>> Your code here.

  ENChange();
}

void KundenDialog::CBNSelendok()
{
  // INSERT>> Your code here.

  ENChange();
}

void KundenDialog::EvTimer(uint timerId)
{
    TDialog::EvTimer(timerId);

    GenericList<Kunde> treffer_liste;

  // INSERT>> Your code here.

   char 		    text[260];
   unsigned long anzahl_treffer;
   Kunde		    found_kunde;

	if(_internal_search_delay < 0) return;

   _internal_search_delay++;

   anzahl_treffer = -1;
   if((_mode != ADD) && (_internal_search_delay > 1))
   {
   	anzahl_treffer = _p_module_broker->GetKundenVerwaltungPtr()->Search(_current_kunde, &treffer_liste);
    	sprintf(text, "%d", anzahl_treffer);
    	pTreffer->SetText(text);

      _internal_search_delay = -1;
   } // if

   switch(_mode)
   {
    case RECHNUNG:
    case SEARCH:
       if(anzahl_treffer == 0)
       {
         pStore->SetCaption("&Trefferliste");
         pStore->EnableWindow(false);
       } // if
       else
          if(anzahl_treffer == 1)
          {
            _p_module_broker->GetKundenVerwaltungPtr()->GetNextMatch(found_kunde);
            SetKunde(found_kunde);

            pAnrede->EnableWindow(false);
            pVorname->EnableWindow(false);
            pNachname->EnableWindow(false);
            pStrasse->EnableWindow(false);
            pPLZ->EnableWindow(false);
            pOrt->EnableWindow(false);
            pTelefon->EnableWindow(false);
            pFax->EnableWindow(false);
            pEMail->EnableWindow(false);
            pPreisListe->EnableWindow(false);
            pGeoGruppe->EnableWindow(false);

            pStore->SetCaption("&Rechnungen");
            pStore->EnableWindow(true);
          } // if
          else
				 if((anzahl_treffer > 1) && ((anzahl_treffer < 500)))
             {
               pStore->SetCaption("&Trefferliste");
               pStore->EnableWindow(true);
             } // if
             else
             {
               pStore->SetCaption("&Trefferliste");
               pStore->EnableWindow(false);
             } // else
       break;
   } // switch

   if(_mode == RECHNUNG)
   {
     if(anzahl_treffer == 1)
     {
        pStore->SetCaption("Neue Rechnung");
        pStore->EnableWindow(true);
     } // if
     else
     {
        pStore->SetCaption("Trefferliste");
        pStore->EnableWindow(true);
     } // else
   } // if
}

bool KundenDialog::CanClose()
{
  bool result;

  result = TDialog::CanClose();

  // INSERT>> Your code here.

  if((_dirty) && ((_mode == ADD) || (_mode == EDIT)))
  {
    if(MessageBox("Sie haben die eingegebenen Daten noch nicht gespeichert. Mchten Sie fortfahren?", "Dialog beenden", MB_YESNO) == IDNO)
    	result = false;
  } // if

  return result;
}


void KundenDialog::CmPressReturn()
{
	// INSERT>> Your code here.

   SetControlFocus(Previous);
}

