//----------------------------------------------------------------------------
//  Project weinadmin
//
//  Copyright  1996. Alle Rechte vorbehalten.
//
//  SUBSYSTEM:    weinadmin.apx Application
//  FILE:         kundenliste.cpp
//  AUTHOR:       Thomas Haenselmann
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of KundenListe (TDialog).
//
//----------------------------------------------------------------------------
#include <owl/pch.h>
#include <stdio.h>

#include "kundenliste.h"
#include "kundendialog.h"
#include "weinliste.h"
#include <pbk/geogruppe/geogruppe.h>

//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE1(KundenListe, TDialog)
//{{KundenListeRSP_TBL_BEGIN}}
  EV_WM_LBUTTONDBLCLK,
  EV_LVN_GETDISPINFO(IDC_KUNDENTABELLE, LVNGetdispinfo),
  EV_LVN_ITEMCHANGING(IDC_KUNDENTABELLE, CmClickTabelle),
  EV_BN_CLICKED(IDC_KL_EDIT, CmEditKunde),
  EV_BN_CLICKED(IDC_KL_DELETE, CmDeleteKunde),
  EV_BN_CLICKED(IDC_KL_RECHNUNG, CmNewRechnung),
//{{KundenListeRSP_TBL_END}}
END_RESPONSE_TABLE;


//{{KundenListe Implementation}}


static KundenListeXfer KundenListeData;

KundenListe::KundenListe(TWindow* parent, TSDIDecFrame* pParentWindow, KundenServer* pKundenServer, ModuleBroker* pModuleBroker, bool fuer_rechnung, TResId resId, TModule* module)
:
    TDialog(parent, resId, module), _p_parent_window(pParentWindow), _p_kunden_server(pKundenServer), _p_module_broker(pModuleBroker), _fuer_rechnung(fuer_rechnung)
{
//{{KundenListeXFER_USE}}
  pKundenTabelle = new TListWindow(this, IDC_KUNDENTABELLE);
  pAdresse = new TEdit(this, IDC_KL_ADRESSE, 255);
  pBemerkung = new TEdit(this, IDC_KL_BEMERKUNG, 255);
  pDelete = new TButton(this, IDC_KL_DELETE);
  pEdit = new TButton(this, IDC_KL_EDIT);
  pEMail = new TEdit(this, IDC_KL_EMAIL, 255);
  pName = new TEdit(this, IDC_KL_NAME, 255);
  pOrt = new TEdit(this, IDC_KL_PLZ_ORT, 255);
  pListen = new TEdit(this, IDC_KL_PREIS_GEO, 255);
  pTel = new TEdit(this, IDC_KL_TEL_FAX, 255);
  pRechnung = new TButton(this, IDC_KL_RECHNUNG);

  SetTransferBuffer(&KundenListeData);
//{{KundenListeXFER_USE_END}}

  // INSERT>> Your constructor code here.

  // Nun lassen wir uns die Kundenliste vom KundenServer fllen

  _p_kunden_liste = new GenericList<Kunde>;
  _p_kunden_server->FillKundenListe(*_p_kunden_liste);
}


KundenListe::~KundenListe()
{
  Destroy(IDCANCEL);

  // INSERT>> Your destructor code here.

	if(_p_kunden_liste) delete _p_kunden_liste;
}


void KundenListe::SetupWindow()
{
  TDialog::SetupWindow();

  // INSERT>> Your code here.

  if(_fuer_rechnung) pRechnung->Show(SW_SHOW);

  pKundenTabelle->InsertColumn(0, TListWindColumn("Name", 160));
  pKundenTabelle->InsertColumn(1, TListWindColumn("Adresse", 170));
  pKundenTabelle->InsertColumn(2, TListWindColumn("PLZ", 50));
  pKundenTabelle->InsertColumn(3, TListWindColumn("Ort", 130));
  pKundenTabelle->InsertColumn(4, TListWindColumn("Telefon", 80));
  pKundenTabelle->InsertColumn(5, TListWindColumn("Fax", 80));
  pKundenTabelle->InsertColumn(6, TListWindColumn("e-mail", 180));

  FillList();
}


void KundenListe::FillList()
{
  char text[400];

  SetCursor(NULL, IDC_WAIT);
  _p_kunden_liste->Empty();
  _p_kunden_server->FillKundenListe(*_p_kunden_liste);
  pKundenTabelle->DeleteAllItems();

  for(int i = 0; i < _p_kunden_liste->GetNoElements(); i++)
  {
    Kunde& kunde = (*_p_kunden_liste)[i];

    sprintf(text, "%s %s %s", kunde.GetAnrede(), kunde.GetVorname(), kunde.GetNachname());

    TListWindItem item(text, 0);
    pKundenTabelle->AddItem(item);
  } // for

  SetCursor(NULL, IDC_ARROW);
  ClearDisplay();
} // KundenListe::FillList


void KundenListe::ClearDisplay()
{
   pAdresse->SetText("");
   pBemerkung->SetText("");
   pEMail->SetText("");
   pName->SetText("");
   pOrt->SetText("");
   pListen->SetText("");
   pTel->SetText("");

   pDelete->EnableWindow(false);
   pEdit->EnableWindow(false);
   pRechnung->EnableWindow(false);
} // KundenListe::ClearDisplay

void KundenListe::EvLButtonDblClk(uint /* modKeys */, TPoint& /* point */)
{
}


void KundenListe::LVNGetdispinfo(TLwDispInfoNotify& lwdin)
{
  // INSERT>> Your code here.

  char 		     text[260];
  TListWindItem& dispInfoItem = *(TListWindItem*)&lwdin.item;
  Kunde& kunde = (*_p_kunden_liste)[dispInfoItem.GetIndex()];

   if((dispInfoItem.GetIndex() % 2) == 0)
     pKundenTabelle->SetTextBkColor(TColor(192, 255, 192));
   else
     pKundenTabelle->SetTextBkColor(TColor(255, 255, 255));

  switch(dispInfoItem.GetSubItem())
  {
    case 1:
      sprintf(text, "%s", kunde.GetStrasse());
      break;

    case 2:
      sprintf(text, "%d", kunde.GetPLZ());
      break;

    case 3:
      sprintf(text, "%s", kunde.GetOrt());
      break;

    case 4:
      sprintf(text, "%s", kunde.GetTelefon());
      break;

    case 5:
      sprintf(text, "%s", kunde.GetFax());
      break;

    case 6:
      sprintf(text, "%s", kunde.GetEMail());
      break;
  } // switch

  dispInfoItem.SetText(text);
}

bool KundenListe::CmClickTabelle(TLwNotify& lwn)
{
  // INSERT>> Your code here.

   char text[300], geo_text[50], preis_text[50];

   _current_kunde = (*_p_kunden_liste)[lwn.iItem];

   sprintf(text, "%s %s %s", _current_kunde.GetAnrede(), _current_kunde.GetVorname(), _current_kunde.GetNachname());
   pName->SetText(text);

   pAdresse->SetText(_current_kunde.GetStrasse());

   sprintf(text, "%d %s", _current_kunde.GetPLZ(), _current_kunde.GetOrt());
   pOrt->SetText(text);

   pBemerkung->SetText(_current_kunde.GetBemerkung());

   pEMail->SetText(_current_kunde.GetEMail());

   sprintf(text, "%s %s", _current_kunde.GetTelefon(), _current_kunde.GetFax());
   pTel->SetText(text);

	if(_current_kunde.GetGeoGruppe() < _p_module_broker->GetGeoGruppePtr()->GetList().GetNoElements())
	   sprintf(geo_text, "%s", _p_module_broker->GetGeoGruppePtr()->GetList()[_current_kunde.GetGeoGruppe()]._text);
   else sprintf(geo_text, "N/A");

	if(_p_module_broker->GetPreisVerwaltungPtr()->GetPreisListeName(_current_kunde.GetPreisliste()) != NULL)
	   sprintf(preis_text, "%s", _p_module_broker->GetPreisVerwaltungPtr()->GetPreisListeName(_current_kunde.GetPreisliste()));
   else sprintf(preis_text, "N/A");

   sprintf(text, "KundenGruppe: %s Preisliste: %s", geo_text, preis_text);

	pListen->SetText(text);

   pDelete->EnableWindow(true);
   pEdit->EnableWindow(true);
   pRechnung->EnableWindow(true);

   return false;
}


void KundenListe::CmEditKunde()
{
  // INSERT>> Your code here.

	KundenDialog MeinEditDialog(this, KundenDialog::EDIT, _p_parent_window, _p_module_broker, &_current_kunde);
   MeinEditDialog.Execute();
   FillList();
}


void KundenListe::CmDeleteKunde()
{
  // INSERT>> Your code here.

   if(MessageBox("Vorsicht: Sie sind im Begriff einen Kunden aus dem Datenbestand zu lschen. Mchten Sie mit dem Vorgang fortfahren?", "Vorgang Kunde lschen", MB_YESNO) == IDYES)
   {
   	_p_module_broker->GetKundenVerwaltungPtr()->Remove(_current_kunde.GetNr());
  	   FillList();
   } // if
   else
   	MessageBox("Der Kunde verbleibt im Datenbestand", "Vorgang Kunde lschen", MB_OK);
}

void KundenListe::CmNewRechnung()
{
	// INSERT>> Your code here.

	WeinListe MeineBestellliste(this, _p_module_broker, WeinListe::BESTELLUNG, &_current_kunde);
   MeineBestellliste.Execute();
   Destroy();
}

