#include <mck/rechnung/MCKrechnung.h>
#include <pbk/wein/weinlistenverwaltung.h>
#include <pbk/customizer/customizer.h>
#include <stdio.h>
#include <string.h>

DEFINE_RESPONSE_TABLE1(PreViewWindow, TWindow)
  EV_WM_SIZE,
END_RESPONSE_TABLE;

PrintRechnung::PreViewWindow::PreViewWindow(TWindow* pParentWin, PrintRechnung* pRechnung) : TFrameWindow(pParentWin, "Druck-Vorschau"), _p_print_rechnung(pRechnung)
{
	_size_set = false;
} // PrintRechnung::PreViewWindow::PreViewWindow

void PrintRechnung::PreViewWindow::Paint(TDC& dc, bool erase_background, TRect& rect)
{
   _p_print_rechnung->RenderPage(dc, 1, rect);
   _size_set = false;
} // PrintRechnung::PreViewWindow::Paint

void PrintRechnung::PreViewWindow::EvSize(uint sizeType, TSize& size)
{
	if(!_size_set)
   {
		_size_set = true;
		TRect rect = GetWindowRect();
      Invalidate();
		MoveWindow(rect.left, rect.top, rect.left+(size.cy*21)/29.6, rect.top+size.cy, true);
   } // if
} // PrintRechnung::PreViewWindow::EvSize


PrintRechnung::RechnungRenderer::RechnungRenderer(const char* pTitle, PrintRechnung* pPrintRechnung)
																  : TPrintout(pTitle), _p_print_rechnung(pPrintRechnung)
{
} // PrintRechnung::RechnungRenderer::RechnungRenderer

void PrintRechnung::RechnungRenderer::GetDialogInfo(int& minPage, int& maxPage, int& selFromPage, int& selToPage)
{
  minPage = 0;
  maxPage = 0;
  selFromPage = selToPage = 0;
} // PrintRechnung::RechnungRenderer::GetDialogInfo

void PrintRechnung::RechnungRenderer::PrintPage(int page, TRect& rect, unsigned flags)
{
	_p_print_rechnung->RenderPage(*DC, page, rect);
} // PrintRechnung::RechnungRenderer::PrintPage

PrintRechnung::PrintRechnung(Mode mode, bool pre_view, ModuleBroker* pModuleBroker, Kunde* pKunde, Rechnung* pRechnung)
									  : _mode(mode), _p_module_broker(pModuleBroker), _p_kunde(pKunde), _p_rechnung(pRechnung), _pre_view(pre_view)
{
	int no_posten, max_posten;

   _no_pages = 1;
	no_posten = _p_rechnung->GetNoPosten();
	pBlatt = NULL;

   switch(_mode)
   {
   	case RECHNUNG:
			max_posten = NO_POSTEN_RECHNUNG;
      	break;

   	case RECHNUNG_INTERN:
			max_posten = NO_POSTEN_RECHNUNG_INTERN;
      	break;

      case PACKLISTE:
			max_posten = NO_POSTEN_PACKLISTE;
      	break;
   } // switch

   while(no_posten > max_posten)
   {
      _no_pages++;
      no_posten -= max_posten;
   } // while
} // PrintRechnung::PrintRechnung

void PrintRechnung::RenderPage(TWindow* pParentWindow)
{
	TPrinter print;
   char     text[500];
	DEVMODE  printer_data;

	if(_pre_view)
   {
   	PreViewWindow* pWin = new PreViewWindow(pParentWindow, this);
      pWin->Create();
      pWin->Show(SW_SHOW);
   }
   else
   {
      printer_data = *(print.GetData()->GetDevMode());

      switch(_mode)
      {
         case RECHNUNG:
            sprintf(text, "(WAdm) Re: %s %s", _p_kunde->GetNachname(), _p_kunde->GetVorname());
				printer_data.dmDefaultSource = DMBIN_UPPER;
            break;

         case RECHNUNG_INTERN:
            sprintf(text, "(WAdm) ReInt: %s %s", _p_kunde->GetNachname(), _p_kunde->GetVorname());
				printer_data.dmDefaultSource = DMBIN_LOWER;
            break;

         case PACKLISTE:
            sprintf(text, "(WAdm) PckLst: %s %s", _p_kunde->GetNachname(), _p_kunde->GetVorname());
				printer_data.dmDefaultSource = DMBIN_LOWER;
            break;
      } // switch

      // Einstellen, aus welchem Schacht das Papier entnommen werden soll

      print.GetData()->SetDevMode(&printer_data);
		print.GetData()->DoPageSetup = false;
      print.SetUserAbort(false);
/*
typedef struct tagDEVMODE
{
    char  dmDeviceName[CCHDEVICENAME];
    UINT  dmSpecVersion;
    UINT  dmDriverVersion;
    UINT  dmSize;
    UINT  dmDriverExtra;
    DWORD dmFields;
    int   dmOrientation;
    int   dmPaperSize;
    int   dmPaperLength;
    int   dmPaperWidth;
    int   dmScale;
    int   dmCopies;
    int   dmDefaultSource;
    int   dmPrintQuality;
    int   dmColor;
    int   dmDuplex;
    int   dmYResolution;
    int   dmTTOption;
} DEVMODE;

dmDefaultSource	Standardschacht fr den Papiereinzug. Dieses Element kann einen der folgenden Werte haben:

DMBIN_AUTO	DMBIN_LOWER 
	DMBIN_CASSETTE	DMBIN_MANUAL
	DMBIN_ENVELOPE	DMBIN_MIDDLE 
	DMBIN_ENVMANUAL	DMBIN_ONLYONE
	DMBIN_FIRST	DMBIN_SMALLFMT 
	DMBIN_LARGECAPACITY	DMBIN_TRACTOR 
	DMBIN_LARGEFMT	DMBIN_UPPER 
	DMBIN_LAST

*/

      pBlatt = new RechnungRenderer(text, this);
      print.Print(pParentWindow, *pBlatt, true);
   } // else
} // PrintRechnung::RenderPage

void PrintRechnung::RenderPage(TDC& dc, int page_no, TRect& rect)
{

	PostenVerwaltung::Posten posten;
	Wein*							 pWein;
   int							 width, height, spacing, line, left_side;
   int						    posten_counter, max_posten, i;
   char							 text[300], betrag[30];
   long							 index;
   float							 gesamt_betrag, end_betrag, steuer;

   width = rect.right - rect.left;
	height = rect.bottom - rect.top;

	line = height/40;
   spacing = height/60;

	_p_rechnung->ResetPostenCursor(index);
   switch(_mode)
   {
   	case RECHNUNG:
         max_posten = NO_POSTEN_RECHNUNG;
		   spacing = height/65;
			line = height/5;

			if(page_no == 1)
         {
            // Drucken der Adresse

				left_side = width/9;

            dc.SelectObject(TFont("Courier New", spacing, 0, 0, 0, FW_NORMAL, DEFAULT_PITCH|FF_DONTCARE, false, false));
            sprintf(text, "%s %s %s", _p_kunde->GetAnrede(), _p_kunde->GetVorname(), _p_kunde->GetNachname());
            dc.TextOut(left_side, line, text); line += spacing;
            sprintf(text, "%s", _p_kunde->GetStrasse());
            dc.TextOut(left_side, line, text); line += spacing*2;
            dc.SelectObject(TFont("Courier New", spacing*1.2, 0, 0, 0, FW_BOLD, DEFAULT_PITCH|FF_DONTCARE, false, false));
				if(_p_kunde->GetPLZ() > 0) sprintf(text, "%d %s", _p_kunde->GetPLZ(), _p_kunde->GetOrt());
            else sprintf(text, "%s", _p_kunde->GetOrt());
            dc.TextOut(left_side, line, text); line += spacing*3;
         } // if
         else line += spacing*5;

			left_side = width/25;
         dc.SelectObject(TFont("Courier New", spacing*1.5, 0, 0, 0, FW_BOLD, DEFAULT_PITCH|FF_DONTCARE, false, false));

         sprintf(text, "Rechnung Nr.: %d", _p_rechnung->GetNrExtern());

         dc.TextOut(left_side, line, text);
         line += spacing*1.5;

         dc.SelectObject(TFont("Courier New", spacing*0.7, 0, 0, 0, FW_BOLD, DEFAULT_PITCH|FF_DONTCARE, false, false));
         dc.TextOut(left_side, line, "(bei Zahlung bitte angeben)"); line += spacing;

         // Datum drucken, falls gltig

         dc.SelectObject(TFont("Courier New", spacing, 0, 0, 0, FW_BOLD, DEFAULT_PITCH|FF_DONTCARE, false, false));
			if((_p_rechnung->GetDatum().Year() > 1975) && (_p_rechnung->GetDatum().Year() < 10000))
			{
				sprintf(text, "%d.%d.%d", _p_rechnung->GetDatum().DayOfMonth(), _p_rechnung->GetDatum().Month(), _p_rechnung->GetDatum().Year());
	         dc.TextOut(width*0.685, height*0.284, text);
         } // if

         // Bemerkung drucken

			if(strcmp(_p_rechnung->GetBemerkung(), "") != 0)
         {
	      	dc.SelectObject(TFont("Courier New", spacing*0.9, 0, 0, 0, FW_BOLD, DEFAULT_PITCH|FF_DONTCARE, false, false));
				sprintf(text, "%s", _p_rechnung->GetBemerkung());
				dc.TextOut(left_side, line, text); line += spacing*0.7;
         } // if

			if((_p_rechnung->GetKisten() > 0) || (_p_rechnung->GetKartons() > 0))
         {
	      	dc.SelectObject(TFont("Courier New", spacing, 0, 0, 0, FW_BOLD, DEFAULT_PITCH|FF_DONTCARE, false, false));
				sprintf(text, "Steigen: %d / Kartons: %d", _p_rechnung->GetKisten(), _p_rechnung->GetKartons());
				dc.TextOut(left_side, line, text); line += spacing;
         } // if

      	dc.SelectObject(TFont("Courier New", spacing, 0, 0, 0, FW_BOLD, DEFAULT_PITCH|FF_DONTCARE, false, true));
			dc.TextOut(left_side, line, "                                                                                 "); line += spacing;
			dc.TextOut(left_side, line, "Zahl  Fllmenge   Weinname & Sorte                       Einzelpreis  Gesamtpreis"); line += spacing*2;
      	dc.SelectObject(TFont("Courier New", spacing, 0, 0, 0, FW_BOLD, DEFAULT_PITCH|FF_DONTCARE, false, false));

		   for(i = 0; i < (page_no-1)*max_posten; i++)
         	_p_rechnung->GetNextPosten(posten, index);
      	break;

   	case RECHNUNG_INTERN:
         max_posten = NO_POSTEN_RECHNUNG_INTERN;
		   spacing = height/70;
			line = height/20;

			if(page_no == 1)
         {
            // Drucken der Adresse

				left_side = width/16;

            dc.SelectObject(TFont("Courier New", spacing, 0, 0, 0, FW_NORMAL, DEFAULT_PITCH|FF_DONTCARE, false, false));
            sprintf(text, "%s %s %s", _p_kunde->GetAnrede(), _p_kunde->GetVorname(), _p_kunde->GetNachname());
            dc.TextOut(left_side, line, text); line += spacing;
            sprintf(text, "%s", _p_kunde->GetStrasse());
            dc.TextOut(left_side, line, text); line += spacing*3;
            dc.SelectObject(TFont("Courier New", spacing*1.2, 0, 0, 0, FW_BOLD, DEFAULT_PITCH|FF_DONTCARE, false, false));
            if(_p_kunde->GetPLZ() > 0) sprintf(text, "%d %s", _p_kunde->GetPLZ(), _p_kunde->GetOrt());
            else sprintf(text, "%s", _p_kunde->GetOrt());
            dc.TextOut(left_side, line, text); line += spacing;
            dc.SelectObject(TFont("Courier New", spacing, 0, 0, 0, FW_BOLD, DEFAULT_PITCH|FF_DONTCARE, false, false));
            sprintf(text, "Telefon/Fax: %s / %s", _p_kunde->GetTelefon(), _p_kunde->GetFax());
            dc.TextOut(left_side, line, text); line += spacing*3;

         } // if

			line = height/20;
			left_side = width/2.3;
         dc.SelectObject(TFont("Courier New", spacing*1.2, 0, 0, 0, FW_BOLD, DEFAULT_PITCH|FF_DONTCARE, false, false));
			if(_no_pages > 1)
         {
	         sprintf(text, "Rechnung Nr.   : %d (Seite %d von %d)", _p_rechnung->GetNrExtern(), page_no, _no_pages);
         } // if
         else
         {
	         sprintf(text, "Rechnung Nr.   : %d", _p_rechnung->GetNrExtern());
         } // else

         dc.TextOut(left_side, line, text);
         line += spacing;

			if((_p_rechnung->GetDatum().Year() > 1970) && (_p_rechnung->GetDatum().Year() < 10000)) sprintf(text, "Datum          : %.2d.%.2d.%.2d", _p_rechnung->GetDatum().DayOfMonth(), _p_rechnung->GetDatum().Month(), _p_rechnung->GetDatum().Year());
         else sprintf(text, "Datum          : N/A");
         dc.TextOut(left_side, line, text);
         line += spacing;

			if(!_p_module_broker->GetCustomizerPtr()->PeterBauer()) {
				sprintf(text, "Erstellt am    : %.2d.%.2d.%.2d %.2d:%.2d:%.2d", _p_rechnung->GetErfassungDatum().DayOfMonth(), _p_rechnung->GetErfassungDatum().Month(), _p_rechnung->GetErfassungDatum().Year(),
				         														 _p_rechnung->GetErfassungZeit().Hour(), _p_rechnung->GetErfassungZeit().Minute(), _p_rechnung->GetErfassungZeit().Second());
	         dc.TextOut(left_side, line, text);
	         line += spacing;

				sprintf(text, "Gewicht insges.: %0.2f kg", (float)_p_rechnung->GetGewicht()/1000.0);
	         dc.TextOut(left_side, line, text);
	         line += spacing*1.5;
         } // if

			line = height/7;
			left_side = width/16;

      	dc.SelectObject(TFont("Courier New", spacing, 0, 0, 0, FW_BOLD, DEFAULT_PITCH|FF_DONTCARE, false, true));
			dc.TextOut(left_side, line, "                                                                                      "); line += spacing;
			dc.TextOut(left_side, line, "Wein Nr. Zahl  Fllmenge   Weinname & Sorte                       Einzel-  Gesamtpreis"); line += spacing*2;
      	dc.SelectObject(TFont("Courier New", spacing, 0, 0, 0, FW_BOLD, DEFAULT_PITCH|FF_DONTCARE, false, false));

		   for(i = 0; i < (page_no-1)*max_posten; i++)
         	_p_rechnung->GetNextPosten(posten, index);
      	break;

      case PACKLISTE:
         max_posten = NO_POSTEN_PACKLISTE;

         left_side = width/15;

         dc.SelectObject(TFont("Courier New", spacing*2, 0, 0, 0, FW_BOLD, DEFAULT_PITCH|FF_DONTCARE, false, true));
         dc.TextOut(left_side, line, "PACKLISTE"); line += spacing*3;

			if(_no_pages > 1)
         {
	         sprintf(text, "Rechnung Nr.: %d (Seite %d von %d)", _p_rechnung->GetNrExtern(), page_no, _no_pages);
         } // if
         else
         {
	         sprintf(text, "Rechnung Nr.: %d", _p_rechnung->GetNrExtern());
         } // else

         dc.SelectObject(TFont("Courier New", spacing, 0, 0, 0, FW_BOLD, DEFAULT_PITCH|FF_DONTCARE, false, false));
         dc.TextOut(left_side, line, text);
         line += spacing*2;

			if(page_no == 1)
			{
            line = width/29;
            left_side = width/2.9;

            sprintf(text, "Kunde: %s %s %s", _p_kunde->GetAnrede(), _p_kunde->GetVorname(), _p_kunde->GetNachname());
            dc.TextOut(left_side, line, text); line += spacing;
            sprintf(text, "       %s", _p_kunde->GetStrasse());
            dc.TextOut(left_side, line, text); line += spacing;
				if(_p_kunde->GetPLZ() > 0) sprintf(text, "       %d %s", _p_kunde->GetPLZ(), _p_kunde->GetOrt());
            else sprintf(text, "       %s", _p_kunde->GetOrt());
            dc.TextOut(left_side, line, text); line += spacing*2;

				sprintf(text, "       Gewicht insges.: %0.2f kg", (float)_p_rechnung->GetGewicht()/1000.0);
	         dc.TextOut(left_side, line, text);
	         line += spacing*1.5;
         } // if

			left_side = width/15;

      	dc.SelectObject(TFont("Courier New", spacing, 0, 0, 0, FW_BOLD, DEFAULT_PITCH|FF_DONTCARE, false, true));
			dc.TextOut(left_side, line, "                                                              "); line += spacing;
			dc.TextOut(left_side, line, "Wein Nr. Name & Bezeichnung                             Anzahl"); line += spacing*2;
      	dc.SelectObject(TFont("Courier New", spacing, 0, 0, 0, FW_BOLD, DEFAULT_PITCH|FF_DONTCARE, false, false));

		   for(int i = 0; i < (page_no-1)*NO_POSTEN_PACKLISTE; i++)
         	_p_rechnung->GetNextPosten(posten, index);
      	break;
   } // switch

   posten_counter = 0;
   while(_p_rechnung->GetNextPosten(posten, index))
   {
	   pWein = (*_p_module_broker->GetWeinlistenVerwaltungPtr())[posten._wein_nr];

		switch(_mode)
      {
      	case RECHNUNG:
            sprintf(text, "%4d                                                       ", posten._anzahl);
            sprintf(&(text[6]), "%s                                                                              ", pWein->GetFuellmenge());
            sprintf(&(text[18]), "%s", pWein->GetName());
            dc.TextOut(left_side, line, text); line += spacing;

            sprintf(text, "                                                                           ");
            sprintf(&(text[18]), "       %s                                                                    ", pWein->GetBezeichnung());
				sprintf(betrag, "%.2f", posten._dm_preis);
            sprintf(&(text[62+(6-strlen(betrag))]), "%s                                                 ", betrag);
				sprintf(betrag, "%.2f", posten._dm_preis*posten._anzahl);
            sprintf(&(text[73+(8-strlen(betrag))]), "%s", betrag);
            dc.TextOut(left_side, line, text); line += spacing*1.75;
         	break;

      	case RECHNUNG_INTERN:
            sprintf(text, "%7d                                                       ", posten._wein_nr);
            sprintf(&(text[9]), "%4d                                                       ", posten._anzahl);
            sprintf(&(text[15]), "%s                                                                              ", pWein->GetFuellmenge());
            sprintf(&(text[27]), "%s", pWein->GetName());
            dc.TextOut(left_side, line, text); line += spacing;

            sprintf(text, "                                                                           ");
            sprintf(&(text[27]), "       %s                                                                    ", pWein->GetBezeichnung());
				sprintf(betrag, "%.2f", posten._dm_preis);
            sprintf(&(text[66+(6-strlen(betrag))]), "%s                                                 ", betrag);
				sprintf(betrag, "%.2f", posten._dm_preis*posten._anzahl);
            sprintf(&(text[77+(8-strlen(betrag))]), "%s", betrag);
            dc.TextOut(left_side, line, text); line += spacing*1.25;
         	break;

         case PACKLISTE:
            sprintf(text, "%7d  %s", pWein->GetNr(), pWein->GetName());
            dc.TextOut(left_side, line, text); line += spacing;
            sprintf(text, "                %s                                                      ", pWein->GetBezeichnung());
            sprintf(&(text[58]), "%4d", posten._anzahl);
            dc.TextOut(left_side, line, text); line += spacing*2;
         	break;
      } // switch

		posten_counter++;
		if(posten_counter >= max_posten) break;
   } // while

   switch(_mode)
   {
   	case RECHNUNG:
			if(_no_pages == page_no)
         {
            steuer = _p_module_broker->GetCustomizerPtr()->GetMehrwertsteuer();

            sprintf(text, "                                                             ____________________");
            dc.TextOut(left_side, line, text); line += spacing*2;

				gesamt_betrag = _p_rechnung->GetBetragGesamt(!(_p_module_broker->GetCustomizerPtr()->GetEuroAktiv()));
				if(_p_module_broker->GetCustomizerPtr()->GetSteuerEnthalten() == false)
            {
               sprintf(text, "                         Summe                                                  ");
               sprintf(betrag, "%s %0.2f", _p_module_broker->GetCustomizerPtr()->GetWaehrungString(), gesamt_betrag);
               sprintf(&(text[63+(18-strlen(betrag))]), "%s", betrag);
               dc.TextOut(left_side, line, text); line += spacing;
            } // if

				if(_p_module_broker->GetCustomizerPtr()->GetSteuerEnthalten() == true)
            {
					end_betrag = gesamt_betrag;

               sprintf(text, "                         ENDBETRAG                                               ");
               sprintf(betrag, "%s %0.2f", _p_module_broker->GetCustomizerPtr()->GetWaehrungString(), end_betrag);
               dc.TextOut(left_side, line, text);
               sprintf(&(text[63+(18-strlen(betrag))]), "%s", betrag);
               dc.TextOut(left_side, line, text); line += spacing;

               sprintf(text, "                                                                                 ");
					for(int i = 0; i < strlen(betrag); i++)
	               sprintf(&(text[63+i+(18-strlen(betrag))]), "=");
               dc.TextOut(left_side, line, text); line += spacing;

		      	dc.SelectObject(TFont("Courier New", spacing*0.75, 0, 0, 0, FW_NORMAL, DEFAULT_PITCH|FF_DONTCARE, false, false));
					sprintf(text, "     der Endbetrag enthlt %s %0.2f (%0.2f %% Umsatzsteuer) ", _p_module_broker->GetCustomizerPtr()->GetWaehrungString(), gesamt_betrag*16.0/100.0, steuer);
               dc.TextOut(left_side, line, text); line += spacing;
		      	dc.SelectObject(TFont("Courier New", spacing, 0, 0, 0, FW_BOLD, DEFAULT_PITCH|FF_DONTCARE, false, false));
				}
            else
            {
					sprintf(text, "                         zzgl. %0.2f % Mehrwertsteuer                                    ", steuer);
					sprintf(betrag, "%s %0.2f", _p_module_broker->GetCustomizerPtr()->GetWaehrungString(), gesamt_betrag*16.0/100.0-0.01);
					end_betrag = gesamt_betrag*(steuer+100.0)/100.0-0.01;

               sprintf(&(text[63+(18-strlen(betrag))]), "%s", betrag);
               dc.TextOut(left_side, line, text); line += spacing*1.5;

               sprintf(text, "                         ENDBETRAG                                               ");
               sprintf(betrag, "%s %0.2f", _p_module_broker->GetCustomizerPtr()->GetWaehrungString(), end_betrag);
               dc.TextOut(left_side, line, text);
               sprintf(&(text[63+(18-strlen(betrag))]), "%s", betrag);
               dc.TextOut(left_side, line, text); line += spacing;

               sprintf(text, "                                                                                 ");
					for(int i = 0; i < strlen(betrag); i++)
	               sprintf(&(text[63+i+(18-strlen(betrag))]), "=");
               dc.TextOut(left_side, line, text); line += spacing;
				}

				line = height - height/10;
            left_side = width/7;

            sprintf(text, "Betrag dankend erhalten");
            dc.TextOut(left_side, line, text); line += spacing;
				if(_p_module_broker->GetCustomizerPtr()->PeterBauer()) sprintf(text, "Freinsheim, den ________________20___");
            else sprintf(text, "_______________, den _____________________");
            dc.TextOut(left_side, line, text); line += spacing;
         } // if
      	break;

   	case RECHNUNG_INTERN:
			if(_no_pages == page_no)
         {
            steuer = _p_module_broker->GetCustomizerPtr()->GetMehrwertsteuer();

            sprintf(text, "                                                                 ____________________");
            dc.TextOut(left_side, line, text); line += spacing*2;

				sprintf(text, "                                Summe                                                  ");
            sprintf(betrag, "%s %0.2f", _p_module_broker->GetCustomizerPtr()->GetWaehrungString(), (gesamt_betrag = _p_rechnung->GetBetragGesamt(!(_p_module_broker->GetCustomizerPtr()->GetEuroAktiv()))));
				sprintf(&(text[67+(18-strlen(betrag))]), "%s", betrag);
            dc.TextOut(left_side, line, text); line += spacing;

				if(_p_module_broker->GetCustomizerPtr()->GetSteuerEnthalten() == true)
            {
					sprintf(text, "                                davon %0.2f % Mehrwertsteuer                                   ", steuer);
					sprintf(betrag, "%s %0.2f", _p_module_broker->GetCustomizerPtr()->GetWaehrungString(), gesamt_betrag*steuer/100.0);
					end_betrag = gesamt_betrag;
				}
            else
            {
					sprintf(text, "                                zzgl. %0.2f % Mehrwertsteuer                                   ", steuer);
					sprintf(betrag, "%s %0.2f", _p_module_broker->GetCustomizerPtr()->GetWaehrungString(), gesamt_betrag*16.0/100.0-0.01);
					end_betrag = gesamt_betrag*(steuer+100.0)/100.0-0.01;
				}

            sprintf(&(text[67+(18-strlen(betrag))]), "%s", betrag);
            dc.TextOut(left_side, line, text); line += spacing;

				sprintf(text, "                                Endbetrag                                               ");
				sprintf(betrag, "%s %0.2f", _p_module_broker->GetCustomizerPtr()->GetWaehrungString(), end_betrag);
            sprintf(&(text[67+(18-strlen(betrag))]), "%s", betrag);
            dc.TextOut(left_side, line, text); line += spacing;

				if(_p_module_broker->GetCustomizerPtr()->PeterBauer()) {
					line = height - height/10;
	            left_side = width/7;

	            sprintf(text, "Betrag dankend erhalten");
	            dc.TextOut(left_side, line, text); line += spacing;
					sprintf(text, "Freinsheim, den ________________20___");
	            dc.TextOut(left_side, line, text); line += spacing;
            } // if
         } // if
      	break;
   } // switch
} // PrintRechnung::RenderPage
