//----------------------------------------------------------------------------
//  Project weinadmin
//
//  Copyright  1996. Alle Rechte vorbehalten.
//
//  SUBSYSTEM:    weinadmin.apx Application
//  FILE:         rechnungsliste.cpp
//  AUTIDC_STOREOR:       Thomas IDC_STOREaenselmann
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of RechnungsListe (TDialog).
//
//----------------------------------------------------------------------------
#include <owl/pch.h>

#include <stdio.h>
#include "mck/rechnung/MCKrechnung.h"
#include "rechnungsliste.h"
#include "weinliste.h"
#include "druckdialog.h"

#include <pbk/wein/weinlistenverwaltung.h>
#include <pbk/preis/preisverwaltung.h>
#include <pbk/rechnung/rechnungsverwaltung.h>


//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE1(RechnungsListe, TDialog)
//{{RechnungsListeRSP_TBL_BEGIN}}
  EV_LVN_GETDISPINFO(IDC_TABELLE, LVNGetdispinfo),
  EV_LVN_ITEMCHANGING(IDC_TABELLE, LVNItemchanging),
  EV_BN_CLICKED(IDC_BEARBEITEN, CmBearbeiten),
  EV_BN_CLICKED(IDC_HEUTE, CmHeute),
  EV_BN_CLICKED(IDC_LEER, CmLeer),
  EV_BN_CLICKED(IDC_STORE, CmStore),
  EV_BN_CLICKED(ODC_DRUCKEN, CmDrucken),
  EV_BN_CLICKED(IDC_CLEAR, CmLoeschen),
  EV_BN_CLICKED(IDC_NUROFFENE, CmNurOffene),
  EV_BN_CLICKED(IDC_DUPLICATERECHNUNG, CmDuplicateRechnung),
  EV_EN_CHANGE(IDC_BETRAGEINGEGANGEN, CmChangeEingegangen),
  EV_BN_CLICKED(IDC_BEGLICHEN, CmDataChange),
  EV_EN_CHANGE(IDC_BEMERKUNG, CmDataChange),
  EV_EN_CHANGE(IDC_DATUMJAHR, CmDataChange),
  EV_EN_CHANGE(IDC_DATUMMONAT, CmDataChange),
  EV_EN_CHANGE(IDC_DATUMTAG, CmDataChange),
  EV_EN_CHANGE(IDC_KARTONS, CmDataChange),
  EV_EN_CHANGE(IDC_NUMMER, CmDataChange),
  EV_EN_CHANGE(IDC_STEIGEN, CmDataChange),
//{{RechnungsListeRSP_TBL_END}}
END_RESPONSE_TABLE;


//{{RechnungsListe Implementation}}


static RechnungsListeXfer RechnungsListeData;

RechnungsListe::RechnungsListe(TWindow* parent, RechnungsServer* pRechnungsServer, ModuleBroker* pModuleBroker, Kunde* pKunde, TResId resId, TModule* module)
										 : TDialog(parent, resId, module), _p_rechnungs_server(pRechnungsServer), _p_module_broker(pModuleBroker), _p_current_kunde(pKunde)
{
//{{RechnungsListeXFER_USE}}
  pBeglichen = new TCheckBox(this, IDC_BEGLICHEN, 0);
  pBemerkung = new TEdit(this, IDC_BEMERKUNG, 255);
  pBetragEingegangen = new TEdit(this, IDC_BETRAGEINGEGANGEN, 255);
  pBetragGesamt = new TEdit(this, IDC_BETRAGGESAMT, 255);
  pDatumJahr = new TEdit(this, IDC_DATUMJAHR, 255);
  pDatumMonat = new TEdit(this, IDC_DATUMMONAT, 255);
  pDatumTag = new TEdit(this, IDC_DATUMTAG, 255);
  pKartons = new TEdit(this, IDC_KARTONS, 255);
  pNummer = new TEdit(this, IDC_NUMMER, 255);
  pSteigen = new TEdit(this, IDC_STEIGEN, 255);
  pTabelle = new TListWindow(this, IDC_TABELLE);
  pBearbeiten = new TButton(this, IDC_BEARBEITEN);
  pNurOffene = new TCheckBox(this, IDC_NUROFFENE, 0);
  pDuplicate = new TButton(this, IDC_DUPLICATERECHNUNG);
  pDrucken = new TButton(this, ODC_DRUCKEN);
  pLoeschen = new TButton(this, IDC_CLEAR);

  SetTransferBuffer(&RechnungsListeData);
//{{RechnungsListeXFER_USE_END}}

  // INSERT>> Your constructor code here.

  _p_current_rechnung = NULL;

  _p_liste_rechnungen = new GenericList<Rechnung>;

  if(!pKunde) _mode = ZEITLISTE;
  else _mode = KUNDENLISTE;
}


RechnungsListe::~RechnungsListe()
{
  Destroy(IDCANCEL);

  // INSERT>> Your destructor code here.

  if(_p_liste_rechnungen) delete _p_liste_rechnungen; _p_liste_rechnungen = NULL;
}


void RechnungsListe::SetupWindow()
{
  TDialog::SetupWindow();

  // INSERT>> Your code here.

  pTabelle->InsertColumn(0, TListWindColumn("Nr.", 45, TListWindColumn::Right));
  pTabelle->InsertColumn(1, TListWindColumn("Datum", 70));
  pTabelle->InsertColumn(2, TListWindColumn("Betrag ges.", 75, TListWindColumn::Right));
  pTabelle->InsertColumn(3, TListWindColumn("Betrag eingeg.", 85, TListWindColumn::Right));
  pTabelle->InsertColumn(4, TListWindColumn("Beglichen", 65, TListWindColumn::Right));
  pTabelle->InsertColumn(5, TListWindColumn("Steigen", 50, TListWindColumn::Right));
  pTabelle->InsertColumn(6, TListWindColumn("Steigen", 45, TListWindColumn::Right));
  if(_mode == KUNDENLISTE) pTabelle->InsertColumn(7, TListWindColumn("Bemerkung", 220));
  if(_mode == ZEITLISTE) pTabelle->InsertColumn(7, TListWindColumn("Kunde", 220));

  pDuplicate->EnableWindow(false);
  pDrucken->EnableWindow(false);
  pLoeschen->EnableWindow(false);
  pBearbeiten->EnableWindow(false);

  pNurOffene->SetCheck(BF_CHECKED);
  _nur_offene = true;

  FillTabelle();
}


void RechnungsListe::FillTabelle()
{
  char text[256];

  SetCursor(NULL, IDC_WAIT);
  _p_liste_rechnungen->Empty();
  _p_rechnungs_server->FillRechnungsListe(*_p_liste_rechnungen, _nur_offene);

  pBearbeiten->EnableWindow(false);

  pBemerkung->Clear();
  pBetragEingegangen->Clear();
  pBetragGesamt->Clear();
  pDatumJahr->Clear();
  pDatumMonat->Clear();
  pDatumTag->Clear();
  pKartons->Clear();
  pNummer->Clear();
  pSteigen->Clear();
  pBeglichen->SetCheck(BF_UNCHECKED);

  pTabelle->DeleteAllItems();

  for(int i = 0; i < _p_liste_rechnungen->GetNoElements(); i++)
  {
    sprintf(text, "%d", (*_p_liste_rechnungen)[i].GetNrExtern());
    TListWindItem item(text, 0);
	 pTabelle->AddItem(item);
  } // for

  SetCursor(NULL, IDC_ARROW);
  _dirty = false; // es wurden noch keine Daten verndert
} // RechnungsListe::FillTabelle

void RechnungsListe::LVNGetdispinfo(TLwDispInfoNotify& lwdin)
{
  // INSERT>> Your code here.

  TListWindItem& dispInfoItem = *(TListWindItem*)&lwdin.item;
  Rechnung       *pRechnung;
  char   		  text[256];

  pRechnung = &((*_p_liste_rechnungen)[dispInfoItem.GetIndex()]);

   if((dispInfoItem.GetIndex() % 2) == 0)
     pTabelle->SetTextBkColor(TColor(192, 255, 192));
   else
     pTabelle->SetTextBkColor(TColor(255, 255, 255));

  switch(dispInfoItem.GetSubItem())
  {
    case 1:
		if((pRechnung->GetDatum().Year() > 1970) && (pRechnung->GetDatum().Year() < 10000))
			sprintf(text, "%d.%d.%d", pRechnung->GetDatum().DayOfMonth(), pRechnung->GetDatum().Month(), pRechnung->GetDatum().Year());
      else sprintf(text, "N/A");
      break;

    case 2:
		sprintf(text, "%0.2f", pRechnung->GetBetragGesamt(!(_p_module_broker->GetCustomizerPtr()->GetEuroAktiv())));
      break;

    case 3:
		sprintf(text, "%0.2f", pRechnung->GetBetragEingegangen());
      break;

    case 4:
		if(pRechnung->GetBeglichen()) strcpy(text, "ja");
      else strcpy(text, "nein");
      break;

    case 5:
		sprintf(text, "%d", pRechnung->GetKartons());
      break;

    case 6:
		sprintf(text, "%d", pRechnung->GetKisten());
      break;

    case 7:
		if(_mode == KUNDENLISTE) sprintf(text, "%s", pRechnung->GetBemerkung());
      if(_mode == ZEITLISTE)
      {
      	Kunde* pLocalKunde;
         pLocalKunde = (*_p_module_broker->GetKundenVerwaltungPtr())[pRechnung->GetKundeNr()];

         if(pLocalKunde)
         {
         	sprintf(text, "%s %s aus %s", pLocalKunde->GetVorname(), pLocalKunde->GetNachname(), pLocalKunde->GetOrt());
         } // if
      } // if
      break;
  } // switch

  dispInfoItem.SetText(text);
}


bool RechnungsListe::LVNItemchanging(TLwNotify& lwn)
{
   // INSERT>> Your code here.

   if(_dirty)
   {
   	if(MessageBox("Vorsicht: Sie haben Daten in den Eingabefeldern verndert ohne danach zu speichern. Sollen die nderungen unbercksichtigt bleiben?", "Neuen Tabelleneintrag whlen", MB_YESNO) == IDNO) return true;
   } // if

   char text[256];

   _p_current_rechnung = &((*_p_liste_rechnungen)[lwn.iItem]);
   _p_current_rechnung->SetModuleBrokerPtr(_p_module_broker);

   sprintf(text, "%d", _p_current_rechnung->GetNrExtern());
   pNummer->SetText(text);

	if((_p_current_rechnung->GetDatum().Year() > 1970) && (_p_current_rechnung->GetDatum().Year() < 10000))
   {
      sprintf(text, "%d", _p_current_rechnung->GetDatum().DayOfMonth());
      pDatumTag->SetText(text);

      sprintf(text, "%d", _p_current_rechnung->GetDatum().Month());
      pDatumMonat->SetText(text);

      sprintf(text, "%d", _p_current_rechnung->GetDatum().Year());
      pDatumJahr->SetText(text);
   } // if
   else
   {
      pDatumTag->SetText("");
      pDatumMonat->SetText("");
      pDatumJahr->SetText("");
   } // else

   pBemerkung->SetText(_p_current_rechnung->GetBemerkung());

   sprintf(text, "%d", _p_current_rechnung->GetKisten());
   pKartons->SetText(text);

   sprintf(text, "%d", _p_current_rechnung->GetKartons());
   pSteigen->SetText(text);

   sprintf(text, "%0.2f", _p_current_rechnung->GetBetragEingegangen());
   pBetragEingegangen->SetText(text);

   sprintf(text, "%0.2f", _p_current_rechnung->GetBetragGesamt(!(_p_module_broker->GetCustomizerPtr()->GetEuroAktiv())));
   pBetragGesamt->SetText(text);

   if(_p_current_rechnung->GetBeglichen()) pBeglichen->SetCheck(BF_CHECKED);
   else pBeglichen->SetCheck(BF_UNCHECKED);

   _dirty = false;

   pDuplicate->EnableWindow(true);
   pDrucken->EnableWindow(true);
   pLoeschen->EnableWindow(true);
   pBearbeiten->EnableWindow(true);

   return false;
}


void RechnungsListe::CmBearbeiten()
{
  // INSERT>> Your code here.

  bool full_list = true;

  if(_mode == ZEITLISTE) full_list = false;

  WeinListe MeineBestellliste(this, _p_module_broker, WeinListe::BESTELLUNG, _p_current_kunde, _p_current_rechnung, full_list);
  MeineBestellliste.Execute();
  FillTabelle();
}


void RechnungsListe::CmHeute()
{
  // INSERT>> Your code here.

  TDate Heute;
  char  text[10];

  sprintf(text, "%d", Heute.DayOfMonth());
  pDatumTag->SetText(text);
  sprintf(text, "%d", Heute.Month());
  pDatumMonat->SetText(text);
  sprintf(text, "%d", Heute.Year());
  pDatumJahr->SetText(text);
}


void RechnungsListe::CmLeer()
{
  // INSERT>> Your code here.

  pDatumTag->Clear();
  pDatumMonat->Clear();
  pDatumJahr->Clear();
}


void RechnungsListe::CmStore()
{
  // INSERT>> Your code here.

   char  text[456];
   short tag, monat, jahr;

	if(_p_current_rechnung)
   {
      pBemerkung->GetText(text, 255);
      _p_current_rechnung->SetBemerkung(text);

      pSteigen->GetText(text, 255);
      _p_current_rechnung->SetKartons(atoi(text));

      pKartons->GetText(text, 255);
      _p_current_rechnung->SetKisten(atoi(text));

      pNummer->GetText(text, 255);
      _p_current_rechnung->SetNrExtern(atoi(text));

      pBetragEingegangen->GetText(text, 255);
      _p_current_rechnung->SetBetragEingegangen((float)atof(text));

      pDatumTag->GetText(text, 255);
      tag = atoi(text);

      pDatumMonat->GetText(text, 255);
      monat = atoi(text);

      pDatumJahr->GetText(text, 255);
      jahr = atoi(text);

      TDate datum_neu(tag, monat, jahr);

      if((jahr > 1970) && ((int)(_p_current_rechnung->GetErfassungDatum() - datum_neu) > _p_module_broker->GetCustomizerPtr()->GetVordatierung()))
      {
			sprintf(text, "Diese Rechnung wurde am %d.%d.%d erstellt. Sie haben versucht die Rechnung mehr als %d Tage vorzudatieren. Damit ist die Grenze zur Vordatierung berschritten. Wenn dies ein Problem ist, kann die Grenze in den 'Grundeinstellungen' bel. verndert werden. WICHTIG: Je grer die Vordatierungsgrenze (in Tagen) gewhlt wird, desto langsamer werden einige Programmfunktionen.", _p_current_rechnung->GetErfassungDatum().DayOfMonth(), _p_current_rechnung->GetErfassungDatum().Month(), _p_current_rechnung->GetErfassungDatum().Year(), _p_module_broker->GetCustomizerPtr()->GetVordatierung());
			MessageBox(text, "Rechnungsdaten ndern", MB_OK);
      	return;
      } // if
		else _p_current_rechnung->SetDatum(datum_neu);

      if(pBeglichen->GetCheck() == BF_CHECKED) _p_current_rechnung->SetBeglichen(true);
      if(pBeglichen->GetCheck() == BF_UNCHECKED) _p_current_rechnung->SetBeglichen(false);

      _p_module_broker->GetRechnungsVerwaltungPtr()->UpdateRechnung(_p_current_rechnung);

      FillTabelle();
	   _dirty = false;
      _p_current_rechnung = NULL;
   } // if
}

void RechnungsListe::CmSysDrucken(TWindow* pParentWindow, ModuleBroker* pModuleBroker, Rechnung* pRechnung, bool output_packliste, bool output_rechnung, bool output_rechnung_intern, bool pre_view)
{
   PrintRechnung::Mode mode;
   bool				     do_print;

   for(int i = 0; i < 3; i++)
   {
      switch(i)
      {
         case 0:
            if(output_packliste)
            {
               mode = PrintRechnung::PACKLISTE;
               do_print = true;
            } // if
            else do_print = false;
            break;

         case 1:
            if(output_rechnung)
            {
               mode = PrintRechnung::RECHNUNG;
               do_print = true;
            } // if
            else do_print = false;
            break;

         case 2:
            if(output_rechnung_intern)
            {
               mode = PrintRechnung::RECHNUNG_INTERN;
               do_print = true;
            } // if
            else do_print = false;
            break;
      } // switch

      if(do_print)
      {
         Kunde* pKunde = (*pModuleBroker->GetKundenVerwaltungPtr())[pRechnung->GetKundeNr()];
         PrintRechnung* pRechnungsDrucker = new PrintRechnung(mode, pre_view, pModuleBroker, pKunde, pRechnung);
         pRechnungsDrucker->RenderPage(pParentWindow);

         // Wenn die Rechnung auf dem Drucker ausgegeben wird, mu
         // PrintRechnung wieder explizit gelscht werden

         if(!pre_view) delete pRechnungsDrucker; pRechnungsDrucker = NULL;
      } // if
   } // for
} // RechnungsListe::CmSysDrucken


void RechnungsListe::CmDrucken()
{
  // INSERT>> Your code here.

   PrintRechnung::Mode mode;
	Rechnung*			  pRechnung;
   bool 					  output_packliste, output_rechnung, output_rechnung_intern, pre_view;
   bool				     do_print, passed;

	if(_p_current_rechnung)
   {
		output_packliste = output_rechnung = pre_view = false;

      DruckDialog LocalDruckDialog(this, output_packliste, output_rechnung, output_rechnung_intern, pre_view);
      LocalDruckDialog.Execute();

      int index = -1; passed = false;
      while((index = pTabelle->GetNextItem(index, TListWindow::Selected)) >= 0)
      {
			passed = true;
			pRechnung = &((*_p_liste_rechnungen)[index]);
			CmSysDrucken(this, _p_module_broker, pRechnung, output_packliste, output_rechnung, output_rechnung_intern, pre_view);
      } // while

      if(!passed)
      	MessageBox("Es wurde keine Rechnung zum Drucken selektiert. Whlen Sie hierzu mit der Maus die Nummer der entsprechenden Rechnung an. Um mehrere Rechnungen auf einmal auszugeben muss die Strg-Taste gedrckt gehalten werden.", "Rechnungen drucken", MB_OK);
   } // if
}

void RechnungsListe::CmLoeschen()
{
  // INSERT>> Your code here.

  	int  answer;
   bool delete_posten;

	if(_p_current_rechnung)
   {
      answer = MessageBox("Vorsicht: Sie sind im Begriff eine Rechnung aus dem Datenbestand zu entfernen. Mchten Sie, da vor dem Lschen der Flaschenbestand der betroffenen Weine berichtigt wird? Whlen Sie Abbruch, fall Sie nicht sicher sind.", "Vorgang Rechnung lschen", MB_YESNOCANCEL);

      switch(answer)
      {
         case IDYES:
            delete_posten = true;
            break;

         case IDNO:
            delete_posten = false;
            break;

         case IDCANCEL:
            MessageBox("Es wurde keine Rechnung gelscht", "Vorgang Rechnung lschen", MB_OK);
            return;
      } // switch

      _p_module_broker->GetRechnungsVerwaltungPtr()->DeleteRechnung(_p_current_rechnung->GetNr(), delete_posten);
      FillTabelle();
      _p_current_rechnung = NULL;
   } // if
   else MessageBox("Es wurde noch gar keine Rechnung markiert", "Vorgang Rechnung lschen", MB_OK);
}


void RechnungsListe::CmNurOffene()
{
  // INSERT>> Your code here.

  _nur_offene = !_nur_offene;
  FillTabelle();
}


void RechnungsListe::CmDuplicateRechnung()
{
  // INSERT>> Your code here.

	if(_p_current_rechnung)
	{
      if(MessageBox("Vorsicht: Sie sind dabei eine Rechnung zu duplizieren. Damit wird auch der Bestand an Weinen verndert. Mchte Sie fortfahren?", "Rechnung duplizieren", MB_YESNO) == IDYES)
      {
        _p_module_broker->GetRechnungsVerwaltungPtr()->Duplicate(_p_current_rechnung);
        FillTabelle();
      } // if
	} // if
}


void RechnungsListe::CmChangeEingegangen()
{
	// INSERT>> Your code here.

   char text[30];

   pBetragEingegangen->GetText(text, 29);

   if((_p_current_rechnung) && atof(text) >= _p_current_rechnung->GetBetragGesamt(!(_p_module_broker->GetCustomizerPtr()->GetEuroAktiv())))
   	pBeglichen->SetCheck(BF_CHECKED);

   CmDataChange();
}


void RechnungsListe::CmDataChange()
{
  // INSERT>> Your code here.

  _dirty = true;
}
