//----------------------------------------------------------------------------
//  Project WeinAdmin
//
//  Copyright  1996. Alle Rechte vorbehalten.
//
//  SUBSYSTEM:    WeinAdmin Application
//  FILE:         weinadminapp.cpp
//  AUTHOR:       Thomas Haenselmann
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of TWeinAdminApp (TApplication).
//
//----------------------------------------------------------------------------

#include <owl/pch.h>

#include <owl/buttonga.h>
// debug
// #include <owl/docmanag.h>
#include <owl/statusba.h>
#include <owl/filedoc.h>
#include <owl/opensave.h>
#include <stdio.h>

#include "weinadminapp.h"
#include "weinadmindlgclient.h"                        // Definition of client class.
#include "weinadminaboutdlg.h"                        // Definition of about dialog.
#include "kundendialog.h"
#include "weinliste.h"
#include "rechnungsliste.h"
#include "rechnungsfiler.h"
#include "dialogcustomizer.h"
#include "merkzetteldlg.h"
#include "geofilterdlg.h"
#include "rechnungsnummer.h"
#include "displayevent.h"

//{{TWeinAdminApp Implementation}}


/*
DEFINE_DOC_TEMPLATE_CLASS(TFileDocument, TWeinAdminDlgClient, DlgDTPL);
DlgDTPL __dlogClnt("All Files", "*.*", 0, "TXT", dtAutoDelete | dtUpdateDir);
*/
//{{DOC_VIEW}}
//{{DOC_VIEW_END}}

//{{DOC_MANAGER}}
//{{DOC_MANAGER_END}}


//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE1(TWeinAdminApp, TApplication)
//{{TWeinAdminAppRSP_TBL_BEGIN}}
// debug
//  EV_OWLVIEW(dnCreate, EvNewView),
//  EV_OWLVIEW(dnClose,  EvCloseView),
  EV_COMMAND(CM_HELPABOUT, CmHelpAbout),
//{{TWeinAdminAppRSP_TBL_END}}
END_RESPONSE_TABLE;


//--------------------------------------------------------
// TWeinAdminApp
// ~~~~~
//
TWeinAdminApp::TWeinAdminApp() : TApplication("WeinAdmin")
{
// debug
//  SetDocManager(new TDocManager(dmSDI, this));

  // INSERT>> Your constructor code here.
}


TWeinAdminApp::~TWeinAdminApp()
{
  // INSERT>> Your destructor code here.
}


void TWeinAdminApp::CreateGadgets(TDockableControlBar* cb, bool server)
{
  cb->Insert(*new TSeparatorGadget(20));
  cb->Insert(*new TButtonGadget(23004, CM_SEARCHKUNDE));
  cb->Insert(*new TButtonGadget(23000, CM_WEINEDIT));
  cb->Insert(*new TButtonGadget(23001, CM_PREISEDIT));
  cb->Insert(*new TButtonGadget(23002, CM_ADDRECHNUNG));
  cb->Insert(*new TButtonGadget(23003, CM_FLASCHENCOUNT));
  cb->Insert(*new TSeparatorGadget(60));
  cb->Insert(*new TButtonGadget(25200, CM_HELPABOUT));

  // Add caption and fly-over help hints.
  //
  cb->SetCaption("Quick-Men");
  cb->SetHintMode(TGadgetWindow::EnterHints);
}


void TWeinAdminApp::SetupSpeedBar(TDecoratedFrame* frame)
{
  ApxHarbor = new THarbor(*frame);

  // Create default toolbar New and associate toolbar buttons with commands.
  //
  TDockableControlBar* cb = new TDockableControlBar(frame, TDockableControlBar::Vertical);
  CreateGadgets(cb);

  // Setup the toolbar ID used by OLE 2 for toolbar negotiation.
  //
  cb->Attr.Id = IDW_TOOLBAR;

  ApxHarbor->Insert(*cb, alNone, &TPoint(15, 75));
}


//--------------------------------------------------------
// TWeinAdminApp
// ~~~~~
// Application main window construction & intialization.
//
void TWeinAdminApp::InitMainWindow()
{
  if (nCmdShow != SW_HIDE)
    nCmdShow = (nCmdShow != SW_SHOWMINNOACTIVE) ? SW_SHOWMAXIMIZED : nCmdShow;

  TWindow *pBackdropWindow;
  pBackdropWindow = new TWindow((TWindow*)NULL);
  pBackdropWindow->SetBkgndColor(TColor(0, 50, 0));

  TSDIDecFrame* frame = new TSDIDecFrame(0, GetName(), pBackdropWindow, true, this);

  // Assign icons for this application.
  //
  frame->SetIcon(this, IDI_SDIAPPLICATION);
  frame->SetIconSm(this, IDI_SDIAPPLICATION);

  // Menu associated with window and accelerator table associated with table.
  //
  frame->AssignMenu(IDM_SDI);

  // Associate with the accelerator table.
  //
  frame->Attr.AccelTable = IDM_SDI;
/*
  TStatusBar* sb = new TStatusBar(frame, TGadget::Recessed,
                                  TStatusBar::CapsLock        |
                                  TStatusBar::NumLock         |
                                  TStatusBar::ScrollLock);
  frame->Insert(*sb, TDecoratedFrame::Bottom);
*/
  SetupSpeedBar(frame);

  SetMainWindow(frame);

  frame->SetMenuDescr(TMenuDescr(IDM_SDI));

}


//--------------------------------------------------------
// TWeinAdminApp
// ~~~~~
// Application instance initialization.
//
void TWeinAdminApp::InitInstance()
{
  TApplication::InitInstance();
// debug
//  GetDocManager()->CreateAnyDoc(0, dtNewDoc);
}



//--------------------------------------------------------
// TWeinAdminApp
// ~~~~~
// Response Table handlers:
//
// debug
/*
void TWeinAdminApp::EvNewView(TView& view)
{
  GetMainWindow()->SetClientWindow(view.GetWindow());
  if (!view.IsOK())
    GetMainWindow()->SetClientWindow(0);
  else if (view.GetViewMenu())
    GetMainWindow()->MergeMenu(*view.GetViewMenu());
}


void TWeinAdminApp::EvCloseView(TView&)
{
  GetMainWindow()->SetClientWindow(0);
  GetMainWindow()->SetCaption("WeinAdmin");
}
*/

//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE1(TSDIDecFrame, TDecoratedFrame)
//{{TSDIDecFrameRSP_TBL_BEGIN}}
  EV_COMMAND(CM_ADDKUNDE, CmAddkunde),
  EV_COMMAND(CM_SEARCHKUNDE, CmSearchkunde),
  EV_COMMAND(CM_WEINEDIT, CmWeinEdit),
  EV_COMMAND(CM_PREISEDIT, CmPreisedit),
  EV_COMMAND(CM_ADDRECHNUNG, CmAddRechnung),
  EV_COMMAND(CM_FLASCHENCOUNT, CmFlaschencount),
  EV_COMMAND(CM_KUNDENEXPORT, CmKundenexport),
  EV_COMMAND(CM_RECALLRECHNUNGEN, CmRecallrechnungen),
  EV_COMMAND(CM_RECALLRECHNUNGEN, CmRecallrechnungen),
  EV_COMMAND(CM_LOADDOC, CmLoadDoc),
  EV_COMMAND(CM_PREFERENCES, CmPreferences),
  EV_COMMAND(CM_MERKZETTEL, CmMerkzettel),
  EV_COMMAND(CM_RECALL_RE_NUMBER, CmRecallReByNumber),
  EV_COMMAND(CM_PREISEXPORT, CmPreisExport),
  EV_WM_SETFOCUS,
//{{TSDIDecFrameRSP_TBL_END}}
END_RESPONSE_TABLE;


//{{TSDIDecFrame Implementation}}


TSDIDecFrame::TSDIDecFrame(TWindow* parent, const char far* title, TWindow* clientWnd, bool trackMenuSelection, TModule* module)
:
  TDecoratedFrame(parent, title, clientWnd, trackMenuSelection, module), // _kunden_verwaltung("daten/kunden.dat"),
  _weinlisten_verwaltung("daten/weinliste.dat"), _preis_verwaltung("daten/weinlisten.txt", "daten"),
  _posten_verwaltung("daten/rechnungsposten.dat"), _rechnungs_verwaltung("daten/rechnungen.dat"),
  _geo_gruppe("daten/geogruppe.txt"), _log_buch("daten/logbuch.txt"), _customizer("daten/preferences.dat")
{
  // INSERT>> Your constructor code here.

  _module_broker.SetKundenVerwaltungPtr(NULL);
  _module_broker.SetWeinlistenVerwaltungPtr(&_weinlisten_verwaltung);
  _module_broker.SetPreisVerwaltungPtr(&_preis_verwaltung);
  _module_broker.SetPostenVerwaltungPtr(&_posten_verwaltung);
  _module_broker.SetRechnungsVerwaltungPtr(&_rechnungs_verwaltung);
  _module_broker.SetGeoGruppePtr(&_geo_gruppe);
  _module_broker.SetLogBuchPtr(&_log_buch);
  _module_broker.SetCustomizerPtr(&_customizer);

  _posten_verwaltung.SetWeinlistenVerwaltungPtr(&_weinlisten_verwaltung);
  _weinlisten_verwaltung.SetLogBuchPtr(&_log_buch);
  _preis_verwaltung.SetLogBuchPtr(&_log_buch);
  _posten_verwaltung.SetLogBuchPtr(&_log_buch);
  _posten_verwaltung.SetRechnungsVerwaltungPtr(&_rechnungs_verwaltung);
  _rechnungs_verwaltung.SetModuleBrokerPtr(&_module_broker);
}


TSDIDecFrame::~TSDIDecFrame()
{
  // INSERT>> Your destructor code here.

  delete _p_merkzettel_verwaltung; _p_merkzettel_verwaltung = NULL;
  if(_p_kunden_verwaltung) delete _p_kunden_verwaltung; _p_kunden_verwaltung = NULL;
}


//--------------------------------------------------------
// TWeinAdminApp
// ~~~~~~~~~~~
//
// Menu Help About WeinAdmin command
void TWeinAdminApp::CmHelpAbout()
{
  // Show the modal dialog.
  //
  TWeinAdminAboutDlg(GetMainWindow()).Execute();
}


int OwlMain(int , char* [])
{
  TWeinAdminApp   app;
  return app.Run();
}

void TSDIDecFrame::CmAddkunde()
{
  // INSERT>> Your code here.

  KundenDialog MeinKundenDialog(this, KundenDialog::ADD, this, &_module_broker);
  MeinKundenDialog.Execute();
}


void TSDIDecFrame::CmSearchkunde()
{
  // INSERT>> Your code here.

  KundenDialog MeinKundenDialog(this, KundenDialog::SEARCH, this, &_module_broker);
  MeinKundenDialog.Execute();
}


void TSDIDecFrame::CmWeinEdit()
{
  // INSERT>> Your code here.

  WeinListe MeineWeinListe(this, &_module_broker, WeinListe::WEINLISTE);
  MeineWeinListe.Execute();
}


void TSDIDecFrame::CmPreisedit()
{
  // INSERT>> Your code here.

  if(_preis_verwaltung.GetNoPreisliste() < 1)
  {
     MessageBox("Vor der Erstellung von Preislisten mssen diese zuvor benannt werden. Dies kann im Dialog Optionen/Grundeinstellungen getan werden. Ausfhrliche Hinweise finden sich in der Hilfe unter dem Men Hife/Benutzerhandbuch.", "Preisliste editieren", MB_OK);
     return;
  } // if

  WeinListe MeineWeinListe(this, &_module_broker, WeinListe::PREISLISTE);
  MeineWeinListe.Execute();

}


void TSDIDecFrame::CmAddRechnung()
{
  // INSERT>> Your code here.

  if(_preis_verwaltung.GetNoPreisliste() < 1)
  {
     MessageBox("Vor der Erstellung einer Rechnung mssen zuvor Preislisten benannt und erstellt werden. Ausfhrliche Hinweise finden sich in der Hilfe unter dem Men Hife/Benutzerhandbuch.", "Rechnung erstellen", MB_OK);
     return;
  } // if

  KundenDialog MeinKundenDialog(this, KundenDialog::RECHNUNG, this, &_module_broker);
  MeinKundenDialog.Execute();
}


void TSDIDecFrame::CmFlaschencount()
{
  // INSERT>> Your code here.

   long	i, last_wein_nr, summe, zahlen_pro_zeile;
   bool  first_time, first_komma;
   char  text[300];

   RechnungsFiler LocalRFilter(this, _module_broker.GetCustomizerPtr()->ErfassungFirst(),
   									 _module_broker.GetCustomizerPtr()->ErfassungLast(),
                               _module_broker.GetCustomizerPtr()->DatierungFirst(),
                               _module_broker.GetCustomizerPtr()->DatierungLast(),
									    _module_broker.GetCustomizerPtr()->UseErfassung(),
                               _module_broker.GetCustomizerPtr()->UseDatierung(), &_module_broker);
   if(LocalRFilter.Execute() == IDOK)
   {
      TOpenSaveDialog::TData FileData((uint32)OFN_NOTESTFILECREATE|OFN_HIDEREADONLY, (char*)"Textdatei (*.TXT)|*.txt|Excel Tabelle (*.XLS)|*.xls|",
                                     (char*)"*.txt", (char*)"c:/tmp", (char*)"txt");

//      FileData.Flags = FileData.Flags & (~OFN_EXPLORER);

      TDate Heute;
      sprintf(FileData.FileName, "fl_%d_%d_%d", Heute.DayOfMonth(), Heute.Month(), Heute.Year());

      if(TFileSaveDialog(this, FileData, 0, "Flaschenliste erzeugen").Execute() == IDOK)
      {
         char fileTitle[MAXPATH];
         TOpenSaveDialog::GetFileTitle(FileData.FileName, fileTitle, MAXPATH);

         MessageBox("Anhngig von der Anzahl der gefundenen Flaschen kann die Erstellung der Flaschenliste einige Zeit in Anspruch nehmen.", "Erstellen der Flaschenliste", MB_OK);

         switch(FileData.FilterIndex)
			{
         	case 1:
               SetCursor(0, IDC_WAIT);
               _rechnungs_verwaltung.ExportFlaschenListe(FileData.FileName, RechnungsVerwaltung::WRITE_TXTLIST);
               SetCursor(0, IDC_ARROW);

               sprintf(text, "notepad.exe %s", FileData.FileName);
               WinExec(text, SW_SHOW);
            	break;

				case 2:
               SetCursor(0, IDC_WAIT);
               _rechnungs_verwaltung.ExportFlaschenListe(FileData.FileName, RechnungsVerwaltung::WRITE_XLSLIST);
               SetCursor(0, IDC_ARROW);
               break;

            default:
            	break;

         } // switch
      } // if
   } // if
}

void TSDIDecFrame::CmKundenexport()
{
  // INSERT>> Your code here.

  GeoFilterDlg LocalGeoFilterDlg(this, &_module_broker);
  LocalGeoFilterDlg.Execute();
}


void TSDIDecFrame::CmRecallrechnungen()
{
  // INSERT>> Your code here.

   RechnungsFiler LocalRFilter(this, _customizer.ErfassungFirst(), _customizer.ErfassungLast(), _customizer.DatierungFirst(), _customizer.DatierungLast(), _customizer.UseErfassung(), _customizer.UseDatierung(), &_module_broker);
   if(LocalRFilter.Execute() == IDOK)
   {
		RechnungsListe LocalRechnungsListe(this, (RechnungsServer*)&LocalRFilter, &_module_broker);
      LocalRechnungsListe.Execute();
   } // if
}


void TSDIDecFrame::CmLoadDoc()
{
  // INSERT>> Your code here.

  WinExec("explorer doc\\index.htm", SW_SHOW);
}


void TSDIDecFrame::CmPreferences()
{
  // INSERT>> Your code here.

  DialogCustomizer LocalCustomizer(this, &_module_broker);
  LocalCustomizer.Execute();
}


void TSDIDecFrame::CmMerkzettel()
{
  // INSERT>> Your code here.

  _p_merkzettel_verwaltung->New();


}


void TSDIDecFrame::SetupWindow()
{
  TDecoratedFrame::SetupWindow();

  // INSERT>> Your code here.

  _p_merkzettel_verwaltung = new MerkzettelVerwaltung("daten/merkzettel.dat");
  _p_merkzettel_verwaltung->SetWindowPtr(this);
}


void TSDIDecFrame::CmRecallReByNumber()
{
	// INSERT>> Your code here.

   RechnungsNummer LocalReNumber(this, &_module_broker);

   LocalReNumber.Execute();
}


void TSDIDecFrame::CmPreisExport()
{
	// INSERT>> Your code here.

   TOpenSaveDialog::TData FileData((uint32)OFN_NOTESTFILECREATE|OFN_HIDEREADONLY, (char*)"Textdatei (*.TXT)|",
                                  (char*)"*.txt", (char*)"", (char*)"txt");

//   FileData.Flags = FileData.Flags & (~OFN_EXPLORER);

   sprintf(FileData.FileName, "preislisten.txt");

   if(TFileSaveDialog(this, FileData, 0, "Preislisten exportieren").Execute() == IDOK)
   {
//      char fileTitle[MAXPATH];
//      TOpenSaveDialog::GetFileTitle(FileData.FileName, fileTitle, MAXPATH);

		_module_broker.GetPreisVerwaltungPtr()->ExportPreislisten(FileData.FileName, &_module_broker);
   } // if
}

void TSDIDecFrame::EvSetFocus(HWND hWndLostFocus)
{
    TDecoratedFrame::EvSetFocus(hWndLostFocus);

	// INSERT>> Your code here.
}

bool TSDIDecFrame::ShowWindow(int cmdShow)
{
  bool result;

  result = TDecoratedFrame::ShowWindow(cmdShow);

  // INSERT>> Your code here.

   DisplayEvent LocalDisplay(this);

   LocalDisplay.Create();
   LocalDisplay.Show(SW_NORMAL);

   _p_kunden_verwaltung = new KundenVerwaltung("daten/kunden.dat");
	_module_broker.SetKundenVerwaltungPtr(_p_kunden_verwaltung);
	_p_kunden_verwaltung->SetLogBuchPtr(&_log_buch);



  return result;
}

