//----------------------------------------------------------------------------
//  Project weinadmin
//
//  Copyright  1996. Alle Rechte vorbehalten.
//
//  SUBSYSTEM:    weinadmin.apx Application
//  FILE:         weinliste.cpp
//  AUTHOR:       Thomas Haenselmann
//                                                                                            
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of WeinListe (TDialog).
//
//----------------------------------------------------------------------------

/*

- Im Posten-Editor sollte besser der Preis des Postens, als der des Weines eingetragen werden, wenn mglich

*/

#include <owl/pch.h>
#include <stdio.h>

#include "weinadminapp.h"
#include "weinliste.h"
#include "druckdialog.h"
#include "rechnungsliste.h"

#include <pbk/wein/weinlistenverwaltung.h>
#include <pbk/preis/preisverwaltung.h>
#include <pbk/rechnung/rechnungsverwaltung.h>

//
// Build a response table for all messages/commands handled by the application.
//
DEFINE_RESPONSE_TABLE1(WeinListe, TDialog)
//{{WeinListeRSP_TBL_BEGIN}}
  EV_COMMAND(IDC_STORE, CmStore),
  EV_LVN_GETDISPINFO(IDC_WEINTABELLE, LVNGetdispinfo),
  EV_LVN_ITEMCHANGING(IDC_WEINTABELLE, LVNItemchanging),
  EV_EN_CHANGE(IDC_WEINNUMMER, ENChange),
  EV_EN_CHANGE(IDC_WEINBESTELLMENGE, CmDataChange),
  EV_CBN_SELENDOK(IDC_PREISLISTE, CBNSelChange),
  EV_BN_CLICKED(IDC_FULLLIST, BNClicked),
  EV_BN_CLICKED(IDOK, CmOk),
  EV_BN_CLICKED(IDC_STORE, CmStore),
  EV_BN_CLICKED(IDC_STORERECHNUNG, CmStoreRechnung),
  EV_BN_CLICKED(IDC_CLEAR, CmClearPosten),
  EV_BN_CLICKED(IDC_HEUTE, CmHeute),
  EV_BN_CLICKED(IDC_LEER, CmLeer),
//{{WeinListeRSP_TBL_END}}
END_RESPONSE_TABLE;


//{{WeinListe Implementation}}


static WeinListeXfer WeinListeData;

WeinListe::WeinListe(TWindow* parent, ModuleBroker* pModuleBroker, Mode mode,
                     Kunde* pKunde, Rechnung* pRechnung, bool full_list, TResId resId, TModule* module)
: TDialog(parent, resId, module), _p_module_broker(pModuleBroker)
{
//{{WeinListeXFER_USE}}
  pBestand = new TEdit(this, IDC_WEINBESTAND, 7);
  pBestellmenge = new TEdit(this, IDC_WEINBESTELLMENGE, 6);
  pFuellmenge = new TEdit(this, IDC_WEINFUELLMENGE, WS_SIZE_FUELLMENGE);
  pGewicht = new TEdit(this, IDC_WEINGEWICHT, 7);
  pName = new TEdit(this, IDC_WEINNAME, WS_SIZE_NAME);
  pNummer = new TEdit(this, IDC_WEINNUMMER, 5);
  pPreis = new TEdit(this, IDC_WEINPREIS, 7);
  pPreisEU = new TEdit(this, IDC_WEINPREISEU, 7);
  pTabelle = new TListWindow(this, IDC_WEINTABELLE);
  pBezeichnung = new TEdit(this, IDC_WEINBEZ, WS_SIZE_BEZEICHNUNG);
  pFullList = new TCheckBox(this, IDC_FULLLIST, 0);
  pKartons = new TEdit(this, IDC_KARTONS, 5);
  pRechnungsNr = new TEdit(this, IDC_RECHNUNGSNR, 7);
  pSteigen = new TEdit(this, IDC_STEIGEN, 5);
  pBetragGesamt = new TEdit(this, IDC_BETRAG, 8);
  pBetragEingegangen = new TEdit(this, IDC_BETRAGEINGEG, 8);
  pDatumJahr = new TEdit(this, IDC_DATUMJAHR, 5);
  pDatumMonat = new TEdit(this, IDC_DATUMMONAT, 3);
  pDatumTag = new TEdit(this, IDC_DATUMTAG, 3);
  pBemerkung = new TEdit(this, IDC_EDIT11, RE_BEMERKUNG_SIZE);
  pBeglichen = new TCheckBox(this, IDC_BEGLICHEN, 0);
  pStorePosten = new TButton(this, IDC_STORE);
  pStoreRechnung = new TButton(this, IDC_STORERECHNUNG);
  pOk = new TButton(this, IDOK);
  pClearPosten = new TButton(this, IDC_CLEAR);
  pZurueck = new TButton(this, IDCANCEL);
  pHeute = new TButton(this, IDC_HEUTE);
  pLeer = new TButton(this, IDC_LEER);
  pTextRechnungsNr = new TStatic(this, IDC_WL_TEXT_RECHNUNGSNR, 255);
  pTextBemerkung = new TStatic(this, IDC_WL_TEXT_BEMERKUNG, 255);
  pTextBestellmenge = new TStatic(this, IDC_WL_TEXT_BESTELLMENGE, 255);
  pTextBetrag = new TStatic(this, IDC_WL_TEXT_BETRAG, 255);
  pTextDatum = new TStatic(this, IDC_WL_TEXT_DATUM, 255);
  pTextPreis = new TStatic(this, IDC_WL_TEXT_PREIS, 255);
  pTextPreisliste = new TStatic(this, IDC_WL_TEXT_PREISLISTE, 255);
  pTextSteigen = new TStatic(this, IDC_WL_TEXT_STEIGEN, 255);
  pBmpPreisListe = new TStatic(this, IDC_BMP_PREISLISTE, 255);
  pBmpWeinListe = new TStatic(this, IDC_BMP_WEINLISTE, 255);
  pBmpRechnungenEditieren = new TStatic(this, IDC_BMP_RECHNUNGENEDIT, 255);

  SetTransferBuffer(&WeinListeData);
//{{WeinListeXFER_USE_END}}

  // INSERT>> Your constructor code here.

  _p_current_rechnung = pRechnung;
  _p_current_kunde = pKunde;
  _mode = mode;
  _full_list = full_list;
  _neue_rechnung = false;

  switch(_mode)
  {
    case WEINLISTE:
      break;

    case PREISLISTE:
		pPreisliste = new TComboBox(this, IDC_PREISLISTE, 426, 216, 79, 120, WS_CHILD|WS_VISIBLE|WS_GROUP|WS_TABSTOP|WS_VSCROLL|CBS_DROPDOWNLIST, 28);
      _p_module_broker->GetPreisVerwaltungPtr()->SetPreisliste(0);
      break;

    case BESTELLUNG:
		pPreisliste = new TComboBox(this, IDC_PREISLISTE, 426, 216, 79, 120, WS_CHILD|WS_VISIBLE|WS_GROUP|WS_TABSTOP|WS_VSCROLL|CBS_DROPDOWNLIST, 28);
      _p_module_broker->GetPreisVerwaltungPtr()->SetPreisliste(0);

      if(!_p_current_rechnung)
      {
        _p_current_rechnung = _p_module_broker->GetRechnungsVerwaltungPtr()->AllocRechnung(_p_current_kunde->GetNr());
        _neue_rechnung = true;
      } // if
      else
        if(pRechnung) _p_current_rechnung = pRechnung;
      break;
  } // switch
}


WeinListe::~WeinListe()
{
  Destroy(IDCANCEL);

  // INSERT>> Your destructor code here.

   switch(_mode)
  {
    case BESTELLUNG:
		if((_p_current_rechnung) && (_neue_rechnung))
	      _p_module_broker->GetRechnungsVerwaltungPtr()->StoreRechnung();
      break;
  } // switch
}


void WeinListe::SetupWindow()
{
  TDialog::SetupWindow();

  // INSERT>> Your code here.

  char text[256];
  int  i;
  bool DM;

  DM = !_p_module_broker->GetCustomizerPtr()->GetEuroAktiv();

  pTabelle->InsertColumn(0, TListWindColumn("Nr.", 45));
  pTabelle->InsertColumn(1, TListWindColumn("Name", 210));
  pTabelle->InsertColumn(2, TListWindColumn("Bezeichnung", 210));
  pTabelle->InsertColumn(3, TListWindColumn("Fllmenge", 60));
  pTabelle->InsertColumn(4, TListWindColumn("Bestand", 60));
  pTabelle->InsertColumn(5, TListWindColumn("Gewicht", 60));

  pBmpPreisListe->Show(SW_HIDE);
  pBmpWeinListe->Show(SW_HIDE);
  pBmpRechnungenEditieren->Show(SW_HIDE);

  if(_full_list) pFullList->SetCheck(BF_CHECKED);
  else pFullList->SetCheck(BF_UNCHECKED);

  switch(_mode)
  {
    case WEINLISTE:
		pTextRechnungsNr->Show(SW_HIDE);
  		pTextBemerkung->Show(SW_HIDE);
  		pTextBestellmenge->Show(SW_HIDE);
  		pTextBetrag->Show(SW_HIDE);
  		pTextDatum->Show(SW_HIDE);
  		pTextPreis->Show(SW_HIDE);
  		pTextPreisliste->Show(SW_HIDE);
  		pTextSteigen->Show(SW_HIDE);

		pFullList->Show(SW_HIDE);
		pKartons->Show(SW_HIDE);
		pRechnungsNr->Show(SW_HIDE);
		pSteigen->Show(SW_HIDE);
		pBetragGesamt->Show(SW_HIDE);
		pBetragEingegangen->Show(SW_HIDE);
		pDatumJahr->Show(SW_HIDE);
		pDatumMonat->Show(SW_HIDE);
		pDatumTag->Show(SW_HIDE);
		pBemerkung->Show(SW_HIDE);
		pBeglichen->Show(SW_HIDE);
		pStoreRechnung->Show(SW_HIDE);
		pZurueck->Show(SW_HIDE);
		pHeute->Show(SW_HIDE);
		pLeer->Show(SW_HIDE);
		pStorePosten->SetCaption("Wein speichern");
		pClearPosten->SetCaption("Wein verbergen");

		pPreis->Show(SW_HIDE);
		pPreisEU->Show(SW_HIDE);
      pBestellmenge->Show(SW_HIDE);
	   pFullList->Show(SW_HIDE);
  	   pKartons->Show(SW_HIDE);
	   pRechnungsNr->Show(SW_HIDE);
	   pSteigen->Show(SW_HIDE);

		pTabelle->InsertColumn(6, TListWindColumn("", 170, TListWindColumn::Right)); // nur zum Auffllen
		pBmpWeinListe->Show(SW_SHOW);
      break;

    case PREISLISTE:
		pTextRechnungsNr->Show(SW_HIDE);
  		pTextBemerkung->Show(SW_HIDE);
  		pTextBestellmenge->Show(SW_HIDE);
  		pTextBetrag->Show(SW_HIDE);
  		pTextDatum->Show(SW_HIDE);
  		pTextSteigen->Show(SW_HIDE);

		pFullList->Show(SW_HIDE);
		pKartons->Show(SW_HIDE);
		pRechnungsNr->Show(SW_HIDE);
		pSteigen->Show(SW_HIDE);
		pBetragGesamt->Show(SW_HIDE);
		pBetragEingegangen->Show(SW_HIDE);
		pDatumJahr->Show(SW_HIDE);
		pDatumMonat->Show(SW_HIDE);
		pDatumTag->Show(SW_HIDE);
		pBemerkung->Show(SW_HIDE);
		pBeglichen->Show(SW_HIDE);
		pStoreRechnung->Show(SW_HIDE);
		pZurueck->Show(SW_HIDE);
		pHeute->Show(SW_HIDE);
		pLeer->Show(SW_HIDE);
		pClearPosten->Show(SW_HIDE);

      pBestellmenge->Show(SW_HIDE);
	   pFullList->Show(SW_HIDE);
  	   pKartons->Show(SW_HIDE);
	   pRechnungsNr->Show(SW_HIDE);
	   pSteigen->Show(SW_HIDE);

		pTabelle->SetColumnWidth(5, 0);
		pTabelle->InsertColumn(6, TListWindColumn("Preis DM", 70, TListWindColumn::Right));
		pTabelle->InsertColumn(7, TListWindColumn("Preis Euro", 70, TListWindColumn::Right));

      pBestand->EnableWindow(false);
      pFuellmenge->EnableWindow(false);
      pGewicht->EnableWindow(false);
      pName->EnableWindow(false);
		pBezeichnung->EnableWindow(false);

      for(i = 0; i < _p_module_broker->GetPreisVerwaltungPtr()->GetNoPreisliste(); i++)
	      pPreisliste->InsertString(_p_module_broker->GetPreisVerwaltungPtr()->GetPreisListeName(i), i);

		pPreisliste->SetSelIndex(0);
		pBmpPreisListe->Show(SW_SHOW);
      break;

    case BESTELLUNG:
		pTabelle->SetColumnWidth(3, 0);
		pTabelle->SetColumnWidth(4, 0);
		pTabelle->SetColumnWidth(5, 0);
		pTabelle->InsertColumn(6, TListWindColumn("Preis DM", 70, TListWindColumn::Right));
		pTabelle->InsertColumn(7, TListWindColumn("Preis Euro", 70, TListWindColumn::Right));
		pTabelle->InsertColumn(8, TListWindColumn("Best.Menge", 80, TListWindColumn::Right));

      pFullList->SetCheck(BF_CHECKED);
      if(_p_current_rechnung)
      {
         sprintf(text, "%d", _p_current_rechnung->GetNrExtern());
         pRechnungsNr->SetText(text);

			sprintf(text, "%d", _p_current_rechnung->GetKisten());
         pKartons->SetText(text);

			sprintf(text, "%d", _p_current_rechnung->GetKartons());
         pSteigen->SetText(text);

			sprintf(text, "%0.2f", _p_current_rechnung->GetBetragEingegangen());
         pBetragEingegangen->SetText(text);

			sprintf(text, "%0.2f", _p_current_rechnung->GetBetragGesamt(DM));
         pBetragGesamt->SetText(text);

			if(_p_current_rechnung->GetDatum().Year() > 1970)
         {
            sprintf(text, "%d", _p_current_rechnung->GetDatum().DayOfMonth());
            pDatumTag->SetText(text);

            sprintf(text, "%d", _p_current_rechnung->GetDatum().Month());
            pDatumMonat->SetText(text);

            sprintf(text, "%d", _p_current_rechnung->GetDatum().Year());
            pDatumJahr->SetText(text);
         } // if
         else
         {
            pDatumTag->SetText("");
            pDatumMonat->SetText("");
            pDatumJahr->SetText("");
         } // else

			UpdateBemerkung();

         if(_p_current_rechnung->GetBeglichen()) pBeglichen->SetCheck(BF_CHECKED);
         else pBeglichen->SetCheck(BF_UNCHECKED);
      } // if

      for(i = 0; i < _p_module_broker->GetPreisVerwaltungPtr()->GetNoPreisliste(); i++)
	      pPreisliste->InsertString(_p_module_broker->GetPreisVerwaltungPtr()->GetPreisListeName(i), i);

		if(!_neue_rechnung)
		{
      	pOk->Show(SW_HIDE);
      } // if
      else
		{
      	pZurueck->Show(SW_HIDE);
         pStoreRechnung->Show(SW_HIDE);
      } // else

      // Die Preisliste des Kunden mu automatisch gesetzt werden

      if(_p_current_kunde)
		{
         if(_p_current_kunde->GetPreisliste() < 0)
         {
         	MessageBox("Dem Kunden wurde bisher noch keine Preisliste zugeordnet. Bitten whlen Sie die passende Liste nun sofort.", "Rechnung erstellen", MB_OK);
            pPreisliste->SetSelIndex(0);
         } // if
         else pPreisliste->SetSelIndex(_p_current_kunde->GetPreisliste());

			UpdateBemerkung();
      } // if

      pBestand->EnableWindow(false);
      pFuellmenge->EnableWindow(false);
      pGewicht->EnableWindow(false);
      pName->EnableWindow(false);
		pBezeichnung->EnableWindow(false);
      pPreis->EnableWindow(false);
      pPreisEU->EnableWindow(false);

 	   pBmpRechnungenEditieren->Show(SW_SHOW);
      pNummer->SetFocus();
      break;
  } // switch

  FillTabelle();
}


void WeinListe::ENChange()
{
  // INSERT>> Your code here.

  PostenVerwaltung::Posten* pPosten;
  long  						    wein_nummer;
  char  							 text[256];
  Wein*							 pWein;
  Preis*						    pTmpPreis;

  // Die Weinnummer hat sich gendert

  pNummer->GetText(text, 255);
  wein_nummer = atoi(text);

  if((pWein = (*_p_module_broker->GetWeinlistenVerwaltungPtr())[wein_nummer]))
  {
    pName->SetText(pWein->GetName());
    pBezeichnung->SetText(pWein->GetBezeichnung());
    pFuellmenge->SetText(pWein->GetFuellmenge());
    sprintf(text, "%d", pWein->GetGewicht());
    pGewicht->SetText(text);
    sprintf(text, "%d", pWein->GetBestand());
    pBestand->SetText(text);

//    if((_mode == BESTELLUNG) && (!pWein->GetVersteckt())) pBestellmenge->SetFocus();

	 switch(_mode)
     {
       case BESTELLUNG:
       case PREISLISTE:
         pTmpPreis = (*_p_module_broker->GetPreisVerwaltungPtr())[wein_nummer];
         if(pTmpPreis)
         {
           sprintf(text, "%0.2f", pTmpPreis->GetDMPreis());
           pPreis->SetText(text);
           sprintf(text, "%0.2f", pTmpPreis->GetEuroPreis());
           pPreisEU->SetText(text);
           if(_p_current_rechnung)
           {
              pPosten = (*_p_current_rechnung)[pWein->GetNr()];
              if(pPosten) sprintf(text, "%d", pPosten->_anzahl);
              else sprintf(text, "");
              pBestellmenge->SetText(text);
           } // if
         } // if
         else
         {
           pPreis->Clear();
           pPreisEU->Clear();
         } // else
         break;
     } // switch
	} // if
   else
   {
      pName->Clear();
      pBezeichnung->Clear();
      pFuellmenge->Clear();
      pGewicht->Clear();
      pBestand->Clear();
      pPreis->Clear();
      pPreisEU->Clear();
		pBestellmenge->Clear();
   } // else
}

void WeinListe::CmDataChange()
{
	_dirty = true;
}


void WeinListe::CBNSelChange()
{
	int index;

	index = pPreisliste->GetSelIndex();
   if(index >= 0) _p_module_broker->GetPreisVerwaltungPtr()->SetPreisliste(index);

	if(_mode == BESTELLUNG)
	{
   	UpdateBemerkung();
	   MessageBox("HINWEIS: Die Bemerkung der Rechnung wurde automatisch verndert.", "Preisliste neu gewhlt", MB_OK);
   } // if

   FillTabelle();
} // WeinListe::CBNSelChange

void WeinListe::CmStore()
{
  Wein  tmp_wein;
  Preis *pTmpPreis;
  char  text[256];
  long  wein_nummer, aktueller_bestand;
  short Bestellmenge;
  float DMPreis, EuroPreis;
  bool  DM;

  DM = !_p_module_broker->GetCustomizerPtr()->GetEuroAktiv();

  pName->GetText(text, 255);
  if(strcmp(text, "import weine") == 0)
  {
    if(MessageBox("Soll die alte Wein- und Preisliste importiert werden? VORSICHT: Dabei gehen alle bisher eingegebenen Weine verloren!", "WeinListe::CmStore()", MB_YESNO) == IDYES)
    {
		SetCursor(GetModule(), IDC_WAIT);
      _p_module_broker->GetWeinlistenVerwaltungPtr()->ReadPascalFile();
		_p_module_broker->GetPreisVerwaltungPtr()->ReadPascalFile(_p_module_broker);
		SetCursor(GetModule(), IDC_ARROW);
      MessageBox("Die alte Weinliste wurde importiert", "WeinListe::CmStore()", MB_OK);
    } // if
    return;
  } // if

  if(strcmp(text, "generate missing data") == 0)
  {
    if(MessageBox("Soll die alte Wein- und Preisliste verndert werden? VORSICHT: Dabei gehen teile der bisher eingegebenen Daten verloren!", "WeinListe::CmStore()", MB_YESNO) == IDYES)
    {
		SetCursor(GetModule(), IDC_WAIT);
      for(int i = 0; i < _p_module_broker->GetWeinlistenVerwaltungPtr()->GetWeinListe().GetNoElements(); i++)
		{
			_p_module_broker->GetWeinlistenVerwaltungPtr()->GetWeinListe()[i].SetBestand(rand() % 1000 + 100);
			_p_module_broker->GetWeinlistenVerwaltungPtr()->GetWeinListe()[i].SetGewicht(rand() % 2000 + 1000);
      } // for

		SetCursor(GetModule(), IDC_ARROW);
      MessageBox("Die alte Weinliste wurde importiert", "WeinListe::CmStore()", MB_OK);
      return;
    } // if
    return;
  } // if

  switch(_mode)
  {
     case WEINLISTE:
        pNummer->GetText(text, 255);
        tmp_wein.SetNr(atoi(text));

        pName->GetText(text, 255);
        tmp_wein.SetName(text);

        pBezeichnung->GetText(text, 255);
        tmp_wein.SetBezeichnung(text);

        pFuellmenge->GetText(text, 255);
        tmp_wein.SetFuellmenge(text);

        pGewicht->GetText(text, 255);
        tmp_wein.SetGewicht(atoi(text));

        pBestand->GetText(text, 255);
        tmp_wein.SetBestand(atoi(text));

        tmp_wein.SetVersteckt(false);

        if(!(*_p_module_broker->GetWeinlistenVerwaltungPtr())[tmp_wein.GetNr()])
          (*_p_module_broker->GetWeinlistenVerwaltungPtr()).Add(tmp_wein.GetNr());

        *(*_p_module_broker->GetWeinlistenVerwaltungPtr())[tmp_wein.GetNr()] = tmp_wein;
        _p_module_broker->GetWeinlistenVerwaltungPtr()->GetWeinListe().Sort();
        FillTabelle();
     	  break;

     case PREISLISTE:
        pNummer->GetText(text, 255);
        wein_nummer = atoi(text);
        pPreis->GetText(text, 255);
        DMPreis = atof(text);
        pPreisEU->GetText(text, 255);
        EuroPreis = atof(text);

        if(!(*_p_module_broker->GetPreisVerwaltungPtr())[wein_nummer])
          _p_module_broker->GetPreisVerwaltungPtr()->Add(Preis(wein_nummer));

        pTmpPreis = (*_p_module_broker->GetPreisVerwaltungPtr())[wein_nummer];
        pTmpPreis->SetDMPreis(DMPreis);
        pTmpPreis->SetEuroPreis(EuroPreis);

        _p_module_broker->GetPreisVerwaltungPtr()->SaveCurrentListe();

        pTabelle->Update(0);
        break;

     case BESTELLUNG:
        pNummer->GetText(text, 255);
        wein_nummer = atoi(text);
        pBestellmenge->GetText(text, 255);
        Bestellmenge = atoi(text);
        pPreis->GetText(text, 255);
        DMPreis = atof(text);
        pPreisEU->GetText(text, 255);
        EuroPreis = atof(text);

        if(!(*_p_module_broker->GetWeinlistenVerwaltungPtr())[wein_nummer])
          MessageBox("Ein Wein mit dieser Nummer konnte nicht gefunden werden.", "Posten speichern", MB_OK);
        else
           if(Bestellmenge > (aktueller_bestand = (*_p_module_broker->GetWeinlistenVerwaltungPtr())[wein_nummer]->GetBestand()))
           {
             sprintf(text, "Der Bestand betrgt nur noch %d Flaschen", aktueller_bestand);
             MessageBox(text, "Posten hinzufgen", MB_OK);
             sprintf(text, "%d", aktueller_bestand);
             pBestellmenge->SetText(text);
           } // if
           else
           {
				 if(Bestellmenge > 0)
             {
                _p_current_rechnung->AddPosten(wein_nummer, DMPreis, EuroPreis, Bestellmenge);
                if(_full_list) pTabelle->Update(0);
                else FillTabelle();

                sprintf(text, "%0.2f", _p_current_rechnung->GetBetragGesamt(DM));
                pBetragGesamt->SetText(text);
                pBestellmenge->Clear();
                pNummer->Clear();
                pNummer->SetFocus();
             } // if
             else
             {
             	// Wenn die Bestellmenge noch Null ist, kann man den Fokus
               // gleich auf das Feld pBestellmenge setzen

               pBestellmenge->SetFocus();
             } // else
           } // else
     	  break;
  } // switch

  _dirty = false;
}

void WeinListe::LVNGetdispinfo(TLwDispInfoNotify& lwdin)
{
  // INSERT>> Your code here.

  Preis* pTmpPreis;
  char   text[256];

  TListWindItem& dispInfoItem = *(TListWindItem*)&lwdin.item;
  pTabelle->GetItemText(dispInfoItem.GetIndex(), 0, text, 255);
  Wein* pWein = (*_p_module_broker->GetWeinlistenVerwaltungPtr())[atoi(text)];

  if(!pWein) throw(xmsg(string("Konnte einen Wein in WeinListe::LVNGetdispinfo() beim Erstellen der Tabelle nicht finden")));

  switch(dispInfoItem.GetSubItem())
  {
    case 1:
      if((dispInfoItem.GetIndex() % 2) == 0)
        pTabelle->SetTextBkColor(TColor(192, 255, 192));
      else
        pTabelle->SetTextBkColor(TColor(255, 255, 255));

      sprintf(text, "%s", pWein->GetName());
      break;

    case 2:
      sprintf(text, "%s", pWein->GetBezeichnung());
      break;

    case 3:
      sprintf(text, "%s", pWein->GetFuellmenge());
      break;

    case 4:
      sprintf(text, "%d", pWein->GetBestand());
      break;

    case 5:
      sprintf(text, "%d", pWein->GetGewicht());
      break;

    case 6:
		if(_mode == WEINLISTE) strcpy(text, "                   ");
      else
      {
	      pTmpPreis = (*_p_module_broker->GetPreisVerwaltungPtr())[pWein->GetNr()];
	      if(pTmpPreis) sprintf(text, "%0.2f", pTmpPreis->GetDMPreis());
   	   else sprintf(text, "N/A");
      } // if
      break;

    case 7:
      pTmpPreis = (*_p_module_broker->GetPreisVerwaltungPtr())[pWein->GetNr()];
      if(pTmpPreis) sprintf(text, "%0.2f", pTmpPreis->GetEuroPreis());
      else sprintf(text, "N/A");
      break;

    case 8:
    	if((*_p_current_rechnung)[pWein->GetNr()] != NULL)
      {
			sprintf(text, "%d", (*_p_current_rechnung)[pWein->GetNr()]->_anzahl);
      } // if
      else sprintf(text, "N/A");
    	break;
  } // switch

  dispInfoItem.SetText(text);
}

void WeinListe::FillTabelle()
{
  char  text[256];
  Wein* pWein;

  pTabelle->DeleteAllItems();

  for(int i = 0; i < _p_module_broker->GetWeinlistenVerwaltungPtr()->GetWeinListe().GetNoElements(); i++)
  {
	 pWein = &_p_module_broker->GetWeinlistenVerwaltungPtr()->GetWeinListe()[i];
	 if((_p_current_rechnung == NULL)  || (_full_list == true) || ((*_p_current_rechnung)[pWein->GetNr()] != NULL))
	 {
		if(((_p_current_rechnung) && ((*_p_current_rechnung)[pWein->GetNr()] != NULL)) || (!_p_module_broker->GetWeinlistenVerwaltungPtr()->GetWeinListe()[i].GetVersteckt()))
      {
			sprintf(text, "%d", _p_module_broker->GetWeinlistenVerwaltungPtr()->GetWeinListe()[i].GetNr());
		   TListWindItem item(text, 0);
		   pTabelle->AddItem(item);
      } // if
    } // if
  } // for

  _dirty = false;
}

bool WeinListe::LVNItemchanging(TLwNotify& lwn)
{
  // INSERT>> Your code here.

	if(_dirty)
	{
		if(MessageBox("Sie haben Daten in den Eingabefeldern verndert. Sollen diese unbercksichtigt bleiben?", "Neuen Tabelleneintrag whlen", MB_YESNO) == IDNO) return true;
	} // if

	char text[256];

	pTabelle->GetItemText(lwn.iItem, 0, text, 255);
	Wein* pWein = (*_p_module_broker->GetWeinlistenVerwaltungPtr())[atoi(text)];

	if(pWein)
	{
		sprintf(text, "%d", pWein->GetNr());
		pNummer->SetText(text);

		_dirty = false;
		return true;
	} // if

	return true;
}


void WeinListe::BNClicked()
{
  // INSERT>> Your code here.

  _full_list = !_full_list;
  FillTabelle();
}

void WeinListe::CmOk()
{
	CmStoreRechnung();
} // WeinListe::CmOk

void WeinListe::CmStoreRechnung()
{
  // INSERT>> Your code here.

	DayTy   tag;
   MonthTy monat;
   long    jahr;
   char    text[456];

	// Speichern der Rechnung

	if(_mode == BESTELLUNG)
	{
      pBemerkung->GetText(text, 255);
      _p_current_rechnung->SetBemerkung(text);

      if(strcmp(text, "generate rechnungen") == 0)
      {
      	if(MessageBox("Vorsicht: Sie sind im Begriff fr 2000 Kunden jeweils max. 20 Testrechnungen zu generieren. Ein nachtrgliches Bereinigen des Datenbestandes nach dieser Operation ist kaum noch mglich. Mchten Sie fortfahren?", "Automatisch Testrechnungen generieren", MB_YESNO) == IDYES)
         {
				SetCursor(0, IDC_WAIT);
            _p_module_broker->GetRechnungsVerwaltungPtr()->StoreRechnung();

            for(long i_kunde = 0; i_kunde < 2000; i_kunde++)
            {
					sprintf(text, "%d", i_kunde);
					SetCaption(text);

					for(long i_rechnung = 0; i_rechnung < (rand() % 24); i_rechnung++)
					{
						Rechnung* pRechnung = _p_module_broker->GetRechnungsVerwaltungPtr()->AllocRechnung(rand() % 1200);
	               pRechnung->SetDatum(TDate(rand() % 29, rand() % 12, (rand() % 10) + 1990));
	               pRechnung->SetErfassungDatum(pRechnung->GetDatum());

						for(long i_posten = 0; i_posten < (rand() % 30); i_posten++)
                  {
							pRechnung->AddPosten((_p_module_broker->GetWeinlistenVerwaltungPtr()->GetWeinListe())[i_posten].GetNr(), (float)(rand() % 24), (float)(rand() % 12), rand() % 12);
                  } // for

	               _p_module_broker->GetRechnungsVerwaltungPtr()->StoreRechnung();
               } // for
            } // for

				SetCursor(0, IDC_ARROW);
         } // if
			Destroy();
         return;
      } // if

      pSteigen->GetText(text, 255);
      _p_current_rechnung->SetKartons(atoi(text));

      pKartons->GetText(text, 255);
      _p_current_rechnung->SetKisten(atoi(text));

      pRechnungsNr->GetText(text, 255);
      _p_current_rechnung->SetNrExtern(atoi(text));

      pBetragEingegangen->GetText(text, 255);
      _p_current_rechnung->SetBetragEingegangen((float)atof(text));

      pDatumTag->GetText(text, 255);
      tag = atoi(text);

      pDatumMonat->GetText(text, 255);
      monat = atoi(text);

      pDatumJahr->GetText(text, 255);
      jahr = atoi(text); if(jahr < 1901) jahr = 1960;

      TDate datum_neu(tag, monat, jahr);

      if((jahr > 1970) && ((int)(_p_current_rechnung->GetErfassungDatum() - datum_neu) > _p_module_broker->GetCustomizerPtr()->GetVordatierung()))
      {
			sprintf(text, "Diese Rechnung wurde am %d.%d.%d erstellt. Sie haben versucht die Rechnung mehr als %d Tage vorzudatieren. Damit ist die Grenze zur Vordatierung berschritten. Wenn dies ein Problem ist, kann die Grenze in den 'Grundeinstellungen' bel. verndert werden. WICHTIG: Je grer die Vordatierungsgrenze (in Tagen) gewhlt wird, desto langsamer werden einige Programmfunktionen. Vorerst wurde das Datum der Erfassung eingesetzt.", _p_current_rechnung->GetErfassungDatum().DayOfMonth(), _p_current_rechnung->GetErfassungDatum().Month(), _p_current_rechnung->GetErfassungDatum().Year(), _p_module_broker->GetCustomizerPtr()->GetVordatierung());
			MessageBox(text, "Rechnungsdaten ndern", MB_OK);
			_p_current_rechnung->SetDatum(_p_current_rechnung->GetErfassungDatum());
      } // if
		else _p_current_rechnung->SetDatum(datum_neu);

      if(pBeglichen->GetCheck() == BF_CHECKED) _p_current_rechnung->SetBeglichen(true);
      if(pBeglichen->GetCheck() == BF_UNCHECKED) _p_current_rechnung->SetBeglichen(false);

		long re_nr = _p_current_rechnung->GetNr();

		if(_neue_rechnung) _p_module_broker->GetRechnungsVerwaltungPtr()->StoreRechnung();
      else _p_module_broker->GetRechnungsVerwaltungPtr()->UpdateRechnung(_p_current_rechnung);

      if(MessageBox("Soll die Rechnung gleich gedruckt werden?", "Rechnung drucken", MB_YESNO) == IDYES)
      {
			bool output_packliste, output_rechnung, output_rechnung_intern, pre_view;

         output_packliste = output_rechnung = pre_view = false;

         DruckDialog LocalDruckDialog(this, output_packliste, output_rechnung, output_rechnung_intern, pre_view);
         LocalDruckDialog.Execute();

			RechnungsListe::CmSysDrucken(GetApplication()->GetMainWindow(), _p_module_broker, (*_p_module_broker->GetRechnungsVerwaltungPtr())[re_nr], output_packliste, output_rechnung, output_rechnung_intern, pre_view);
      } // if

	   _p_current_rechnung = NULL;
   } // if

   TDialog::CmOk();
}


void WeinListe::CmClearPosten()
{
  // INSERT>> Your code here.

  char 			 text[256];
  unsigned long wein_nummer;

  switch(_mode)
  {
    case WEINLISTE:
	   pNummer->GetText(text, 255);
      wein_nummer = atoi(text);

		if((*_p_module_broker->GetWeinlistenVerwaltungPtr())[wein_nummer])
		{
	      (*_p_module_broker->GetWeinlistenVerwaltungPtr())[wein_nummer]->SetVersteckt(true);
	      FillTabelle();
      } // if
      break;

    case BESTELLUNG:
      if(_p_current_rechnung)
      {
        // der Posten soll aus der Rechnung gelscht werden

        pNummer->GetText(text, 255);
        wein_nummer = atoi(text);

        _p_current_rechnung->ClearPosten(wein_nummer);

        if(_full_list) pTabelle->Update(0);
        else FillTabelle();

        pNummer->Clear();
        pNummer->SetFocus();
      } // if
      break;
  } // switch
}


void WeinListe::CmHeute()
{
  // INSERT>> Your code here.

  TDate Heute;
  char  text[10];

  sprintf(text, "%d", Heute.DayOfMonth());
  pDatumTag->SetText(text);
  sprintf(text, "%d", Heute.Month());
  pDatumMonat->SetText(text);
  sprintf(text, "%d", Heute.Year());
  pDatumJahr->SetText(text);
}


void WeinListe::CmLeer()
{
  // INSERT>> Your code here.

  pDatumTag->Clear();
  pDatumMonat->Clear();
  pDatumJahr->Clear();
}

bool WeinListe::CanClose()
{
  bool result;

  result = TDialog::CanClose();

  // INSERT>> Your code here.

  return result;
}

void WeinListe::UpdateBemerkung()
{
	char text[900], add_on_text[100];

	strcpy(add_on_text, "");

   if(!_neue_rechnung) strcpy(text, _p_current_rechnung->GetBemerkung());
   else
	{
   	switch(pPreisliste->GetSelIndex())
      {
      	case 0:
				strcpy(add_on_text, "LKW / NO");
         	break;

      	case 1:
				strcpy(add_on_text, "LKW / NO/S");
         	break;

      	case 2:
         	break;

      	case 3:
				strcpy(add_on_text, "Abholung");
         	break;
      } // switch

		sprintf(text, "%s %s", _p_module_broker->GetCustomizerPtr()->GetRechnungBemerkung(), add_on_text);
   } // else

	pBemerkung->SetText(text);
} // WeinListe::UpdateBemerkung
