#ifndef _Kunde_h
#define _Kunde_h

#include <fstream.h>
#include <values.h>
#include <classlib/date.h>
#include <classlib/time.h>

#include <list/list.h>

#define KS_SIZE_ANREDE     31
#define KS_SIZE_VORNAME    31
#define KS_SIZE_NACHNAME   31
#define KS_SIZE_STRASSE	   31
#define KS_SIZE_PLZ        sizeof(unsigned long)
#define KS_SIZE_ORT        31
#define KS_SIZE_TELEFON    21
#define KS_SIZE_FAX        21
#define KS_SIZE_EMAIL      51
#define KS_SIZE_BEMERKUNG 256

class Kunde
{
	protected:
      unsigned long _nr;
      char 			  _anrede[KS_SIZE_ANREDE], _vorname[KS_SIZE_VORNAME], _nachname[KS_SIZE_NACHNAME];
      char 			  _strasse[KS_SIZE_STRASSE];
      unsigned long _plz;
      char 			  _ort[KS_SIZE_ORT];
      char 			  _telefon[KS_SIZE_TELEFON], _fax[KS_SIZE_FAX], _email[KS_SIZE_EMAIL];
      char 			  _bemerkung[KS_SIZE_BEMERKUNG];
      short			  _preisliste_index;
      short			  _geogruppe_index;
      TDate			  _erfassung_datum;
      TTime			  _erfassung_zeit;
      long			  _last_rechnung_index;

	public:
   	Kunde(const unsigned long = MAXLONG);
      virtual ~Kunde();

      Kunde& operator=(const Kunde&);

      unsigned long GetNr() const {return _nr;};
      const char*   GetAnrede() const {return _anrede;};
      const char*   GetVorname() const {return _vorname;};
      const char*   GetNachname() const {return _nachname;};
      const char*   GetStrasse() const {return _strasse;};
      unsigned long GetPLZ() const {return _plz;};
      const char*   GetOrt() const {return _ort;};
      const char*   GetTelefon() const {return _telefon;};
      const char*   GetFax() const {return _fax;};
      const char*   GetEMail() const {return _email;};
      const char*   GetBemerkung() const {return _bemerkung;};
		short			  GetPreisliste() const {return _preisliste_index;};
		short			  GetGeoGruppe() const {return _geogruppe_index;};
      const TDate&  GetErfassungDatum() const {return _erfassung_datum;};
      const TTime&  GetErfassungZeit() const {return _erfassung_zeit;};
      bool			  IsValid() const {return _nachname[0] != 0;};
      long			  GetLastRechnungIndex() {return _last_rechnung_index;};

      void 			  SetNr(unsigned long nr) {_nr = nr;};
      void  		  SetAnrede(const char const* pText) {strcpy(_anrede, pText);};
      void  		  SetVorname(const char const* pText) {strcpy(_vorname, pText);};
      void  		  SetNachname(const char const* pText) {strcpy(_nachname, pText);};
      void  		  SetStrasse(const char const* pText) {strcpy(_strasse, pText);};
      void 			  SetPLZ(const unsigned long plz) {_plz = plz;};
      void  		  SetOrt(const char const* pText) {strcpy(_ort, pText);};
      void  		  SetTelefon(const char const* pText) {strcpy(_telefon, pText);};
      void  		  SetFax(const char const* pText) {strcpy(_fax, pText);};
      void  		  SetEMail(const char const* pText) {strcpy(_email, pText);};
      void  		  SetBemerkung(const char const* pText) {strcpy(_bemerkung, pText);};
		void 			  SetPreisliste(short preisliste_index) {_preisliste_index = preisliste_index;};
		void 			  SetGeoGruppe(short geogruppe_index) {_geogruppe_index = geogruppe_index;};
		void 			  SetLastRechnungIndex(long last_rechnung_index) {_last_rechnung_index = last_rechnung_index;};

      bool 			  DoesMatch(const Kunde&);

      void Init();
      void Read(fstream&);
      void Write(fstream&) const;

      static size_t SizeOf() {return sizeof(Kunde);};
}; // class Kunde

// Klassen, die von KundenServer erben, knnen Kundenlisten fllen

class KundenServer
{
	public:
   	virtual void FillKundenListe(GenericList<Kunde>&) = 0;
}; // class KundenServer
#endif
