/*

- GetNextKundenNr() ist noch ein dummy
- Was passiert mit den Rechnungen eines gelschten Kunden?

*/

#include <pbk/kunde/kundenverwaltung.h>
#include <stdio.h>
#include <cstring.h>
#include <fstream.h>

unsigned long KundenVerwaltung::_last_kunden_nr = 0;

KundenVerwaltung::KundenVerwaltung(const char* filename) : _kunden_liste(filename)
{
   _kunde_found_index = -1;
   _p_log_buch = NULL;

   for(long i = 0; i < _kunden_liste.GetNoElements(); i++)
   	if(_kunden_liste[i].GetNr() > _last_kunden_nr) _last_kunden_nr = _kunden_liste[i].GetNr();
} // KundenVerwaltung::KundenVerwaltung

KundenVerwaltung::~KundenVerwaltung()
{
} // KundenVerwaltung::~KundenVerwaltung

Kunde* KundenVerwaltung::operator[](const unsigned long kunde_nr)
{
	for(int i = 0; i < _kunden_liste.GetNoElements(); i++)
		if(_kunden_liste[i].GetNr() == kunde_nr) return &_kunden_liste[i];

   return NULL;
} // KundenVerwaltung::operator[]

void KundenVerwaltung::Add(Kunde& kunde) throw(xmsg)
{
	kunde.SetNr(GetNextKundenNr());
	_kunden_liste.Add(kunde);
} // KundenVerwaltung::Add

void KundenVerwaltung::Remove(const unsigned long kunde_nr)
{
	for(int i = 0; i < _kunden_liste.GetNoElements(); i++)
		if(_kunden_liste[i].GetNr() == kunde_nr)
      {
			// Der Kunde soll vor dem Lschen noch in die Log-Datei eingetragen werden

         if(_p_log_buch)
         {
         	fstream& stream = _p_log_buch->GetOutputStream();
            Kunde& kunde = _kunden_liste[i];
            TDate erf_dat = kunde.GetErfassungDatum();
            TTime erf_zei = kunde.GetErfassungZeit();

            stream << ">> KUNDE GELSCHT << (erfasst am " << erf_dat.DayOfMonth() << "." << erf_dat.Month() << "." << erf_dat.Year() << " " << erf_zei.Hour() << ":" << erf_zei.Minute() << ":" << erf_zei.Second() << ") -> ";
				stream << kunde.GetNr() << ",\042" << kunde.GetAnrede() << "\042,\042" << kunde.GetVorname() << "\042,\042" <<
            kunde.GetNachname() << "\042,\042" << kunde.GetStrasse() << "\042," << kunde.GetPLZ() << ",\042" << kunde.GetOrt() << "\042,\042" <<
            kunde.GetTelefon() << "\042,\042" << kunde.GetFax() << "\042,\042" << kunde.GetEMail() << "\042,\042" << kunde.GetBemerkung() << "\042," <<
            kunde.GetPreisliste() << "," << kunde.GetGeoGruppe() << endl;
         } // if

			_kunden_liste.Remove(i); // an dieser Stelle wird das letzte Element eingefgt
			break;
      } // if
} // KundenVerwaltung::Remove

void KundenVerwaltung::Update(const Kunde& para_kunde)
{
   Kunde* kunde;

	kunde = (*this)[para_kunde.GetNr()];

   // Der Zustand vor der Vernderung soll festgehalten werden

   if(_p_log_buch)
   {
      fstream& stream = _p_log_buch->GetOutputStream();
      TDate erf_dat = kunde->GetErfassungDatum();
      TTime erf_zei = kunde->GetErfassungZeit();

      stream << ">> KUNDE GENDERT << (erfasst am " << erf_dat.DayOfMonth() << "." << erf_dat.Month() << "." << erf_dat.Year() << " " << erf_zei.Hour() << ":" << erf_zei.Minute() << ":" << erf_zei.Second() << ") -> ";
      stream << kunde->GetNr() << ",\042" << kunde->GetAnrede() << "\042,\042" << kunde->GetVorname() << "\042,\042" <<
      kunde->GetNachname() << "\042,\042" << kunde->GetStrasse() << "\042," << kunde->GetPLZ() << ",\042" << kunde->GetOrt() << "\042,\042" <<
      kunde->GetTelefon() << "\042,\042" << kunde->GetFax() << "\042,\042" << kunde->GetEMail() << "\042,\042" << kunde->GetBemerkung() << "\042," <<
      kunde->GetPreisliste() << "," << kunde->GetGeoGruppe() << endl;
   } // if

   *kunde = para_kunde;
} // KundenVerwaltung::Update

unsigned int KundenVerwaltung::Search(const Kunde& kunde, GenericList<Kunde>* p_search_list)
{
	unsigned int no_matches = 0;

	_search_kunde = kunde;
   _kunde_found_index = -1;

	for(int i = 0; i < _kunden_liste.GetNoElements(); i++)
   {
		if(_kunden_liste[i].DoesMatch(kunde))
		{
      	if(p_search_list) p_search_list->Add(_kunden_liste[i]);
			if(_kunde_found_index == -1) _kunde_found_index = i;
         no_matches++;
      } // if
   } // for

   return no_matches;
} // KundenVerwaltung::Search

bool KundenVerwaltung::GetNextMatch(Kunde& foundKunde)
{
	bool found = false;

	if(_kunde_found_index != -1)
   {
      for(int i = _kunde_found_index; i < _kunden_liste.GetNoElements(); i++)
      {
         if(_kunden_liste[i].DoesMatch(_search_kunde))
         {
         	_kunde_found_index = i;
            found = true;
            break;
         } // if
      } // for

		if(found == true)
      {
      	foundKunde = _kunden_liste[_kunde_found_index++];
         return true;
      } // if
   } // if

   _kunde_found_index = -1;
   return false;
} // KundenVerwaltung::GetNextMatch

void KundenVerwaltung::ReadPascalData()
{
  char 		  filename[MAXDIR], text[300];
  const char* pC;
  int 		  string_size;
  bool		  match;

  ofstream kill_stream("daten/kunden.dat", ios::out|ios::binary);
  kill_stream.close();

  for(char i = 'A'; i < 'Z'; i++)
  {
    sprintf(filename, "daten_alt/%c.dat", i);
    ifstream stream(filename, ios::in|ios::binary);

    while(!stream.eof())
    {
	   Kunde tmp_kunde;
      string my_str;

      stream.read(text, 31); string_size = (int)(text[0]); text[string_size+1] = 0;
      my_str.assign(text+1); my_str.oem_to_ansi(); tmp_kunde.SetAnrede(my_str.c_str());
      stream.read(text, 31); string_size = (int)(text[0]); text[string_size+1] = 0;
      my_str.assign(text+1); my_str.oem_to_ansi(); tmp_kunde.SetVorname(my_str.c_str());
      stream.read(text, 31); string_size = (int)(text[0]); text[string_size+1] = 0;
      my_str.assign(text+1); my_str.oem_to_ansi(); tmp_kunde.SetNachname(my_str.c_str());
      stream.read(text, 31); string_size = (int)(text[0]); text[string_size+1] = 0;
      my_str.assign(text+1); my_str.oem_to_ansi(); tmp_kunde.SetStrasse(my_str.c_str());
      stream.read(text, 31); string_size = (int)(text[0]); text[string_size+1] = 0;
      my_str.assign(text+1); my_str.oem_to_ansi();
      tmp_kunde.SetPLZ(atoi(my_str.c_str()));
		for(pC = my_str.c_str(); *pC != 0; pC++)
		{
      	if(*pC == ' ')
         {
				tmp_kunde.SetOrt(pC+1);
            break;
         } // if
      } // for
      if(*pC == 0) tmp_kunde.SetOrt(my_str.c_str());
      stream.read(text, 21); string_size = (int)(text[0]); text[string_size+1] = 0;
      my_str.assign(text+1); my_str.oem_to_ansi(); tmp_kunde.SetTelefon(my_str.c_str());
      stream.read(text, 6); string_size = (int)(text[0]); text[string_size+1] = 0;

		// MessageBox(NULL, text, "", MB_OK);

      // Hier findet die Zuordnung der Klartextangaben der Preislisten statt

		match = false;

      if(strpbrk(text, "S") != NULL)
		{
      	// der Kunde hat NO/S

			tmp_kunde.SetPreisliste(1);
         match = true;
      } // if

      if(strpbrk(text, "W") != NULL)
		{
      	// der Kunde hat W

			tmp_kunde.SetPreisliste(1);
         match = true;
      } // if

      if(strpbrk(text, "A") != NULL)
		{
      	// der Kunde hat MA

			tmp_kunde.SetPreisliste(3);
         match = true;
      } // if

      if(!match)
		{
         // der Kunde hat NO, da sonst nichst gepasst hat

      	tmp_kunde.SetPreisliste(0);
      } // if

      Add(tmp_kunde);
    } // while
  } // for
} // KundenVerwaltung::ReadPascalData

void KundenVerwaltung::GetNextString(char* p_dest, char*& p_source)
{
	while((*p_source != ';') && (*p_source != 0)) {
   	*p_dest = *p_source;
	   p_source++;
      p_dest++;
   } // while

   if(*p_source == ';') p_source++;
   *p_dest = 0;
} // KundenVerwaltung::GetNextString

void KundenVerwaltung::DevideFirstLastName(char* tmp_vorname, char* tmp_nachname, char* p_work_text)
{
	int i;

	for(i = strlen(p_work_text)-1; i > 0; i--) {
   	if(p_work_text[i] == ' ') break;
   } // for

   if(i < 2) {
   	strcpy(tmp_nachname, p_work_text);
      tmp_vorname[0] = 0;
      return;
   } // if

   p_work_text[i] = 0;
   strcpy(tmp_nachname, &(p_work_text[i+1]));
   strcpy(tmp_vorname, p_work_text);
} // KundenVerwaltung::DevideFirstLastName

bool KundenVerwaltung::AtInString(char* p_string)
{
	for(int i = 0; i < strlen(p_string); i++) {
   	if(p_string[i] == '@') return true;
   } // for

   return false;
} // KundenVerwaltung::AtInString

void KundenVerwaltung::ReadCSVData()
{
	const int MAX_TEXT_LENGTH = 4096;
	char      text[MAX_TEXT_LENGTH], tmp_string[MAX_TEXT_LENGTH], *p_work_text;
   char      tmp_vorname[MAX_TEXT_LENGTH], tmp_nachname[MAX_TEXT_LENGTH];
   Kunde     tmp_kunde;

	ifstream stream("kunden.csv", ios::in);

   while(!stream.eof()) {
   	stream.getline(text, MAX_TEXT_LENGTH);

      p_work_text = text;
      tmp_kunde.Init();

      GetNextString(tmp_string, p_work_text);

      DevideFirstLastName(tmp_vorname, tmp_nachname, tmp_string);
      tmp_kunde.SetVorname(tmp_vorname);
      tmp_kunde.SetNachname(tmp_nachname);

      GetNextString(tmp_string, p_work_text);
      tmp_kunde.SetStrasse(tmp_string);

      GetNextString(tmp_string, p_work_text);
      for(int i = 0; i < strlen(tmp_string); i++) {
      	if((tmp_string[i] >= '0') && (tmp_string[i] <= '9')) break;
         tmp_string[i] = '0';
      } // if

      tmp_kunde.SetPLZ(atoi(tmp_string));

      GetNextString(tmp_string, p_work_text);
      tmp_kunde.SetOrt(tmp_string);

      GetNextString(tmp_string, p_work_text);
      tmp_kunde.SetTelefon(tmp_string);

      GetNextString(tmp_string, p_work_text);
      tmp_kunde.SetFax(tmp_string);

      GetNextString(tmp_string, p_work_text);
      if(AtInString(tmp_string)) tmp_kunde.SetEMail(tmp_string);
      else tmp_kunde.SetBemerkung(tmp_string);

      Add(tmp_kunde);
   } // while

} // KundenVerwaltung::ReadCSVData

void KundenVerwaltung::ExportAsCSV(fstream& stream, bool use_geo, bool use_plz, long geo_gruppe, long plz_min, long plz_max)
{
	stream << "Anrede,Vorname,Nachname,Strasse,PLZ,Ort,Telefon,Fax,EMail" << endl;

   for(long i = 0; i < _kunden_liste.GetNoElements(); i++)
	{
		if((!use_plz) || ((_kunden_liste[i].GetPLZ() >= plz_min) && (_kunden_liste[i].GetPLZ() <= plz_max)))
      {
      	if((!use_geo) || (_kunden_liste[i].GetGeoGruppe() == geo_gruppe))
         {
            stream << _kunden_liste[i].GetAnrede() << ",";
            stream << _kunden_liste[i].GetVorname() << ",";
            stream << _kunden_liste[i].GetNachname() << ",";
            stream << _kunden_liste[i].GetStrasse() << ",";
            stream << _kunden_liste[i].GetPLZ() << ",";
            stream << _kunden_liste[i].GetOrt() << ",";
            stream << _kunden_liste[i].GetTelefon() << ",";
            stream << _kunden_liste[i].GetFax() << ",";
            stream << _kunden_liste[i].GetEMail() << endl;
         } // if
      } // if
   } // for
} // KundenVerwaltung::ExportAsCSV

unsigned long KundenVerwaltung::GetNextKundenNr()
{
   return ++_last_kunden_nr;
} // KundenVerwaltung::GetNextKundenNr

void KundenVerwaltung::FindOrt(const long PLZ, char* pOrt)
{
	for(long i = 0; i < _kunden_liste.GetNoElements(); i++)
   {
   	if(_kunden_liste[i].GetPLZ() == PLZ)
      {
      	strcpy(pOrt, _kunden_liste[i].GetOrt());
         return;
      } // if
   } // for
} // KundenVerwaltung::FindOrt
