#include <pbk/preis/preisverwaltung.h>
#include <pbk/wein/weinlistenverwaltung.h>
#include <pbk/wein/wein.h>

#include <stdlib.h>
#include <stdio.h>

PreisVerwaltung::PreisVerwaltung(const char* filename, const char* listen_directory) : _preis_listen_array()
{
	strcpy(_filename, filename);
	strcpy(_listen_directory, listen_directory);

	_current_preis_listen_index = -1;
	_no_preislisten = 0;
   _p_log_buch = NULL;

   Load();
} // PreisVerwaltung::PreisVerwaltung

PreisVerwaltung::~PreisVerwaltung()
{
	for(int i = 0; i < _no_preislisten; i++)
   	delete _preis_listen_array[i];
} // PreisVerwaltung::PreisVerwaltung

void PreisVerwaltung::SetPreisliste(const short listen_index)
{
	if((listen_index < 0) || (listen_index >= _no_preislisten))
   	throw(xmsg(string("Der Funktion PreisVerwaltung::SetPreisliste() wurde ein unzulssiger Index bergeben")));

	_current_preis_listen_index = listen_index;
} // PreisVerwaltung::SetPreisliste

long PreisVerwaltung::GetPreisIndex(const long wein_nr)
{
	Preisliste* pPreis_liste;

	if(_current_preis_listen_index > -1)
   {
   	pPreis_liste = _preis_listen_array[_current_preis_listen_index];

   	for(long i = 0; i < pPreis_liste->_preis_liste.GetNoElements(); i++)
			if(pPreis_liste->_preis_liste[i].GetWeinNr() == wein_nr) return i;
   } // if

   return -1;
} // PreisVerwaltung::GetPreisIndex

void PreisVerwaltung::Add(const Preis& preis)
{
	if(_current_preis_listen_index > -1)
   {
   	Preisliste& preis_liste = *_preis_listen_array[_current_preis_listen_index];

		if(GetPreisIndex(preis.GetWeinNr()) < 0)
      	preis_liste._preis_liste.Add(preis);
   } // if
} // PreisVerwaltung::Add

void PreisVerwaltung::AddPreisliste(const char* pPreisliste)
{
   fstream stream(_filename, ios::in|ios::out);
	char	  filename[MAXDIR];

   size_t  size;
   stream.seekg(0, ios::end);
   size = stream.tellg();

   if(size != 0) stream << endl; // Return, falls schon eine Zeile existiert
   stream << pPreisliste;

   sprintf(filename, "%s/%d.wl", _listen_directory, _no_preislisten);
   _preis_listen_array[_no_preislisten] = new Preisliste(_no_preislisten, pPreisliste, filename);
   _no_preislisten++;
} // PreisVerwaltung::AddPreisliste

void PreisVerwaltung::Remove(const long wein_nr)
{
	long index;

	if(_current_preis_listen_index > -1)
   {
   	Preisliste& preis_liste = *_preis_listen_array[_current_preis_listen_index];

      index = GetPreisIndex(wein_nr);

      if(index > -1)
			preis_liste._preis_liste.Remove(index);
   } // if
} // PreisVerwaltung::Remove

Preis* PreisVerwaltung::operator[](const long wein_nr)
{
	long index;

	if(_current_preis_listen_index > -1)
   {
   	Preisliste& preis_liste = *_preis_listen_array[_current_preis_listen_index];

      index = GetPreisIndex(wein_nr);

      if(index > -1)
         return &(preis_liste._preis_liste[index]);
   } // if

   return NULL;
} // PreisVerwaltung::operator[]

const char* PreisVerwaltung::GetPreisListeName(const short index)
{
  	if((index < _no_preislisten) && (index >= 0))
		return _preis_listen_array[index]->_name;

   return NULL;
} // PreisVerwaltung::GetPreisListeName

void PreisVerwaltung::Load()
{
	char		text[256], filename[MAXDIR];
	ifstream stream(_filename, ios::in);

   // Einlesen der ASCII-Datei mit den Bezeichnungen fr die Preislisten

   if(!stream.bad())
   {
		_no_preislisten = 0;

		while(!stream.eof())
      {
	      if(_no_preislisten >= PV_NO_PREISLISTEN) throw(xmsg(string("Mehr als 20 Preislisten drfen nicht erzeugt werden")));

         stream.getline(text, 255);
	      if(stream.gcount() == 0) break;
		   sprintf(filename, "%s/%d.wl", _listen_directory, _no_preislisten);
         _preis_listen_array[_no_preislisten] = new Preisliste(_no_preislisten, text, filename);
			_no_preislisten++;
      } // while
   } // if
} // PreisVerwaltung::Load

void PreisVerwaltung::SaveCurrentListe()
{
	if(_current_preis_listen_index > -1)
   	_preis_listen_array[_current_preis_listen_index]->_preis_liste.Flush();
} // PreisVerwaltung::SaveCurrentListe

void PreisVerwaltung::ReadPascalFile(ModuleBroker* pModuleBroker)
{
   char   		  text[300], debug_text[500];
   unsigned char string_size;
   float  	  	  f_preis;

	for(int i_liste = 0; i_liste < _no_preislisten; i_liste++)
   {
		SetPreisliste(i_liste);

		sprintf(text, "daten_alt/wl-%d", i_liste+1);
      ifstream stream(text, ios::in|ios::binary);

      if(!stream.bad())
      {
         for(long i = 0; i < 999; i++)
         {
            stream.read((char*)text, 11);

            if(stream.gcount() != 11)
            	MessageBox(NULL, "PreisVerwaltung::ReadPascalFile", "Es wurden != 11 Zeichen gelesen", MB_OK);

            text[(int)text[0]+1] = 0;

            f_preis = atof(&(text[0])+1);

            if(f_preis > 0.0)
            {
//	               sprintf(debug_text, "Wein Nr.: %d - Preis: %f", i, f_preis);
//	               MessageBox(NULL, debug_text, "", MB_OK);

               Preis preis(i);

               preis.SetDMPreis(f_preis);
               preis.SetEuroPreis(f_preis*0.5);

               Add(preis);
            } // if
         } // for
      } // if

      SaveCurrentListe();
   } // for
} // PreisVerwaltung::ReadPascalFile

void PreisVerwaltung::ExportPreislisten(const char* filename, ModuleBroker* pModuleBroker)
{
	ofstream stream(filename, ios::out);
   char     text[300];

	if(!stream.bad())
   {
      for(int i_liste = 0; i_liste < _no_preislisten; i_liste++)
      {
         SetPreisliste(i_liste);

         GenericSortableList<Wein>& wein_liste = pModuleBroker->GetWeinlistenVerwaltungPtr()->GetWeinListe();

         for(long i = 0; i < wein_liste.GetNoElements(); i++)
         {
         	strcpy(text, "                                                                                         ");
            sprintf(text, "%d                                                                      ", wein_liste[i].GetNr());
            sprintf(&(text[7]), "%s                                                                      ", wein_liste[i].GetName());
            sprintf(&(text[45]), "%s                                                                      ", wein_liste[i].GetFuellmenge());
            sprintf(&(text[55]), "%0.2f                                                                      ", 7.25);
				stream << text << endl;
            stream << "              " << wein_liste[i].GetBezeichnung() << endl << endl;
         } // for
      } // for
   } // if
} // PreisVerwaltung::ExportPreislisten
