#ifndef _PreisVerwaltung_h
#define _PreisVerwaltung_h

#include <list/list.h>
#include <string.h>
#include <fstream.h>
#include <dir.h>
#include <except.h>
#include <cstring.h>

#include <pbk/preis/preis.h>
#include <pbk/logbuch/logbuch.h>
#include <pbk/modulebroker/modulebroker.h>

#define PV_NO_PREISLISTEN 20

class PreisVerwaltung
{
	protected:
   	class Preisliste
      {
			public:
            short 				 _nr;
            char  				 _name[30];
            GenericList<Preis> _preis_liste;

			public:
            Preisliste() : _preis_liste()
            {
            };

            Preisliste(const short nr, const char* name, const char* filename) : _preis_liste(filename)
            {
               _nr = nr;
               strncpy(_name, name, 29);
            };

            Preisliste& operator=(const Preisliste& liste)
            {
            	_nr = liste._nr;
               strcpy(_name, liste._name);
					_preis_liste = liste._preis_liste;
               return *this;
            };

            void Read(fstream& stream) {stream.read((char*)_nr, sizeof(_nr));stream.read((char*)_name, sizeof(_name));};
            void Write(fstream& stream) const {stream.write((char*)_nr, sizeof(_nr));stream.write((char*)_name, sizeof(_name));};

            static size_t SizeOf() {return sizeof(short)+30;};
      }; // struct Preisliste

      LogBuch*				      _p_log_buch;
		char						   _filename[MAXFILE], _listen_directory[MAXFILE];
      Preisliste*				   _preis_listen_array[PV_NO_PREISLISTEN];
      long							_current_preis_listen_index, _no_preislisten;

	public:
   	PreisVerwaltung(const char*, const char*);
   	~PreisVerwaltung();

		Preis* operator[](const long);

		void 	 SetLogBuchPtr(LogBuch* pLogBuch) {_p_log_buch = pLogBuch;};
		void   Remove(const long);         // Lschen von der akt. Preisliste den Preis zur WeinNr.
		void   Add(const Preis&);			  // Fgt der akt. Preisliste ggfs. einen neuen Preis hinzu
		void   AddPreisliste(const char*); // Fgt eine weitere Preisliste hinzu
		long   GetPreisIndex(const long);  // Ermittelt die Pos. des Preises zu einer WeinNr. i.d. akt. Preisliste
		void   SetPreisliste(const short); // Setzt eine der Preislisten in _preis_listen_array

      unsigned long GetNoPreisliste() {return _no_preislisten;};
      const char*   GetPreisListeName(const short index);
      void			  SaveCurrentListe();    // aktuelle Preisliste speichern
		void 			  ReadPascalFile(ModuleBroker*);
		void 			  ExportPreislisten(const char*, ModuleBroker*);

   protected:
		void Load();
}; // class PreisVerwaltung
#endif
