/*

- Nach dem ndern oder Lschen von Posten mssen die Bestnde aktualisiert werden

*/

#include "pbk/rechnung/postenverwaltung.h"
#include "pbk/rechnung/rechnungsverwaltung.h"
#include <limits.h>

void PostenVerwaltung::Posten::Read(fstream& stream)
{
	stream.read((char*)&_wein_nr, sizeof(_wein_nr));
	stream.read((char*)&_rechnung_nr, sizeof(_rechnung_nr));
	stream.read((char*)&_dm_preis, sizeof(_dm_preis));
	stream.read((char*)&_euro_preis, sizeof(_euro_preis));
	stream.read((char*)&_anzahl, sizeof(_anzahl));
	stream.read((char*)&_prev_posten, sizeof(_prev_posten));
} // PostenVerwaltung::Posten::Read

void PostenVerwaltung::Posten::Write(fstream& stream) const
{
	stream.write((char*)&_wein_nr, sizeof(_wein_nr));
	stream.write((char*)&_rechnung_nr, sizeof(_rechnung_nr));
	stream.write((char*)&_dm_preis, sizeof(_dm_preis));
	stream.write((char*)&_euro_preis, sizeof(_euro_preis));
	stream.write((char*)&_anzahl, sizeof(_anzahl));
	stream.write((char*)&_prev_posten, sizeof(_prev_posten));
} // PostenVerwaltung::Posten::Write

PostenVerwaltung::PostenVerwaltung(const char* filename)
{
	_no_posten = 0;
   _p_weinlisten_verwaltung = NULL;
   _p_log_buch = NULL;
   strcpy(_filename, filename);

   fstream stream(filename, ios::in|ios::binary);

   if(stream.bad()) return;

   stream.seekp(0, ios::end);
   static int _i = stream.tellg();
   _no_posten = stream.tellg() / Posten::SizeOf();
} // PostenVerwaltung::PostenVerwaltung

PostenVerwaltung::~PostenVerwaltung()
{
//	_posten_cache.Empty();
} // PostenVerwaltung::~PostenVerwaltung

void PostenVerwaltung::SetWeinlistenVerwaltungPtr(WeinlistenVerwaltung* pWeinVerwaltung)
{
	_p_weinlisten_verwaltung = pWeinVerwaltung;
} // PostenVerwaltung::SetWeinlistenVerwaltungPtr

void PostenVerwaltung::SetRechnungsVerwaltungPtr(RechnungsVerwaltung* pRechnungsVerwaltung)
{
	_p_rechnungs_verwaltung = pRechnungsVerwaltung;
} // PostenVerwaltung::SetRechnungsVerwaltungPtr

long PostenVerwaltung::Add(Rechnung* pRechnung, unsigned long wein_nr, const float DMPreis, const float EuroPreis, const short anzahl, const unsigned long rechnung_nr)
{
   return Add(pRechnung, Posten(rechnung_nr, wein_nr, DMPreis, EuroPreis, anzahl));
} // PostenVerwaltung::Add

long PostenVerwaltung::Add(Rechnung* pRechnung, Posten& posten)
{
	Rechnung* pLocalRechnung;

	fstream stream(_filename, ios::out|ios::app|ios::binary);

	posten._prev_posten = pRechnung->GetPrevPostenIndex();
	stream.seekp(0, ios::end);
   posten.Write(stream);
	_no_posten++;

   // Updaten des Flaschenbestandes

   _p_weinlisten_verwaltung->AddBestand(posten._wein_nr, (long)(-posten._anzahl));

   return _no_posten-1;
} // PostenVerwaltung::Add

void PostenVerwaltung::Update(Posten& posten, long last_posten) throw(xmsg)
{
	fstream stream(_filename, ios::in|ios::out|ios::binary);
   Posten  tmp_posten;
   long    i = last_posten;

   stream.width(1);

   while(i >= 0)
   {
	   stream.seekp(Posten::SizeOf()*i, ios::beg);
      tmp_posten.Read(stream);

      if(tmp_posten._rechnung_nr == posten._rechnung_nr)
		{
			if(tmp_posten._wein_nr == posten._wein_nr)
         {
				if(posten._anzahl == 0)
            {
               // der Posten soll vor dem Lschen noch in die Log-Datei eingetragen werden

               if(_p_log_buch)
               {
                  fstream& stream = _p_log_buch->GetOutputStream();

                  stream << ">> POSTEN GELSCHT << -> ";
                  stream << tmp_posten._wein_nr << "," << tmp_posten._rechnung_nr << "," << tmp_posten._dm_preis << "," <<
                  tmp_posten._euro_preis << "," << tmp_posten._anzahl << endl;
               } // if

					// Dieser Posten soll eigentlich gelscht werden. Daher wird die _rechnung_nr neutral
               // gesetzt, damit der Posten nicht mehr auffindbar ist.

               posten._rechnung_nr = ULONG_MAX;
            } // if
				else
            {
               // der Posten soll vor der nderung noch in die Log-Datei eingetragen werden

               if(_p_log_buch)
               {
                  fstream& stream = _p_log_buch->GetOutputStream();

                  stream << ">> POSTEN VERNDERT << -> ";
                  stream << tmp_posten._wein_nr << "," << tmp_posten._rechnung_nr << "," << tmp_posten._dm_preis << "," <<
                  tmp_posten._euro_preis << "," << tmp_posten._anzahl << endl;
               } // if

            } // else

			   stream.seekp(Posten::SizeOf()*i, ios::beg);

            // den vorherigen Posten mssen wir uns in jedem Fall merken
				posten._prev_posten = tmp_posten._prev_posten;

	         posten.Write(stream);

				// Der Alte Bestand des Weines mu wiederhergestellt werden
            // und auch der neue Bestand bercksichtigt

            _p_weinlisten_verwaltung->AddBestand(posten._wein_nr, (long)(tmp_posten._anzahl-posten._anzahl));
	         return;
         } // if
      } // if

      i = tmp_posten._prev_posten;
   } // while

   throw(xmsg(string("Es wurde in PostenVerwaltung::Update() ein Posten upgedated, den es noch gar nicht gibt")));
} // PostenVerwaltung::Update

bool PostenVerwaltung::GetPosten(Posten& posten, long& index, unsigned long rechnung_nr) throw(xmsg)
{
	if(index < 0) throw(string("Es wurde in PostenVerwaltung::GetPosten() ein index < 0 angegeben"));

	fstream stream(_filename, ios::in|ios::binary);
   stream.width(1);

	do
   {
   	stream.seekg(Posten::SizeOf()*index, ios::beg);
	   posten.Read(stream);
		index = posten._prev_posten;

      if(posten._rechnung_nr == rechnung_nr)
         return true;
   } while(index >= 0);

   return false;
} // PostenVerwaltung::GetPosten
