#ifndef _Rechnung_h
#define _Rechnung_h

#include <pbk/rechnung/postenverwaltung.h>
#include <pbk/modulebroker/modulebroker.h>

#include <classlib/date.h>
#include <classlib/time.h>
#include <fstream.h>
#include <limits.h>
#include <list/list.h>

#define RE_BEMERKUNG_SIZE 256

class Rechnung
{
	private:
      ModuleBroker*   						  	  _p_module_broker;
		bool											  _posten_verwaltung_valid;
      GenericList<PostenVerwaltung::Posten> _posten_cache;
      bool											  _posten_cache_valid;

	protected:
		unsigned long _nr;
      unsigned long _nr_extern;
   	unsigned long _kunde_nr;
		short			  _kisten, _kartons;
		TDate			  _datum;               // Datum der Rechnung - kann nach der Erfassung sein
      float			  _betrag_eingegangen;
      bool			  _beglichen;           // auch Rechnungen m. kleinen Fehlbetrgen knnen beglichen sein
      TDate			  _erfasst_datum;
      TTime			  _erfasst_zeit;
      char			  _bemerkung[RE_BEMERKUNG_SIZE];
		long			  _last_posten; 		   // erste Posten von hinten in der Posten-Datei
      long			  _prev_kunde_rechnung; // Index der letzten Rechnung unseres Kunden

   public:
   	Rechnung(ModuleBroker* = NULL, unsigned long = LONG_MAX, unsigned long = LONG_MAX, unsigned long = LONG_MAX);

      unsigned long GetNr() const {return _nr;};
      unsigned long GetNrExtern() const {return _nr_extern;};
      unsigned long GetKundeNr() const {return _kunde_nr;};
      short 		  GetKisten() const {return _kisten;};
      short 		  GetKartons() const {return _kartons;};
      TDate			  GetDatum() const {return _datum;};
      float         GetBetragEingegangen() const {return _betrag_eingegangen;};
      bool          GetBeglichen() const {return _beglichen;};
      const char*   GetBemerkung() const {return _bemerkung;};
      const TDate&  GetErfassungDatum() const {return _erfasst_datum;};
      const TTime&  GetErfassungZeit() const {return _erfasst_zeit;};
      const long	  GetPrevPostenIndex() const {return _last_posten;};
      const long	  GetPrevKundeRechnungIndex() const {return _prev_kunde_rechnung;};

      float			  GetBetragGesamt(bool = true);
		long 			  GetGewicht();
      int			  GetNoPosten();

      void SetNr(unsigned long nr) {_nr = nr;};
      void SetNrExtern(unsigned long nr_extern) {_nr_extern = nr_extern;};
      void SetKundeNr(unsigned long kunde_nr) {_kunde_nr = kunde_nr;};
      void SetKisten(short kisten) {_kisten = kisten;};
      void SetKartons(short kartons) {_kartons = kartons;};
      void SetDatum(TDate& datum) {_datum = datum;};
// debug
      void SetErfassungDatum(TDate& datum) {_erfasst_datum = datum;};
      void SetBetragEingegangen(float betrag_eingegangen) {_betrag_eingegangen = betrag_eingegangen;};
      void SetBeglichen(bool beglichen) {_beglichen = beglichen;};
      void SetBemerkung(const char* bemerkung) {strcpy(_bemerkung, bemerkung);};
      void SetModuleBrokerPtr(ModuleBroker* pModuleBroker) {_p_module_broker = pModuleBroker;};

      void 	  						  AddPosten(const unsigned long, const float, const float, short);
		void							  ClearPosten(const unsigned long);
		PostenVerwaltung::Posten* operator[](unsigned long);
      Rechnung&					  operator=(const Rechnung&);

		void ResetPostenCursor(long&);				            // Setzt den Zeiger auf den letzten Posten (umgekehrte Richtung)
		bool GetNextPosten(PostenVerwaltung::Posten&, long&); // Holt den nchsten Posten und setzt den Zeiger auf vorherigen Posten

      static size_t SizeOf() {return sizeof(unsigned long)+sizeof(unsigned long)+sizeof(unsigned long)+sizeof(short)+
      										 sizeof(short)+sizeof(TDate)+sizeof(float)+sizeof(bool)+sizeof(TDate)+sizeof(TTime)
                                     +RE_BEMERKUNG_SIZE+sizeof(long)+sizeof(long);};

      void Read(fstream&);
      void Write(fstream&) const;
}; // class Rechnung

class RechnungsServer
{
	public:
   	virtual void FillRechnungsListe(GenericList<Rechnung>&, bool) = 0;
}; // class RechnungsServer
#endif