#include <stdio.h>
#include <except.h>
#include <cstring.h>
#include <limits.h>
#include <fstream.h>
#include <math.h>

#include <BIFFOut/BIFFOut.h>

#include <pbk/rechnung/rechnungsverwaltung.h>
#include <pbk/rechnung/postenverwaltung.h>
#include <pbk/kunde/kundenverwaltung.h>
#include <pbk/rechnung/rechnung.h>
#include <pbk/logbuch/logbuch.h>
#include <pbk/customizer/customizer.h>

RechnungsVerwaltung::RechnungsVerwaltung(const char* pFilename)
{
	_new_rechnung_valid = false;
	_current_rechnung_pos = -1;
   _last_rechnung_nr = _last_rechnung_nr_extern = 0l;
   _p_new_rechnung = NULL;

   strcpy(_filename, pFilename);
} // RechnungsVerwaltung::RechnugsVerwaltung

RechnungsVerwaltung::~RechnungsVerwaltung()
{
   if(_p_current_rechnung) delete _p_current_rechnung; _p_current_rechnung = NULL;
   if(_p_new_rechnung) delete _p_new_rechnung; _p_new_rechnung = NULL;
} // RechnungsVerwaltung::~RechnungsVerwaltung

void RechnungsVerwaltung::SetModuleBrokerPtr(ModuleBroker* pModuleBroker)
{
	_p_module_broker = pModuleBroker;
   _p_current_rechnung = new Rechnung(_p_module_broker);

	// Letzte Rechungsnummer ermitteln

   fstream stream(_filename, ios::in|ios::binary);
   if(!stream.bad())
   {
		Rechnung tmpRechnung(_p_module_broker);
      stream.seekg(-Rechnung::SizeOf(), ios::end);
      tmpRechnung.Read(stream);
      _last_rechnung_nr = tmpRechnung.GetNr()+1;
      _last_rechnung_nr_extern = tmpRechnung.GetNrExtern()+1;
   } // if
   stream.close();
} // RechnungsVerwaltung::SetModuleBrokerPtr

Rechnung* RechnungsVerwaltung::AllocRechnung(unsigned long kunde_nr) throw(xmsg)
{
	if(_new_rechnung_valid) throw(xmsg(string("Es wurde eine Rechnung in RechnungsVerwaltung::AllocRechnung() allokiert, obwohl die letzte noch nicht gespeichert wrude")));

	_p_new_rechnung = new Rechnung(_p_module_broker, GetNextRechnungNr(), GetNextRechnungNrExtern(), kunde_nr);
   _new_rechnung_valid = true;
	return _p_new_rechnung;
} // AddRechnung::AllocRechnung

void RechnungsVerwaltung::StoreRechnung()
{
	if(!_new_rechnung_valid) throw(xmsg(string("Es wurde in RechnungsVerwaltung::StoreRechnung() versucht eine Rechnung zu speichern, obwohl noch gar keine reserviert wurde")));

	fstream stream(_filename, ios::out|ios::app|ios::binary);

   if(stream.bad()) throw(xmsg(string("In RechnungsVerwaltung::StoreRechnung() konnte die Datei nicht geffnet werden")));
   _p_new_rechnung->Write(stream);
	_new_rechnung_valid = false;
   delete _p_new_rechnung; _p_new_rechnung = NULL;
} // RechnungsVerwaltung::StoreRechnung

Rechnung* RechnungsVerwaltung::operator[](unsigned long pos)
{
	fstream stream(_filename, ios::in|ios::app|ios::binary);

   for(long i = _last_rechnung_nr-1; i >= 0; i--)
   {
      _current_rechnung_pos = i;
      stream.seekg(Rechnung::SizeOf()*i, ios::beg);
      _p_current_rechnung->Read(stream);
      if(_p_current_rechnung->GetNr() == pos) return _p_current_rechnung;

      // Wenn die Nummer der gelesenen Rechnung kleiner als die gesuchte wird, so mu nicht mehr
      // weiter gelesen werden, da (interne) Rechnungsnummern nur aufsteigend sein drfen

      if(_p_current_rechnung->GetNr() < pos) break;
   } // for

   return NULL;
} // RechnungsVerwaltung::operator[]

void RechnungsVerwaltung::UpdateCurrentRechnung()
{
	Rechnung tmp_rechnung;

	if(_current_rechnung_pos < 0) throw(xmsg(string("Es wurde in RechnungsVerwaltung::UpdateCurrentRechnung() versucht eine Rechnung zu erneuern, obwohl keine angefordert wurde")));

	fstream stream(_filename, ios::out|ios::in|ios::binary);
   stream.width(1);

   stream.seekp(Rechnung::SizeOf()*_current_rechnung_pos, ios::beg);
   tmp_rechnung.Read(stream);

	// Der Zustand vor der Vernderung soll festgehalten werden

   if(_p_module_broker->GetLogBuchPtr())
   {
      fstream& stream = _p_module_broker->GetLogBuchPtr()->GetOutputStream();
      TDate erf_dat = tmp_rechnung.GetErfassungDatum();
      TTime erf_zei = tmp_rechnung.GetErfassungZeit();

      stream << ">> RECHNUNG GENDERT << (erfasst am " << erf_dat.DayOfMonth() << "." << erf_dat.Month() << "." << erf_dat.Year() << " " << erf_zei.Hour() << ":" << erf_zei.Minute() << ":" << erf_zei.Second() << ") -> ";
      stream << tmp_rechnung.GetNr() << "," << tmp_rechnung.GetNrExtern() << "," << tmp_rechnung.GetKundeNr() << "," <<
      tmp_rechnung.GetKisten() << "," << tmp_rechnung.GetKartons() << ",\042" << tmp_rechnung.GetDatum() << "\042," << tmp_rechnung.GetBetragEingegangen() << "," <<
      tmp_rechnung.GetBeglichen() << ",\042" << tmp_rechnung.GetBemerkung() << "\042" << endl;
   } // if

   stream.seekp(Rechnung::SizeOf()*_current_rechnung_pos, ios::beg);
   _p_current_rechnung->Write(stream);
	_current_rechnung_pos = -1;
} // RechnungsVerwaltung::UpdateCurrentRechnung

void RechnungsVerwaltung::UpdateRechnung(Rechnung* pRechnung)
{
	// eine neu allokierte Rechnung mu nicht zwischengespeichert werden
   // dies geschieht am Ende ohnehin durch StoreRechnung()

	if(pRechnung == _p_new_rechnung) return;

	Rechnung* pLocalRechnung = (*this)[pRechnung->GetNr()];

   if(pLocalRechnung)
   {
   	*pLocalRechnung = *pRechnung;
      UpdateCurrentRechnung();
   } // if
} // RechnungsVerwaltung::UpdateRechnung

void RechnungsVerwaltung::DeleteRechnung(const unsigned long nr, bool delete_posten)
{
   PostenVerwaltung::Posten posten;
	Rechnung* 					 pLocalRechnung = (*this)[nr];
   long							 index;

   if(!pLocalRechnung) throw(xmsg(string("Die referenzierte Rechnung konnte in RechnungsVerwaltung::DeleteRechnung() nicht gefunden werden")));

	if(delete_posten)
   {
   	pLocalRechnung->ResetPostenCursor(index);

      while(pLocalRechnung->GetNextPosten(posten, index))
			pLocalRechnung->ClearPosten(posten._wein_nr);
   } // if

	// Kisten und Steigen == SHRT_MAX bedeutet, da diese Rechnung gelscht ist

   // Die Rechnung soll vor dem Lschen noch in die Log-Datei eingetragen werden

   if(_p_module_broker->GetLogBuchPtr())
   {
      fstream& stream = _p_module_broker->GetLogBuchPtr()->GetOutputStream();
      TDate erf_dat = pLocalRechnung->GetErfassungDatum();
      TTime erf_zei = pLocalRechnung->GetErfassungZeit();

      stream << ">> RECHNUNG GELSCHT << (erfasst am " << erf_dat.DayOfMonth() << "." << erf_dat.Month() << "." << erf_dat.Year() << " " << erf_zei.Hour() << ":" << erf_zei.Minute() << ":" << erf_zei.Second() << ") -> ";
      stream << pLocalRechnung->GetNr() << "," << pLocalRechnung->GetNrExtern() << "," << pLocalRechnung->GetKundeNr() << "," <<
      pLocalRechnung->GetKisten() << "," << pLocalRechnung->GetKartons() << ",\042" << pLocalRechnung->GetDatum() << "\042," << pLocalRechnung->GetBetragEingegangen() << "," <<
      pLocalRechnung->GetBeglichen() << ",\042" << pLocalRechnung->GetBemerkung() << "\042" << endl;
   } // if

   pLocalRechnung->SetKisten(SHRT_MAX);
   pLocalRechnung->SetKartons(SHRT_MAX);

   UpdateCurrentRechnung();
} // RechnungsVerwaltung::DeleteRechnung

void RechnungsVerwaltung::Duplicate(Rechnung* pRechnung)
{
	PostenVerwaltung::Posten posten;
   Rechnung* 					 pNewRechnung = AllocRechnung(pRechnung->GetKundeNr());
   long		 					 index;
   char							 text[300];
   TDate					       Heute;
   TTime							 Jetzt;

	sprintf(text, "Abgeleitet aus Rechnung Nr. %d am %d.%d.%d %d:%d:%d", pRechnung->GetNrExtern(), Heute.DayOfMonth(), Heute.Month(), Heute.Year(), Jetzt.Hour(), Jetzt.Minute(), Jetzt.Second());
   pNewRechnung->SetBemerkung(text);
   pNewRechnung->SetBetragEingegangen(0.0);
   pNewRechnung->SetBeglichen(false);
   pNewRechnung->SetKartons(pRechnung->GetKartons());
   pNewRechnung->SetKisten(pRechnung->GetKisten());
   pNewRechnung->SetDatum(pRechnung->GetDatum());

   pRechnung->ResetPostenCursor(index);
   while(pRechnung->GetNextPosten(posten, index))
   {
		pNewRechnung->AddPosten(posten._wein_nr, posten._dm_preis, posten._euro_preis, posten._anzahl);
   } // while
   StoreRechnung();
} // RechnungsVerwaltung::Duplicate

void RechnungsVerwaltung::GetListeByKunde(const unsigned long kunde_nr, GenericList<Rechnung>& liste, bool nur_offene)
{
	Rechnung tmpRechnung(_p_module_broker);
   long		last_rechnung_index;

	fstream stream(_filename, ios::in|ios::binary);
   stream.width(1);

   last_rechnung_index = (*_p_module_broker->GetKundenVerwaltungPtr())[kunde_nr]->GetLastRechnungIndex();

	while(last_rechnung_index >= 0)
   {
	   stream.seekg(Rechnung::SizeOf()*last_rechnung_index, ios::beg);
	   tmpRechnung.Read(stream);

      // Steigen == SHRT_MAX interessieren uns nicht, da diese als gelscht gelten

      if((tmpRechnung.GetKartons() != SHRT_MAX) && (tmpRechnung.GetKundeNr() == kunde_nr) && ((!nur_offene) || (!tmpRechnung.GetBeglichen())))
      	liste.Add(tmpRechnung);

      last_rechnung_index = tmpRechnung.GetPrevKundeRechnungIndex();
   } // while
} // RechnungsVerwaltung::GetListByKunde

void RechnungsVerwaltung::GetZeitraumliste(GenericList<Rechnung>& liste, const TDate erfassung_first,
														 const TDate erfassung_last, const TDate datierung_first, const TDate datierung_last,
                                           const bool use_erfassung, const bool use_datierung, bool nur_offene)
{
	Rechnung tmpRechnung(_p_module_broker);

	fstream stream(_filename, ios::in|ios::binary);
   stream.width(1);

	for(long i = _last_rechnung_nr-1; i >= 0; i--)
   {
	   stream.seekg(Rechnung::SizeOf()*i, ios::beg);
	   tmpRechnung.Read(stream);

      /* Wenn das frhe Datum des Intervalls im Verhltnis zur Erfassung
      	weiter zurckliegt, als die Datierungsgrenze erlaubt, so kann
         abgebrochen werden.															  */
/*
		char text[400];

      sprintf(text, "datierung_first: %d.%d.%d / tmpRechnung.ErfassungDatum: %d.%d.%d", datierung_first.DayOfMonth(), datierung_first.Month(), datierung_first.Year(),
      																											 tmpRechnung.GetErfassungDatum().DayOfMonth(), tmpRechnung.GetErfassungDatum().Month(), tmpRechnung.GetErfassungDatum().Year());

      MessageBox(NULL, text, "debug", MB_OK);
*/
      if((use_datierung) && ((int)(datierung_first - tmpRechnung.GetErfassungDatum()) > _p_module_broker->GetCustomizerPtr()->GetVordatierung()))
      	return;

      // Steigen == SHRT_MAX interessieren uns nicht, da diese als gelscht gelten

      if(tmpRechnung.GetKartons() != SHRT_MAX)
			if((!use_erfassung) || ((tmpRechnung.GetErfassungDatum() >= erfassung_first) && (tmpRechnung.GetErfassungDatum() <= erfassung_last)))
				if((!use_datierung) || ((tmpRechnung.GetDatum() >= datierung_first) && (tmpRechnung.GetDatum() <= datierung_last)))
					if((!nur_offene) || (!tmpRechnung.GetBeglichen()))
			      	liste.Add(tmpRechnung);
   } // for
} // RechnungsVerwaltung::GetZeitraumliste

void RechnungsVerwaltung::ErzeugeFlaschenliste(GenericList<Rechnung>& rechnungs_liste, GenericSortableList<FlaschenPosten>& flaschen_liste)
{
	PostenVerwaltung::Posten posten;
	long 							 index;

	for(long i = 0; i < rechnungs_liste.GetNoElements(); i++)
   {
		rechnungs_liste[i].ResetPostenCursor(index);
      while(rechnungs_liste[i].GetNextPosten(posten, index))
      {
	      flaschen_liste.Add(FlaschenPosten(posten._wein_nr, posten._anzahl));
      } // while
   } // for

   flaschen_liste.Sort();
} // RechnungsVerwaltung::ErzeugeFlaschenliste

void RechnungsVerwaltung::ExportFlaschenListe(const char* filename, FlaschenExportType mode)
{
   long	last_wein_nr, summe, zahlen_pro_zeile;
   bool  first_time, first_komma;
   char  text[300];
   long  current_line, current_row;

   GenericList<Rechnung> rechnungsliste;

   GetZeitraumliste(rechnungsliste, _p_module_broker->GetCustomizerPtr()->ErfassungFirst(), _p_module_broker->GetCustomizerPtr()->ErfassungLast(),
   					  _p_module_broker->GetCustomizerPtr()->DatierungFirst(), _p_module_broker->GetCustomizerPtr()->DatierungLast(),
                    _p_module_broker->GetCustomizerPtr()->UseErfassung(), _p_module_broker->GetCustomizerPtr()->UseDatierung(), false);

   GenericSortableList<FlaschenPosten> flaschen_liste;

   ErzeugeFlaschenliste(rechnungsliste, flaschen_liste);

   if(mode == WRITE_TXTLIST)
   {
      ofstream stream(filename, ios::out);

      stream << "Datei: " << filename << endl << endl;
      if(_p_module_broker->GetCustomizerPtr()->UseDatierung())
         stream << "Flaschenliste fr den Zeitraum vom " << _p_module_broker->GetCustomizerPtr()->DatierungFirst().DayOfMonth() << "." <<
         	_p_module_broker->GetCustomizerPtr()->DatierungFirst().Month() << "." <<
            _p_module_broker->GetCustomizerPtr()->DatierungFirst().Year() << " bis zum " <<
            _p_module_broker->GetCustomizerPtr()->DatierungLast().DayOfMonth() << "." <<
            _p_module_broker->GetCustomizerPtr()->DatierungLast().Month() << "." <<
            _p_module_broker->GetCustomizerPtr()->DatierungLast().Year() << endl << endl << endl;

      last_wein_nr = LONG_MAX;
      summe = 0;
      first_time = true;
      for(int i = 0; i < flaschen_liste.GetNoElements(); i++)
      {
         if(last_wein_nr != flaschen_liste[i]._wein_nr)
         {
            if(first_time)
            {
               first_time = false;
               summe = 0;
            } // if
            else
            {
               stream << endl << "Summe: " << summe << endl << endl;
               summe = 0;
            } // else

            last_wein_nr = flaschen_liste[i]._wein_nr;
            stream << "Wein Nummer (" << flaschen_liste[i]._wein_nr << "): ";
            first_komma = true;
            zahlen_pro_zeile = 5;
         } // if

         if(first_komma)
            first_komma = false;
         else stream << ", ";
         stream << flaschen_liste[i]._anzahl;
         summe += flaschen_liste[i]._anzahl;
         zahlen_pro_zeile++;
         if(zahlen_pro_zeile > 25)
         {
            zahlen_pro_zeile = 0;
            stream << "," << endl;
            first_komma = true;
         } // if
      } // for

      stream.close();
   } // if

	if(mode == WRITE_XLSLIST)
   {
   	BIFFOut Sheet(filename);

		Sheet.SetStandardColWidth(10);

      sprintf(text, "Liste verkaufter Flaschen");
      Sheet.WriteLABEL(0, 0, text);

      if(_p_module_broker->GetCustomizerPtr()->UseDatierung())
		{
         sprintf(text, "Datierung der Rechnungen von %d.%d.%d bis %d.%d.%d", _p_module_broker->GetCustomizerPtr()->DatierungFirst().DayOfMonth(),
               _p_module_broker->GetCustomizerPtr()->DatierungFirst().Month(),
               _p_module_broker->GetCustomizerPtr()->DatierungFirst().Year(),
               _p_module_broker->GetCustomizerPtr()->DatierungLast().DayOfMonth(),
               _p_module_broker->GetCustomizerPtr()->DatierungLast().Month(),
               _p_module_broker->GetCustomizerPtr()->DatierungLast().Year());
         Sheet.WriteLABEL(1, 0, text);
      } // if

      last_wein_nr = LONG_MAX;
      summe = 0; current_row = 0;
      for(int i = 0; i < flaschen_liste.GetNoElements(); i++)
      {
         if(last_wein_nr != flaschen_liste[i]._wein_nr)
         {
            summe = 0;

            last_wein_nr = flaschen_liste[i]._wein_nr;

				sprintf(text, "Wein %d", flaschen_liste[i]._wein_nr);
            current_line = 5;
            Sheet.WriteLABEL(3, current_row++, text);
         } // if

         Sheet.WriteNUMBER(current_line++, current_row, flaschen_liste[i]._anzahl);
         summe += flaschen_liste[i]._anzahl;
      } // for
   } // if

//      void SetStandardColWidth(unsigned short);
//      void WriteLABEL(unsigned short, unsigned short, char*);
//      void WriteLABELLeftAlign(unsigned short, unsigned short, char*);
//      void WriteNUMBER(unsigned short, unsigned short, double);

} // RechnungsVerwaltung::ExportFlaschenListe

void RechnungsVerwaltung::GetListeByNummerOrBetrag(const long nummer, const float betrag, const float abweichung, GenericList<Rechnung>& liste)
{
	long smallest_nr = 0;

   // Hier kann die Anzahl der durchsuchten Rechnungen eingetragen werden

//   if((nummer < 0) && (_last_rechnung_nr > 300)) smallest_nr = _last_rechnung_nr - 300;

	Rechnung tmpRechnung(_p_module_broker);

	fstream stream(_filename, ios::in|ios::binary);
   stream.width(1);

	for(long i = _last_rechnung_nr-1; i >= smallest_nr; i--)
   {
	   stream.seekg(Rechnung::SizeOf()*i);
	   tmpRechnung.Read(stream);

      // Steigen == SHRT_MAX interessieren uns nicht, da diese als gelscht gelten

      if(tmpRechnung.GetKartons() != SHRT_MAX)
			if(((nummer >= 0) && (tmpRechnung.GetNrExtern() == nummer)) ||
            ((betrag > 0.0) && ((float)fabs((double)(tmpRechnung.GetBetragGesamt(!(_p_module_broker->GetCustomizerPtr()->GetEuroAktiv())) - betrag)) <= abweichung)))
				{
		      	liste.Add(tmpRechnung);
            } // if
   } // for
} // RechnungsVerwaltung::GetListeByNummerOrBetrag
