#include <BIFFOut/BIFFOut.h>

// Elementfunktionen von BIFFOut

BIFFOut::BIFFOut(const char *pOutputFileName)
{
	pOutStream = new ofstream(pOutputFileName, ios::out|ios::binary);
	WriteBOF();
	WriteINDEX();
} // BIFFOut::BIFFOut

BIFFOut::~BIFFOut()
{
	WriteEOF();
	delete pOutStream; pOutStream = NULL;
} // BIFFOut::~BIFFOut

void BIFFOut::WriteBOF()
{
	unsigned short Rec[5];

	Rec[0] = 0x209;	// Recordtyp
	Rec[1] = 0x06;	// Lnge des Records
	Rec[2] = 0x07;	// Versionsnummer
	Rec[3] = 0x10;	// Dokumenttyp = Kalkulationsblatt (.XLS)
	Rec[4] = 0x00; // reserviert

	pOutStream->write((unsigned char*)Rec, 10);
} // BIFFOut::WriteBOF

void BIFFOut::WriteINDEX()
{
	unsigned short Rec[8];

	Rec[0] = 0x20B;	// Recordtyp INDEX
	Rec[1] = 0x0C;	   // Lnge des Records
	Rec[2] = 0x00;    // Offset 1. NAME-Record
	Rec[3] = 0x00;    // Highword Offset
	Rec[4] = 0x00;    // erste Reihe
	Rec[5] = 0x00;    // letzte Reihe
	Rec[6] = 0x00;	   // Offset XF-Record
	Rec[7] = 0x00;	   // Highword Offset

	pOutStream->write((unsigned char*)Rec, 16);
} // BIFFOut::WriteINDEX

void BIFFOut::SetStandardColWidth(unsigned short NoCharacters)
{
	unsigned short Rec[3];

	Rec[0] = 0x55;	 		  // Recordtyp DEFCOLWIDTH
	Rec[1] = 0x2;	 		  // Lnge des Records
	Rec[2] = NoCharacters; // Standardbreite Spalte

	pOutStream->write((unsigned char*)Rec, 6);
} // BIFFOut::SetStandardColWidth

void BIFFOut::WriteEOF()
{
	unsigned short Rec[2];

	Rec[0] = 0x0A;   // Recordtyp
	Rec[1] = 0x00;	// Lnge des Records

	pOutStream->write((unsigned char*)Rec, 4);
} // BIFFOut::WriteEOF

void BIFFOut::WriteLABEL(unsigned short line, unsigned short row, char *text)
{
	unsigned short Rec[4];
	unsigned char BRec[4];

	Rec[0] = 0x04;					// Recordtyp LABEL
	Rec[1] = 8+strlen(text);	// Lnge des Records
	Rec[2] = line;					// Zeilennummer
	Rec[3] = row;					// Spaltennummer

	pOutStream->write((unsigned char*)Rec, 8);

	BRec[0] = 0x00;				// Zellattribute
	BRec[1] = 0x00;
	BRec[2] = 0x03;				// Text rechtsbndig
	BRec[3] = (unsigned char)strlen(text);

	pOutStream->write((unsigned char*)BRec, 4);

	pOutStream->write((char*)text, strlen(text));
} // BIFFOut::WriteLABEL

void BIFFOut::WriteLABELLeftAlign(unsigned short line, unsigned short row, char *text)
{
	unsigned short Rec[4];
	unsigned char BRec[4];

	Rec[0] = 0x04;					// Recordtyp LABEL
	Rec[1] = 8+strlen(text);	// Lnge des Records
	Rec[2] = line;					// Zeilennummer
	Rec[3] = row;					// Spaltennummer

	pOutStream->write((unsigned char*)Rec, 8);

	BRec[0] = 0x00;				// Zellattribute
	BRec[1] = 0x00;
	BRec[2] = 0x01;				// Text linksbndig
	BRec[3] = (unsigned char)strlen(text);

	pOutStream->write((unsigned char*)BRec, 4);

	pOutStream->write((char*)text, strlen(text));
} // BIFFOut::WriteLABELLeftAlign

void BIFFOut::WriteNUMBER(unsigned short line, unsigned short row, double Number)
{
	unsigned short Rec[4];
	unsigned char	BRec[3];

	Rec[0] = 0x03;					// Recordtyp NUMBER
	Rec[1] = 0x0F;					// Lnge des Records
	Rec[2] = line;					// Zeilennummer
	Rec[3] = row;					// Spaltennummer
	pOutStream->write((unsigned char*)Rec, 8);

	BRec[0] = 0x00;				// Zellattribute
	BRec[1] = 0x00;
	BRec[2] = 0x00;
	pOutStream->write((unsigned char*)BRec, 3);

	pOutStream->write((unsigned char*)&Number, 8);
} // BIFFOut::WriteNUMBER
