#ifndef _VectorR2_h_
#define _VectorR2_h_

#include <except.h>
#include <cstring.h>
#include <math.h>

#include "global/global.h"
#include "list/list.h"

class VectorR2
{
	protected:
   	double value[2];

   public:
   	VectorR2() {};
   	VectorR2(const double& x, const double& y) {value[0] = x; value[1] = y;};

		inline VectorR2 operator- (const VectorR2&);
		inline VectorR2 operator+ (const VectorR2&);
		inline VectorR2& operator+= (const VectorR2&);
		inline VectorR2  operator* (double) const;

      inline double& operator[](int) throw(xmsg);
      inline void    SetUnitLength();
		inline double  GetLength();
}; // class VectorR2

typedef GenericList<VectorR2> VectorR2List;

double& VectorR2::operator[](int i) throw(xmsg)
{
   if((i >= 0) && (i <= 1))
      return value[i];
   else throw(xmsg(string("Operator[] allows only index 0 or 1 in VektorR2")));
}; // VectorR2::operator[]

void VectorR2::SetUnitLength()
{
	double length = GetLength();

   value[0] /= length;
   value[1] /= length;
}; // VectorR2::SetUnitLength

VectorR2& VectorR2::operator+= (const VectorR2& v)
{
	value[0] += v[0];
	value[1] += v[1];

   return *this;
} // VectorR2::operator+=

VectorR2 VectorR2::operator* (double mult) const
{
	VectorR2 tmpVec = *this;

   tmpVec[0] *= mult;
   tmpVec[1] *= mult;

   return tmpVec;
} // VectorR2::operator*

VectorR2 VectorR2::operator- (const VectorR2& v1)
{
	VectorR2 tmpVector;

   tmpVector[0] = value[0] - v1[0];
   tmpVector[1] = value[1] - v1[1];

   return tmpVector;
} // VectorR2::operator-

VectorR2 VectorR2::operator+ (const VectorR2& v1)
{
	VectorR2 tmpVector;

   tmpVector[0] = value[0] + v1[0];
   tmpVector[1] = value[1] + v1[1];

   return tmpVector;
} // VectorR2::operator+

double VectorR2::GetLength()
{
	return sqrt(sqr(value[0]) + sqr(value[1]));
} // VectorR2::GetLength
#endif
